using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace CRMWebParts.Common.Security
{
	public class SimpleAES : IEncryptionProvider
	{
		private byte[] _encryptionKey;

		public SimpleAES()
		{
		}

		public string Decrypt(string encrypted)
		{
			if (string.IsNullOrEmpty(encrypted))
			{
				throw new ArgumentNullException(encrypted);
			}
			byte[] numArray = Convert.FromBase64String(encrypted);
			if ((int)numArray.Length < 17)
			{
				throw new ArgumentException("Not a valid encrypted string", "encrypted");
			}
			return this.DecryptString(numArray, this._encryptionKey);
		}

		public string DecryptString(byte[] encryptedString, byte[] encryptionKey)
		{
			string end;
			using (AesCryptoServiceProvider aesCryptoServiceProvider = new AesCryptoServiceProvider())
			{
				aesCryptoServiceProvider.Key = encryptionKey;
				using (MemoryStream memoryStream = new MemoryStream(encryptedString))
				{
					byte[] numArray = new byte[16];
					memoryStream.Read(numArray, 0, 16);
					aesCryptoServiceProvider.IV = numArray;
					using (ICryptoTransform cryptoTransform = aesCryptoServiceProvider.CreateDecryptor())
					{
						using (CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoTransform, CryptoStreamMode.Read))
						{
							using (StreamReader streamReader = new StreamReader(cryptoStream))
							{
								end = streamReader.ReadToEnd();
							}
						}
					}
				}
			}
			return end;
		}

		public string Encrypt(string unencrypted)
		{
			if (string.IsNullOrEmpty(unencrypted))
			{
				throw new ArgumentNullException(unencrypted);
			}
			string base64String = Convert.ToBase64String(this.EncryptString(unencrypted, this._encryptionKey));
			return base64String;
		}

		public byte[] EncryptString(string toEncrypt, byte[] encryptionKey)
		{
			byte[] array;
			if (string.IsNullOrEmpty(toEncrypt))
			{
				throw new ArgumentException("toEncrypt");
			}
			if ((encryptionKey == null ? true : encryptionKey.Length == 0))
			{
				throw new ArgumentException("encryptionKey");
			}
			byte[] bytes = Encoding.UTF8.GetBytes(toEncrypt);
			using (AesCryptoServiceProvider aesCryptoServiceProvider = new AesCryptoServiceProvider())
			{
				aesCryptoServiceProvider.Key = encryptionKey;
				aesCryptoServiceProvider.Mode = CipherMode.CBC;
				aesCryptoServiceProvider.Padding = PaddingMode.PKCS7;
				using (ICryptoTransform cryptoTransform = aesCryptoServiceProvider.CreateEncryptor(aesCryptoServiceProvider.Key, aesCryptoServiceProvider.IV))
				{
					using (MemoryStream memoryStream = new MemoryStream())
					{
						memoryStream.Write(aesCryptoServiceProvider.IV, 0, 16);
						using (CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoTransform, CryptoStreamMode.Write))
						{
							cryptoStream.Write(bytes, 0, (int)bytes.Length);
							cryptoStream.FlushFinalBlock();
						}
						array = memoryStream.ToArray();
					}
				}
			}
			return array;
		}

		public void InitEncryptionKey(byte[] encryptionEncryptionKey)
		{
			this._encryptionKey = encryptionEncryptionKey;
		}
	}
}