using CRMWebParts;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using Microsoft.Xrm.Sdk.Workflow;
using System;
using System.Activities;
using System.Collections.Generic;
using System.Runtime.CompilerServices;

namespace VA.BAH.Core.Utilities
{
	public class GetServiceTicket : CodeActivity
	{
		private string _processName = "GetServiceTicket";

		private string _clientName = string.Empty;

		private string _clientIdentityKey = string.Empty;

		private string _clientEncryptionKey = string.Empty;

		private string _configString = string.Empty;

		private string _encryptionSalt = string.Empty;

		private int _minutesToLive = 0;

		[Input("Identifier value")]
		public InArgument<string> IdentifierValue
		{
			get;
			set;
		}

		[Output("Service ticket")]
		public OutArgument<string> ServiceTicket
		{
			get;
			set;
		}

		public GetServiceTicket()
		{
		}

		protected override void Execute(CodeActivityContext executionContext)
		{
			ITracingService extension = executionContext.GetExtension<ITracingService>();
			if (extension == null)
			{
				throw new InvalidPluginExecutionException("Failed to retrieve tracing service.");
			}
			extension.Trace(string.Concat("Entered ", this._processName, ".Execute(), Activity Instance Id: {0}, Workflow Instance Id: {1}"), new object[] { executionContext.ActivityInstanceId, executionContext.WorkflowInstanceId });
			IWorkflowContext workflowContext = executionContext.GetExtension<IWorkflowContext>();
			if (workflowContext == null)
			{
				throw new InvalidPluginExecutionException("Failed to retrieve workflow context.");
			}
			extension.Trace(string.Concat(this._processName, ".Execute(), Correlation Id: {0}, Initiating User: {1}"), new object[] { workflowContext.CorrelationId, workflowContext.InitiatingUserId });
			if (string.IsNullOrEmpty(this.IdentifierValue.Get(executionContext)))
			{
				throw new InvalidPluginExecutionException("Identifier was not supplied.");
			}
			IOrganizationService organizationService = executionContext.GetExtension<IOrganizationServiceFactory>().CreateOrganizationService(null);
			if (organizationService == null)
			{
				throw new InvalidPluginExecutionException("Failed to create organization service.");
			}
			this.GetEncryptionSettings(organizationService);
			Entity entity = organizationService.Retrieve("systemuser", workflowContext.InitiatingUserId, new ColumnSet(new string[] { "domainname" }));
			string attributeValue = entity.GetAttributeValue<string>("domainname");
			extension.Trace("ClientName: {0}", new object[] { this._clientName });
			extension.Trace("ClientIdentityKey: {0}", new object[] { this._clientIdentityKey });
			extension.Trace("ClientEncryptionKeyString: {0}", new object[] { this._clientEncryptionKey });
			extension.Trace("ClientUserName: {0}", new object[] { attributeValue });
			SecurityTokenGenerator securityTokenGenerator = new SecurityTokenGenerator();
			string str = securityTokenGenerator.GenerateToken(this._clientName, this._clientIdentityKey, attributeValue, this._minutesToLive, this._encryptionSalt, this._clientEncryptionKey, this.IdentifierValue.Get(executionContext));
			this.ServiceTicket.Set(executionContext, str);
			extension.Trace(string.Concat("Exiting ", this._processName, ".Execute(), Correlation Id: {0}"), new object[] { workflowContext.CorrelationId });
		}

		private void GetEncryptionSettings(IOrganizationService service)
		{
			foreach (Entity entity in service.RetrieveMultiple(new FetchExpression("<fetch version='1.0' output-format='xml-platform' mapping='logical' distinct='false'>\r\n              <entity name='bah_securekeyvaluepair'>\r\n                <attribute name='bah_securekeyvaluepairid' />\r\n                <attribute name='bah_name' />\r\n                <attribute name='bah_value_text' />\r\n                <attribute name='bah_value_text' />\r\n                <order attribute='bah_name' descending='false' />\r\n                <filter type='and'>\r\n                  <condition attribute='bah_name' operator='like' value='ServiceTicket%' />\r\n                </filter>\r\n              </entity>\r\n            </fetch>")).Entities)
			{
				string upper = entity.GetAttributeValue<string>("bah_name").ToUpper();
				if (upper == "SERVICETICKETCLIENTNAME")
				{
					this._clientName = entity.GetAttributeValue<string>("bah_value_text");
				}
				else if (upper == "SERVICETICKETCLIENTIDENTITYKEY")
				{
					this._clientIdentityKey = entity.GetAttributeValue<string>("bah_value_text");
				}
				else if (upper == "SERVICETICKETCLIENTENCRYPTIONKEY")
				{
					this._clientEncryptionKey = entity.GetAttributeValue<string>("bah_value_text");
				}
				else if (upper == "SERVICETICKETCLIENTENCRYPTIONSALT")
				{
					this._encryptionSalt = entity.GetAttributeValue<string>("bah_value_text");
				}
				else if (upper == "SERVICETICKETCLIENTMINUTESTOLIVE")
				{
					string attributeValue = entity.GetAttributeValue<string>("bah_value_text");
					int.TryParse(attributeValue, out this._minutesToLive);
				}
			}
			if (string.IsNullOrEmpty(this._clientName))
			{
				throw new InvalidPluginExecutionException("Could not find service ticket client name.");
			}
			if (string.IsNullOrEmpty(this._clientIdentityKey))
			{
				throw new InvalidPluginExecutionException("Could not find service ticket client identity key.");
			}
			if (string.IsNullOrEmpty(this._clientEncryptionKey))
			{
				throw new InvalidPluginExecutionException("Could not find service ticket client encryption key.");
			}
		}
	}
}