﻿using Microsoft.Xrm.Sdk;
using MVI_Search.Plugins.Messages;
using System;
using System.Activities;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MVI_Search.Plugins
{
    public abstract class BaseAttachmentAction : BaseAction
    {
        
        public abstract void Execute(ActionContext localContext, VeisConfig config, string siteUrl, string listName, string userName, string password);

        protected override void Execute(CodeActivityContext context)
        {
            // Use a 'using' statement to dispose of the service context properly
            // To use a specific early bound entity replace the 'Entity' below with the appropriate class type
            using (var localContext = new ActionContext(context))
            {
                try
                {
                    localContext.Trace("Entered the ViewAttachment Execute");
                    if (!localContext.WorkflowExecutionContext.InputParameters.Contains("Target"))
                    {
                        throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Parameter Target was not found."));
                    }
                    localContext.Trace("Target exists");
                    if (!localContext.WorkflowExecutionContext.InputParameters.Contains("SharepointID"))
                    {
                        throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Parameters SharepointID were not found."));
                    }
                    localContext.Trace("InputParmeters exist");
                    Guid guid = new Guid(localContext.WorkflowExecutionContext.InputParameters["SharepointID"].ToString());
                    if (!(guid.ToString() != ""))
                    {
                        throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "The data for parameter SharepointID is missing."));
                    }
                    Entity settings = localContext.RetrieveActiveSettings("patsr_sharepointpassword_text", "patsr_sharepointusername_text", "patsr_sharepointsiteurl_text", "patsr_sharepointlist_text");

                    string siteUrl = settings.Attributes["patsr_sharepointsiteurl_text"].ToString();
                    string listName = settings.Attributes["patsr_sharepointlist_text"].ToString();
                    string userName = settings.Attributes["patsr_sharepointusername_text"].ToString();
                    string password = localContext.DecryptCredential(settings.Attributes["patsr_sharepointpassword_text"].ToString(), 5);
                    if (string.IsNullOrEmpty(siteUrl) || string.IsNullOrEmpty(listName) || string.IsNullOrEmpty(userName) || string.IsNullOrEmpty(password))
                    {
                        throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Sharepoint settings not available."));
                    }
                    localContext.Trace("Retrieved settings");
                    VeisConfig config = localContext.RetrieveVeisConfig("patsr_sharepointserviceurl", "patsr_sharepointsubscriptionkey");
                    this.Execute(localContext, config, siteUrl, listName, userName, password);
                }
                catch (Exception exception1)
                {
                    Exception exception = exception1;
                    localContext.Trace(string.Concat("Exception on View Attachment: ", exception.Message));

                    string str2 = string.Concat("Exception on View Attachment: ", exception.Message);
                    throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", str2));
                }
            }
        }

    }
}
