﻿using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using VEIS.Plugins.Messages;
using System;
using System.Activities;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace VEIS.Plugins
{
    public class FindCreateContactAction : BaseAction
    {
        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                Response.Set(context, "");
                // Use a 'using' statement to dispose of the service context properly
                // To use a specific early bound entity replace the 'Entity' below with the appropriate class type
                using (var localContext = new ActionContext(context))
                {
                    ContactFindCreateRequest request = JsonHelper.Deserialize<ContactFindCreateRequest>(this.Request.Get<string>(context));
                    if (request.Person == null)
                    {
                        throw new ArgumentNullException("Person Not Specified in Request");
                    }

                    if (!string.IsNullOrEmpty(request.Person.VetNumber) || !string.IsNullOrEmpty(request.Person.EDIPI) || !string.IsNullOrEmpty(request.Person.FileNumber))
                    {
                        QueryByAttribute query = new QueryByAttribute
                        {
                            //change these fields as required for your implementation
                            ColumnSet = new ColumnSet("contactid", "governmentid", "veo_sensitiveveteran", "veo_edipi", "veo_filenumber"),
                            EntityName = "contact"
                        };

                        if(!string.IsNullOrEmpty(request.Person.VetNumber))
                            query.AddAttributeValue("governmentid", request.Person.VetNumber);
                        else if(!string.IsNullOrEmpty(request.Person.EDIPI))
                            query.AddAttributeValue("veo_edipi", request.Person.EDIPI);
                        else if (!string.IsNullOrEmpty(request.Person.FileNumber))
                            query.AddAttributeValue("veo_filenumber", request.Person.FileNumber);

                        EntityCollection results = localContext.OrganizationService.RetrieveMultiple(query);
                        if (results.Entities.Count == 0)
                        {
                            Entity entity = new Entity("contact");
                            entity["firstname"] = request.Person.FirstName;
                            entity["lastname"] = request.Person.LastName;
                            entity["governmentid"] = request.Person.VetNumber;
                            entity["veo_edipi"] = request.Person.EDIPI;
                            entity["veo_filenumber"] = request.Person.FileNumber;
                            entity["telephone1"] = request.Person.PrimaryPhone;
                            entity["address1_line1"] = request.Person.StreetAddressLine;
                            entity["address1_city"] = request.Person.City;
                            entity["address1_stateorprovince"] = request.Person.State;
                            entity["address1_postalcode"] = request.Person.PostalCode;
                            entity["patsr_icn"] = request.Person.ICN;
                            int sensitivityLevel;
                            entity["veo_sensitiveveteran"] = (!string.IsNullOrEmpty(request.Person.VeteranSensitivityLevel) && Int32.TryParse(request.Person.VeteranSensitivityLevel, out sensitivityLevel) && sensitivityLevel > 0);

                            entity.Id = localContext.OrganizationService.Create(entity);
                            entity = localContext.OrganizationService.Retrieve("contact", entity.Id, new ColumnSet("contactid", "fullname", "veo_sensitiveveteran"));

                            ContactFindCreateResponse response = new ContactFindCreateResponse()
                            {
                                Id = entity.Id,
                                IsSensitive = (entity.Contains("veo_sensitiveveteran") ? entity.GetAttributeValue<bool>("veo_sensitiveveteran") : false),
                                FullName = (entity.Contains("fullname") ? entity.GetAttributeValue<string>("fullname") : request.Person.FullName),
                            };
                            this.Response.Set(localContext.ActivityContext, JsonHelper.Serialize(response));
                        }
                        else if (results.Entities.Count == 1)
                        {
                            ContactFindCreateResponse response = new ContactFindCreateResponse()
                            {
                                Id = results.Entities[0].Id,
                                IsSensitive = (results.Entities[0].Contains("veo_sensitiveveteran") ? results.Entities[0].GetAttributeValue<bool>("veo_sensitiveveteran") : false),
                                FullName = (results.Entities[0].Contains("fullname") ? results.Entities[0].GetAttributeValue<string>("fullname") : request.Person.FullName)
                            };
                            //  update address/phone in crm
                            Entity vet = new Entity("contact");
                            vet.Id = results.Entities[0].Id;
                            vet["telephone1"] = request.Person.PrimaryPhone;
                            vet["address1_line1"] = request.Person.StreetAddressLine;
                            vet["address1_city"] = request.Person.City;
                            vet["address1_stateorprovince"] = request.Person.State;
                            vet["address1_postalcode"] = request.Person.PostalCode;
                            vet["patsr_icn"] = request.Person.ICN;

                            localContext.OrganizationService.Update(vet);

                            this.Response.Set(localContext.ActivityContext, JsonHelper.Serialize(response));
                        }
                        else
                        {
                            throw new Exception("Nultiple contacts found for the same SSN");
                        }
                    }
                    else
                    {
                        throw new ArgumentNullException("SSN, EDIPI or FileNumber");
                    }

                }
            }
            catch (Exception ex)
            {
                throw new InvalidPluginExecutionException(WebApiUtility.BuildExceptionMessage(ex));
            }
        }
    }
}