using System;
using System.Activities;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.ServiceModel;
using System.Runtime.Serialization;
using System.IO;
using System.Net;

using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using VEIS.Plugins.Messages;
using Microsoft.Xrm.Sdk.Workflow;

namespace VEIS.Plugins
{
    public partial class PatientFlagsAction : BaseAction
    {

        protected override void Execute(CodeActivityContext context)
        {
            // Use a 'using' statement to dispose of the service context properly
            // To use a specific early bound entity replace the 'Entity' below with the appropriate class type
            using (var localContext = new ActionContext(context))
            {
                PatientFlagsApiResponse response = new PatientFlagsApiResponse();
                response.Status = "";
                try
                {
                    string req = this.Request.Get<string>(context);
                    Entity settings = getSettings(localContext);
                    localContext.Trace("Settings retrieved.");
                    //  query patient flags api
                    string veisUrl = settings.GetAttributeValue<string>("patsr_veisservicebaseurl");
                    string patientFlagsUrl = settings.GetAttributeValue<string>("patsr_flagsapiurl");
                    string apiUrl = String.Format("{0}{1}{2}?noFilter=true", veisUrl , patientFlagsUrl,req);
                    string subKey = settings.GetAttributeValue<string>("patsr_flagsapikey");
                    localContext.Trace("apiUrl: {0}", apiUrl);
                    localContext.Trace("subKey: {0}", subKey);

                    response.Status = string.Format("api url: {0}", apiUrl);
                    HttpWebRequest apiRequest = (HttpWebRequest)WebRequest.Create(apiUrl);
                    apiRequest.Headers.Add("Ocp-Apim-Subscription-Key", subKey);
                    apiRequest.Method = "GET";
                    
                    WebResponse webResponse = apiRequest.GetResponse();
                    Stream webStream = webResponse.GetResponseStream();
                    StreamReader responseReader = new StreamReader(webStream);
                    string apiStringResponse = responseReader.ReadToEnd();
                    responseReader.Close();
                     //  Just send the string back out for now
                    Response.Set(localContext.ActivityContext, apiStringResponse);
                    //PatientFlagsApiResponse apiResponse = JsonHelper.Deserialize<PatientFlagsApiResponse>(apiStringResponse);
                    //Response.Set(localContext.ActivityContext, JsonHelper.Serialize(apiResponse));
               }
                catch (Exception ex)
                {
                    response.ErrorOccurred = true;
                    response.ErrorMessage = ex.Message;
                    Response.Set(localContext.ActivityContext, JsonHelper.Serialize(response));
                }
            }

        }

        private Entity getSettings(ActionContext localContext)
        {
            Entity settings = null;
            QueryByAttribute query = new QueryByAttribute
            {
                //change these fields as required for your implementation
                ColumnSet = new ColumnSet("patsr_veisservicebaseurl", "patsr_flagsapiurl", "patsr_flagsapikey"),
                EntityName = "patsr_setting"
            };
            query.AddAttributeValue("patsr_name", "Active Settings");

            EntityCollection results = localContext.OrganizationService.RetrieveMultiple(query);
            if (results.Entities.Count > 0)
            {
                settings = results.Entities[0];
                return settings;
            }
            throw new Exception("There are no Acitve Settings records in the system.");
        }
    }

}
