using System;
using System.Activities;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.ServiceModel;

using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using VEIS.Plugins.Messages;

namespace VEIS.Plugins
{
    public partial class PersonSearchAction : BaseAction
    {
        protected override void Execute(CodeActivityContext context)
        {
            // Use a 'using' statement to dispose of the service context properly
            // To use a specific early bound entity replace the 'Entity' below with the appropriate class type
            using (var localContext = new ActionContext(context))
            {
                MVISearchRequest request = JsonHelper.Deserialize<MVISearchRequest>(this.Request.Get<string>(context));
                Query qryObj = request.Query;
                if (qryObj.type == "person")
                {
                    QueryExpression qry = new QueryExpression(qryObj.entity)
                    {
                        ColumnSet = (qryObj.select != null) ? new ColumnSet(qryObj.select.ToLower().Split(',')) : new ColumnSet(true),
                        Criteria = new FilterExpression(LogicalOperator.Or)
                    };
                    if (qryObj.criteria != null)
                    {
                        foreach (Criteria c in qryObj.criteria)
                        {
                            qry.Criteria.Conditions.Add(new ConditionExpression(c.attribute, ConditionOperator.Equal, c.value));
                        }
                    }
                    EntityCollection results = localContext.OrganizationService.RetrieveMultiple(qry);
                    Response.Set(localContext.ActivityContext, JsonHelper.Serialize(results));
                }
            }
        }
    }
}
