using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Collections.ObjectModel;

namespace VEIS.Plugins
{
    public class VEFTInteractionStatusChange : BasePlugin
    {
		public VEFTInteractionStatusChange()
		{
		}

		public override void Execute(IServiceProvider serviceProvider)
		{
            using (var localContext = new PluginContext(serviceProvider))
            {
                try
                {
                    localContext.Trace("Entered the Execute");
                    if (!localContext.PluginExecutionContext.InputParameters.Contains("Target"))
                    {
                        throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Parameter Target was not found."));
                    }

                    localContext.Trace("getting entity image.");

                    Entity postImage = localContext.PluginExecutionContext.PostEntityImages["entity"];
                    localContext.Trace("got entity image.");

                    string strpostImage = "MISSING";

                    //get modified by name either from post image or current entity
                    if (postImage.Contains("modifiedby"))
                    {
                        strpostImage = (postImage.Contains("modifiedby") ? postImage.GetAttributeValue<EntityReference>("modifiedby").Name : "MISSING");
                    }
                    else
                    {
                        Entity entity = localContext.OrganizationService.Retrieve("incident", postImage.Id, new ColumnSet(new string[] { "modifiedby" }));//REQUEST
                        strpostImage = (entity.Contains("modifiedby") ? entity.GetAttributeValue<EntityReference>("modifiedby").Name : "MISSING");
                    }

                    //use name to get system user if still missing
                    if ((!string.IsNullOrEmpty(strpostImage) ? false : postImage.Contains("modifiedby")))
                    {
                        Entity entSysUser = localContext.OrganizationService.Retrieve("systemuser", ((EntityReference)postImage["modifiedby"]).Id, new ColumnSet(new string[] { "fullname" }));
                        strpostImage = (string)entSysUser["fullname"];
                    }
                    localContext.Trace(string.Concat("modifiedByString: ", strpostImage));

                    string strStatusCode = postImage.FormattedValues["statuscode"].ToString();
                    string strSurveyId = postImage.GetAttributeValue<string>("patsr_surveyid").ToString();
                    string attributeValue = postImage.GetAttributeValue<string>("description");

                    OptionSetValue osvContactMethod = (OptionSetValue)postImage["veo_contactmethod"];

                    if (((osvContactMethod.Value != 100000006 /*VEFT*/|| !(strStatusCode == "Assigned") &&
                        !(strStatusCode == "Inactive")) &&
                        (osvContactMethod.Value != 100000012 /*white house hotline*/ || !(strStatusCode == "Resolved")) ? false : strSurveyId != null))
                    {
                        localContext.Trace("inside first big IF.");

                        if (osvContactMethod.Value == 100000006)//VEFT
                        {
                            UpdateStatusRequest updateStatusRequest = new UpdateStatusRequest()
                            {
                                SurveyID = strSurveyId,
                                Status = strStatusCode
                            };

                            try
                            {
                                VeisConfig config = localContext.RetrieveVeisConfig("patsr_medalliaapiurl", "patsr_mvisubscriptionkey");
                                UpdateStatusResponse response = WebApiUtility.SendReceiveVeisRequest<UpdateStatusResponse>(config, "Status", updateStatusRequest);
                            }
                            catch (Exception exception2)
                            {
                                throw new InvalidPluginExecutionException(string.Format("Exception occurred while transmitting to DAC Server: {0}", exception2.ToString()));
                            }
                        }
                        else if (osvContactMethod.Value == 100000012)//white house hotline
                        {
                            localContext.Trace("interaction is WHH.");
                            string str3 = (postImage.Contains("patsr_caseguid") ? (string)postImage["patsr_caseguid"] : string.Empty);
                            string str4 = (postImage.Contains("ftp_whhresolutiondetails_text") ? (string)postImage["patsr_whhResolutionDetails"] : string.Empty);
                            if (string.IsNullOrEmpty(str3))
                            {
                                localContext.Trace("missing caseGuidText.");
                            }
                            else
                            {
                                localContext.Trace("have caseGuidText.");
                                localContext.Trace(string.Concat("modifiedby: ", strpostImage));

                                WHHUpdateStatusRequest wHHUpdateStatusRequest = new WHHUpdateStatusRequest()
                                {
                                    CaseId = str3,
                                    SurveyID = attributeValue,
                                    Status = strStatusCode,
                                    DispositionNotes = str4,
                                    ModifiedBy = strpostImage
                                };

                                if (postImage.Contains("modifiedon"))
                                {
                                    DateTime universalTime = (DateTime)postImage["modifiedon"];
                                    universalTime = universalTime.ToUniversalTime();
                                    wHHUpdateStatusRequest.ModifiedOn = universalTime.ToString("o");
                                }
                                else
                                {
                                    wHHUpdateStatusRequest.ModifiedOn = DateTime.UtcNow.ToString("o");
                                }

                                MessageLogging messageLogging = new MessageLogging()
                                {
                                    OrganizationName = localContext.OrganizationName,
                                    MaxRetry = -1,
                                    MessageName = "WHHUpdateStatusRequest",
                                    MessageType = "LOB",
                                    ParentMessageName = "VEFTInteractionStatusChange",
                                    IgnoreState = true
                                };

                                wHHUpdateStatusRequest.MessageLogging = messageLogging;
                                wHHUpdateStatusRequest.MessageId = Guid.NewGuid().ToString();
                                WHHUpdateStatusResponse wHHUpdateStatusResponse;
                                try
                                {
                                    localContext.Trace("ready to send WHHUpdateStatusRequest.");
                                    VeisConfig config = localContext.RetrieveVeisConfig("patsr_medalliaapiurl", "patsr_mvisubscriptionkey");
                                    wHHUpdateStatusResponse = WebApiUtility.SendReceiveVeisRequest<WHHUpdateStatusResponse>(config, "Status", wHHUpdateStatusRequest);

                                    localContext.Trace("got whhRes object.");
                                }
                                catch (Exception exception3)
                                {
                                    throw new InvalidPluginExecutionException(string.Format("WHH: Exception occurred while transmitting to DAC Server: {0}", exception3.Message));
                                }

                                try
                                {
                                    localContext.Trace("logging to ftp_integrationresult table.");
                                    //entIntegrationResult.Id = organizationService.Create(entIntegrationResult);
                                }
                                catch (Exception exception4)
                                {
                                    localContext.Trace(string.Concat("error logging to ftp_integrationresult table: ", exception4.ToString()));
                                }
                            }
                        }
                    }
                }
                catch (Exception exception1)
                {
                    localContext.Trace(string.Concat("Exception on Execute: ", exception1.ToString()));

                    throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", string.Concat("Exception: ", exception1.ToString())));
                }
            }
        }
	}
}