﻿using System;
using System.Collections.Generic;
using Microsoft.Xrm.Sdk;
using VRM.Integration.Servicebus.Core;
//using CRM007.CRM.SDK.Core;
using Iris.VEFT.Messages;
using CRMOLConnect.Config;
using CRMOLConnect;
using Microsoft.Xrm.Sdk.Client;
using System.ServiceModel.Description;
using System.Configuration;
using Microsoft.Xrm.Sdk.Query;
//ing VRM.Integration.Servicebus.Extensions;

namespace Iris.VEFT.WebApi.Processors
{
    class IrisCreateInteractionProcessor
    {
        private Entity interaction = new Entity("incident");

        public IMessageBase Execute(IrisCreateInteractionRequest request)
        {
            IrisCreateInteractionResponse response = new IrisCreateInteractionResponse()
            {
                ExceptionOccurred = false,
                ExceptionMessage = "",
                //FailureDetails = new List<FailureDetail>()
            };
            try
            {
                //Data Validation

                //Build Interaction Object
                addDate(request.contactDate, "patsr_dateofcontact");
                addString(request.incidentNumber, "patsr_caseguid");
                addString(request.firstName, "veo_firstname");
                addString(request.lastName, "veo_lastname");
                addString(request.ssn, "veo_lastfourssn");
                addString(request.issueText, "veo_description");
                addString(request.methodOfContact, "patsr_methodofcontactdetail");
                if (!string.IsNullOrEmpty(request.methodOfContactData))
                {
                    switch (request.methodOfContact)
                    {
                        case "Email":
                            interaction["veo_veteranemail"] = request.methodOfContactData;
                            break;
                        case "Phone":
                            interaction["patsr_callbacknumber"] = request.methodOfContactData;
                            break;
                        case "US Mail":
                            interaction["patsr_irisaddress"] = request.methodOfContactData;
                            break;
                        default:
                            break;
                    }
                }
                if (!string.IsNullOrEmpty(request.relationToVet))
                {
                    switch (request.relationToVet)
                    {
                        case "Authorized 3rd Party":
                        case "Guardian/Fiduciary":
                        case "Attorney":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000007);
                            break;
                        case "Father":
                        case "Mother":
                        case "Son":
                        case "Daughter":
                        case "Sibling":
                        case "Dependent Child":
                        case "Spouse":
                        case "Surviving Spouse":
                        case "Ex-Spouse":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000001);
                            break;
                        case "Veteran":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000000);
                            break;
                        case "General Question: Not Applicable":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000010);
                            break;
                        case "VSO":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000006);
                            break;
                        case "Helpless Child":
                        case "Funeral Director":
                            interaction["patsr_contactingentity"] = new OptionSetValue(100000011);
                            break;
                        default:
                            break;
                    }
                }
                addString(request.institution, "patsr_division");
                //static values
                interaction["statecode"] = new OptionSetValue(0);
                interaction["patsr_casetype"] = new OptionSetValue(100000000);
                interaction["veo_contactmethod"] = new OptionSetValue(100000007);
                interaction["patsr_lob"] = new OptionSetValue(100000000);
                interaction["customerid"] = new EntityReference("contact", new Guid("EBEDDB33-190D-E811-945F-0050568D743D"));

            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Unexpected Exception during data validation and Interaction building {ex.Message}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
            }

            try
            {
                Guid FTPInteraction = Guid.Empty;

                using (var OrgServiceProxy = VEIS.Utilities.CRMConnect.ConnectToCRMOL(request.organizationName.ToUpper()).Result)
                {
                    QueryByAttribute query = new QueryByAttribute
                    {
                        //change these fields as required for your implementation
                        ColumnSet = new ColumnSet("patsr_facilityid"),
                        EntityName = "patsr_facility"
                    };
                    query.AddAttributeValue("patsr_facilitycode", request.institution);

                    EntityCollection results = OrgServiceProxy.RetrieveMultiple(query);
                    if (results != null && results.Entities != null && results.Entities.Count > 0)
                    {
                        interaction["patsr_requestfacility"] = new EntityReference("patsr_facility", results.Entities[0].Id);
                    }
                    Logger.Instance.Error("After new OrgServiceProxy");
                    FTPInteraction = OrgServiceProxy.Create(interaction);
                }
                if (FTPInteraction != null && FTPInteraction != Guid.Empty)
                {
                    Logger.Instance.Debug("Interaction created");
                    response.InteractionID = FTPInteraction.ToString();
                }
                else
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Exception during Interaction creation. Null value returned";
                    //response.FailureDetails.Add(new FailureDetail()
                    //{
                    //    Reason = FailureReason.OtherException,
                    //    Details = "CRM create failure: No interaction created."
                    //});
                }
            }
            catch (TimeoutException ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Timeout during Interaction creation {ex.ToString()}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.Timeout,
                //    Details = ex.Message
                //});
            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception during Interaction creation {ex.ToString()}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
            }

            return response;
        }

        private void addDate(string date, string dest)
        {
            DateTime val = new DateTime();
            if (!DateTime.TryParse(date, out val))
            {
                Logger.Instance.Warn($"Unable to parse value {date} to {dest}");
            }
            else interaction[dest] = val;
        }

        private void addString(string val, string dest)
        {
            if (!string.IsNullOrEmpty(val))
            {
                interaction[dest] = val;
            }
        }
    }
}