﻿using System;
using System.Collections.Generic;
using Microsoft.Xrm.Sdk;
using VRM.Integration.Servicebus.Core;
//using CRM007.CRM.SDK.Core;
using Iris.VEFT.Messages;
using CRMOLConnect.Config;
using CRMOLConnect;
using System.ServiceModel.Description;
using Microsoft.Xrm.Sdk.Client;
//ing VRM.Integration.Servicebus.Extensions;

namespace Iris.VEFT.Processors
{
    class IrisCreateInteractionProcessor
    {
        private Entity interaction = new Entity("ftp_interaction");

        public IMessageBase Execute(IrisCreateInteractionRequest request)
        {
            IrisCreateInteractionResponse response = new IrisCreateInteractionResponse()
            {
                ExceptionOccurred = false,
                ExceptionMessage = "",
                //FailureDetails = new List<FailureDetail>()
            };
            try
            {
                //Data Validation

                //Build Interaction Object
                addDate(request.contactDate, "ftp_interactedon");
                //interaction["ftp_interactedon"] = request.contactDate;
                addString(request.incidentNumber, "ftp_caseguid_text");
                //interaction["ftp_caseguid_text"] = request.incidentNumber;
                addString(request.firstName, "ftp_firstname_text");
                //interaction["ftp_firstname_text"] = request.firstName;
                addString(request.lastName, "ftp_lastname_text");
                //interaction["ftp_lastname_text"] = request.lastName;
                addString(request.ssn, "ftp_lastfour_ssn_text");
                //interaction["ftp_lastfour_ssn_text"] = request.ssn;
                addString(request.issueText, "ftp_details");
                //interaction["ftp_details"] = request.issueText;
                addString(request.methodOfContact, "ftp_methodofcontactotherdesc_text");
                //interaction["ftp_methodofcontactotherdesc_text"] = request.methodOfContact;
                if (!string.IsNullOrEmpty(request.methodOfContactData))
                {
                    switch (request.methodOfContact)
                    {
                        case "Email":
                            interaction["ftp_veteranemail_text"] = request.methodOfContactData;
                            break;
                        case "Phone":
                            interaction["ftp_callbacknumber"] = request.methodOfContactData;
                            break;
                        case "US Mail":
                            interaction["ftp_irisaddress_text"] = request.methodOfContactData;
                            break;
                        default:
                            break;
                    }
                }
                if (!string.IsNullOrEmpty(request.relationToVet))
                {
                    switch (request.relationToVet)
                    {
                        case "Authorized 3rd Party":
                        case "Guardian/Fiduciary":
                        case "Attorney":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000007);
                            break;
                        case "Father":
                        case "Mother":
                        case "Son":
                        case "Daughter":
                        case "Sibling":
                        case "Dependent Child":
                        case "Spouse":
                        case "Surviving Spouse":
                        case "Ex-Spouse":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000001);
                            break;
                        case "Veteran":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000000);
                            break;
                        case "General Question: Not Applicable":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000010);
                            break;
                        case "VSO":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000006);
                            break;
                        case "Helpless Child":
                        case "Funeral Director":
                            interaction["ftp_contactingentity_code"] = new OptionSetValue(100000011);
                            break;
                        default:
                            break;
                    }
                }
                addString(request.institution, "ftp_division_text");
                //interaction["ftp_division_text"] = request.institution;
                //static values
                interaction["statecode"] = new OptionSetValue(0);
                interaction["ftp_casetype_code"] = new OptionSetValue(100000000);
                interaction["ftp_contactmethod_code"] = new OptionSetValue(100000007);
                interaction["ftp_lineofbusiness_code"] = new OptionSetValue(100000000);
                interaction["ftp_veteran"] = new EntityReference("contact", new Guid("EBEDDB33-190D-E811-945F-0050568D743D"));

            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Unexpected Exception during data validation and Interaction building {ex.Message}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
            }

            try
            {
                var customConfigurationFileReader = new VRM.Integration.Servicebus.Core.CustomConfigurationFileReader();

                var section = customConfigurationFileReader.GetCustomConfig<CrmConnectionParmsConfiguration>(VIMTConfigurationLocation.ConfigDefiningAssemblyPath,
                    VIMTConfigurationLocation.ConfigFilePath,
                    "CrmConnectionParmsConfiguration");


                var connectionParms = CrmConnectionConfiguration.Current.GetCrmConnectionParmsByName(request.organizationName.ToUpper());

                //var connectionParms = CrmConnectionConfiguration.Current.GetCrmConnectionParmsByName(request.OrganizationName.ToUpper());
                Entity interactionQuery = null;
                Guid FTPInteraction = Guid.Empty;
                if (connectionParms != null)
                {

                    var cc = new ClientCredentials();
                    cc.Windows.ClientCredential = (System.Net.NetworkCredential)System.Net.CredentialCache.DefaultCredentials;

                    var OrgServiceProxy = new OrganizationServiceProxy(new Uri(connectionParms.OrganizationServiceUrl), null, cc, null);
                    Logger.Instance.Error("After new OrgServiceProxy");
                    Logger.Instance.Error(OrgServiceProxy.IsAuthenticated.ToString());


                    FTPInteraction = OrgServiceProxy.Create(interaction);
                    OrgServiceProxy.Dispose();
                }
                if (FTPInteraction != null && FTPInteraction != Guid.Empty)
                {
                    Logger.Instance.Debug("Interaction created");
                    response.InteractionID = FTPInteraction.ToString();
                }
                else
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Exception during Interaction creation. Null value returned";
                    //response.FailureDetails.Add(new FailureDetail()
                    //{
                    //    Reason = FailureReason.OtherException,
                    //    Details = "CRM create failure: No interaction created."
                    //});
                }
            }
            catch (TimeoutException ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Timeout during Interaction creation {ex.Message}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.Timeout,
                //    Details = ex.Message
                //});
            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception during Interaction creation {ex.Message}";
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
            }

            return response;
        }

        private void addDate(string date, string dest)
        {
            DateTime val = new DateTime();
            if (!DateTime.TryParse(date, out val))
            {
                Logger.Instance.Warn($"Unable to parse value {date} to {dest}");
            }
            else interaction[dest] = val;
        }

        private void addString(string val, string dest)
        {
            if (!string.IsNullOrEmpty(val))
            {
                interaction[dest] = val;
            }
        }
    }
}