﻿using System;
using System.Collections.Generic;
using Microsoft.Xrm.Sdk;
using System.Text;
using VRM.Integration.Servicebus.Core;
//using CRM007.CRM.SDK.Core;
using Medallia.VEFT.Messages;
using VRM.Integration.Servicebus.Extensions;
using System.ServiceModel;
using CRMOLConnect.Config;
using CRMOLConnect;
using System.ServiceModel.Description;
using Microsoft.Xrm.Sdk.Client;
using System.Configuration;
using Microsoft.Xrm.Sdk.Query;

namespace Medallia.VEFT.WebApi.Processors
{
    class CreateInteractionProcessor
    {
        private string _logBuffer { get; set; }

        public IMessageBase Execute(CreateInteractionRequest request)
        {
            CreateInteractionResponse response = new CreateInteractionResponse();
            response.FailureDetails = new List<FailureDetail>();
            var progressString = new StringBuilder();
            progressString.Append("top of processor.\n");
            _logBuffer = string.Empty;

            if (request == null)
            {
                response.ExceptionMessage = "Called with no message";
                response.ExceptionOccurred = true;
                return response;
            }

            try
            {
                #region Validate Data
                //Code for handling good and bad data

                if (string.IsNullOrEmpty(request.CallbackNumber))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Callback Number";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Callback Number"
                    });
                }

                if (string.IsNullOrEmpty(request.ContactFirstName))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Contact First Name";

                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Contact First Name"
                    });
                }

                if (string.IsNullOrEmpty(request.ContactLastName))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Contact Last Name";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Contact Last Name"
                    });
                }

                if (!string.IsNullOrEmpty(request.DateOfBirth))
                {
                    var newDate = System.DateTime.MinValue;
                    request.DateOfBirth = request.DateOfBirth.Replace("_", " ");
                    var temp = DateTime.TryParse(request.DateOfBirth, out newDate);
                    if (!temp)
                    {
                        request.DateOfBirth = "";
                    }
                }

                if (string.IsNullOrEmpty(request.DateOfContact))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Date of Contact";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Date Of Contact"
                    });
                }
                else
                {
                    var newDate = System.DateTime.MinValue;
                    request.DateOfContact = request.DateOfContact.Replace("_", " ");
                    var temp = DateTime.TryParse(request.DateOfContact, out newDate);
                    if (!temp)
                    {
                        response.ExceptionOccurred = true;
                        response.ExceptionMessage = "Data Validation: Invalid Date of Contact";
                        response.FailureDetails.Add(new FailureDetail()
                        {
                            Reason = FailureReason.DataValidation,
                            Details = "Invalid Date Of Contact"
                        });
                    }
                }

                if (string.IsNullOrEmpty(request.ExperienceComments))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Experience Comments";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Experience Comments"
                    });
                }

                if (string.IsNullOrEmpty(request.MessageId))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing MessageId";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing MessageId"
                    });
                }

                if (string.IsNullOrEmpty(request.OrganizationName))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Organization Name";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Organization Name"
                    });
                }

                if (string.IsNullOrEmpty(request.SurveyID))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing SurveyId";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing SurveyId"
                    });
                }
                else
                {
                    var newInt = 0;
                    var temp = Int32.TryParse(request.SurveyID, out newInt);
                    if (!temp || newInt == 0)
                    {
                        response.ExceptionOccurred = true;
                        response.ExceptionMessage = "Data Validation: Missing SurveyId";
                        response.FailureDetails.Add(new FailureDetail()
                        {
                            Reason = FailureReason.DataValidation,
                            Details = "Invalid SurveyId"
                        });
                    }
                }

                if (string.IsNullOrEmpty(request.StationID))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing StationId";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing StationId"
                    });
                }

                if (string.IsNullOrEmpty(request.SurveyGroup))
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation: Missing Survey Group";
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Missing Survey Group"
                    });
                }
                if (response.ExceptionOccurred)
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Data Validation failures encountered. Refer to database logs for details";
                    return response;
                }
                #endregion
                else
                {
                    //Connect to CRM
                    //OrgServiceProxy = CrmConnection.Connect(connectionParms);
                    using (var OrgServiceProxy = VEIS.Utilities.CRMConnect.ConnectToCRMOL(request.OrganizationName.ToUpper()).Result)
                    {
                        Logger.Instance.Debug($"OrgSvc");

                        #region create interaction
                        var newInteraction = new Entity("incident");
                        newInteraction["veo_firstname"] = request.ContactFirstName;
                        newInteraction["veo_lastname"] = request.ContactLastName;
                        newInteraction["patsr_dateofcontact"] = Convert.ToDateTime(request.DateOfContact);
                        newInteraction["patsr_surveyid"] = request.SurveyID;
                        newInteraction["veo_lastfourssn"] = request.LastFour;
                        newInteraction["patsr_callbacknumber"] = request.CallbackNumber;
                        newInteraction["veo_contactmethod"] = new OptionSetValue(100000006); //VEFT
                                                                                                //newInteraction["ftp_surveylink_text"] = request.SurveyLink;
                        newInteraction["veo_veteranemail"] = request.ContactEmail;
                        if (!string.IsNullOrEmpty(request.DateOfBirth))
                            newInteraction["veo_dateofbirth"] = Convert.ToDateTime(request.DateOfBirth);
                        newInteraction["patsr_feedbacktype"] = request.FeedbackType;
                        newInteraction["patsr_experiencerating"] = request.ExperienceScore;
                        newInteraction["veo_description"] = request.ExperienceComments;
                        newInteraction["patsr_lob"] = new OptionSetValue(100000000);
                        newInteraction["patsr_division"] = request.StationID;

                        QueryByAttribute query = new QueryByAttribute
                        {
                            //change these fields as required for your implementation
                            ColumnSet = new ColumnSet("patsr_facilityid"),
                            EntityName = "patsr_facility"
                        };
                        query.AddAttributeValue("patsr_facilitycode", request.StationID);

                        EntityCollection results = OrgServiceProxy.RetrieveMultiple(query);
                        if (results != null && results.Entities != null && results.Entities.Count > 0)
                        {
                            newInteraction["patsr_requestfacility"] = new EntityReference("patsr_facility", results.Entities[0].Id);
                        }
                        newInteraction["patsr_crisiskeyword"] = request.AlertType;
                        newInteraction["patsr_methodofcontactdetail"] = request.SurveyGroup;
                        newInteraction["customerid"] = new EntityReference("contact", new Guid("{EBEDDB33-190D-E811-945F-0050568D743D}")); //Static guid provided by CRM dev team
                        var FTPInteraction = OrgServiceProxy.Create(newInteraction);
                        #endregion
                        if (FTPInteraction != null && FTPInteraction != new Guid())
                            response.InteractionID = FTPInteraction.ToString();
                        else
                        {
                            response.ExceptionOccurred = true;
                            response.ExceptionMessage = "Exception during Interaction creation. Null value returned";
                        }
                    }
                }
                return response;
            }
            catch (FaultException<OrganizationServiceFault> ex)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.ConnectivityException,
                    Details = $"Exception Connecting to Organization Service: {ex.ToString()}"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception occurred during Create of Interaction: {ex.ToString()}";
                return response;
            }
            catch (Exception ex)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.UnexpectedException,
                    Details = $"Exception during create of Interaction: {ex.ToString()}"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception occurred during Create of Interaction: {ex.ToString()}";
                return response;
            }
        }
    }
}
