﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Configuration;
using System.Collections.Generic;
using VRM.Integration.Servicebus.Core;
using VRM.Integration.Servicebus.Extensions;
using Medallia.VEFT.Messages;
using Medallia.VEFT.WebApi.Configuration;
//using WHH.VEFT.Messages;
//using VRMRest.WebApi;

namespace Medallia.VEFT.WebApi.Processors
{
    class MedalliaUpdateStatusProcessor
    {
        private string _bodyText = "";
        private int _retry = 0;
        private UpdateStatusResponse response = new UpdateStatusResponse();
        
        public IMessageBase Execute(UpdateStatusRequest request)
        {
            response.FailureDetails = new List<FailureDetail>();
            response.MessageId = request.MessageId;
            response.ExceptionOccurred = false;
            response.ExceptionMessage = "";
            var progressString = new StringBuilder();
            string root = MedalliaSecurityConfiguration.Current.FilePath;
            System.IO.Directory.CreateDirectory(root);
            progressString.Append("top of processor.\n");

            #region message validation
            if (request == null)
            {
                response.ExceptionMessage = "Called with no message";
                response.ExceptionOccurred = true;
                return response;
            }

            if (string.IsNullOrEmpty(request.SurveyID))
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.DataValidation,
                    Details = "Missing SurveyId"
                });
            }
            else
            {
                var newInt = 0;
                var temp = Int32.TryParse(request.SurveyID, out newInt);
                if (!temp || newInt == 0)
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.DataValidation,
                        Details = "Invalid SurveyId"
                    });
                }
            }

            if (string.IsNullOrEmpty(request.Status))
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.DataValidation,
                    Details = "Missing Request Status"
                });
            }

            #endregion
            try
            {
                _bodyText = tryGetFile(root, "PendingMedalliaUpdates.csv");

                if (_bodyText != null)
                {
                    if (_bodyText != "") _bodyText += "\r\n";
                    else _bodyText += "Status,SurveyID\r\n";
                    _bodyText += request.Status + "," + request.SurveyID;

                    tryWriteFile(root, "PendingMedalliaUpdates.csv", _bodyText);
                }
                else
                {
                    _bodyText = tryGetFile(root, "PendingMedalliaUpdates1.csv");
                    if (_bodyText != "") _bodyText += "\r\n";
                    else _bodyText += "Status,SurveyID\r\n";
                    _bodyText += request.Status + "," + request.SurveyID;
                    tryWriteFile(root, "PendingMedalliaUpdates1.csv", _bodyText);
                }

                return response;
            }
            catch (Exception ex)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.UnexpectedException,
                    Details = $"Unknown exception while trying to write pending medallia updates: {ex.Message}"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                return response;
            }
        }

        private void tryWriteFile(string root, string fileName, string _bodyText)
        {
            try
            {
                File.WriteAllText(root + fileName, _bodyText);
                _retry = 0;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    tryWriteFile(root, fileName, _bodyText);
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to write pending medallia updates: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to write pending medallia updates: {ex.Message}";
                }
            }
        }

        private string tryGetFile(string root, string fileName)
        {
            try
            {
                string getString = File.ReadAllText(root + fileName);
                _retry = 0;
                return getString;
            }
            catch (FileNotFoundException)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.OtherException,
                    Details = "File Not Found Exception: Unable to find Pending Medallia Updates File"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "File Not Found Exception: Unable to find Pending Medallia Updates File";
                
                //TODO: Add file create here: validate logic first

                return null;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    var result = tryGetFile(root, fileName);
                    return result;
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to read pending medallia updates: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to read pending medallia updates: {ex.Message}";
                    return null;
                }
            }
        }
    }
}
