﻿using System;
using System.IO;
using System.Threading;
using System.Collections.Generic;
using VRM.Integration.Servicebus.Core;
using VRM.Integration.Servicebus.Extensions;
using Medallia.VEFT.Messages;
using Medallia.VEFT.WebApi.Configuration;
using Amazon;
using Amazon.S3;
using Amazon.S3.Model;

namespace Medallia.VEFT.WebApi.Processors
{
    class PostToS3Processor
    {
        private string _logBuffer { get; set; }
        static IAmazonS3 _client;
        private string _bucketName = MedalliaSecurityConfiguration.Current.Bucket_Name;
        private string _key = $"UpdateStatus_{DateTime.Now}"; //Nonproduction
        private int _retry = 0;
        PostToS3Response response = new PostToS3Response();

        public IMessageBase Execute(PostToS3Request request)
        {
            try
            {
                #region initialize
                //response.FailureDetails = new List<FailureDetail>();
                response.ExceptionOccurred = false;
                response.ExceptionMessage = "";
                response.FailureDetails = new List<FailureDetail>();
                string root = MedalliaSecurityConfiguration.Current.FilePath;
                System.IO.Directory.CreateDirectory(root);

                if (request == null)
                {
                    response.ExceptionMessage = "Called with no message";
                    response.ExceptionOccurred = true;
                    return response;
                }
                #endregion
                #region POST to S3
                var s3Config = new AmazonS3Config
                {
                    ServiceURL = MedalliaSecurityConfiguration.Current.Service_URI,
                    //ServiceURL = "https://s3-fips-us-gov-west-1.amazonaws.com",
                    RegionEndpoint = RegionEndpoint.USGovCloudWest1
                };
                //using (_client = new AmazonS3Client("AKIAKYKWOYKEIURPP4OQ", "2rfT2PnRRyuZa+bL3nqs5d+hfftlHupcg9PPsSur", s3Config))
                using (_client = new AmazonS3Client(MedalliaSecurityConfiguration.Current.S3_Key, MedalliaSecurityConfiguration.Current.S3_Secret, s3Config))
                {
                    var _bodyText = tryGetFile(root, "PendingMedalliaUpdates.csv");
                    //var _bodyText = File.ReadAllText("D:\\WWWRoot\\Inetpub\\VEFT\\VINEXT\\PendingMedalliaUpdates.csv"); ;

                    if (_bodyText != null && _bodyText != "")
                    {
                        PutObjectRequest putRequest = new PutObjectRequest()
                        {
                            ContentBody = _bodyText,
                            BucketName = _bucketName,
                            Key = _key
                        };
                        putRequest.Metadata.Add("title", $"UpdateStatus_{DateTime.Now}"); //Nonproduction
                        //putRequest.Metadata.Add("title", $"UpdateProdStatus_{DateTime.Now}"); //Production
                        PutObjectResponse putResponse = _client.PutObject(putRequest);
                    }
                }
                #endregion
                tryWriteFile(root, "PendingMedalliaUpdates.csv", "");
            }
            catch (AmazonS3Exception amazonS3Exception)
            {
                if (amazonS3Exception.ErrorCode != null &&
                    (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") ||
                    amazonS3Exception.ErrorCode.Equals("InvalidSecurity")))
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.SecurityException,
                        Details = $"AWS Credential Failure: {amazonS3Exception.Message}"
                    });
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.OtherException,
                        Details = $"AWS Failure during upload: {amazonS3Exception.Message}"
                    });
                }
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception occurred when Putting to S3: {amazonS3Exception.Message}";
                return response;
            }
            catch (Exception ex)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.UnexpectedException,
                    Details = $"Unexpected Failure during upload: {ex.Message}"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception occurred when Putting to S3: {ex.Message}";
                return response;
            }
            return response;
        }
        private string tryGetFile(string root, string fileName)
        {
            try
            {
                string getString = File.ReadAllText(root + fileName);
                _retry = 0;
                return getString;
            }
            catch (FileNotFoundException)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.OtherException,
                    Details = "File Not Found Exception: Unable to find temp Medallia import File"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "File Not Found Exception: Unable to find temp Medallia import file";
                return null;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    var result = tryGetFile(root, fileName);
                    return result;
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to read temp Medallia import file: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to read temp Medallia import file: {ex.Message}";
                    return null;
                }
            }
        }
        private void tryWriteFile(string root, string fileName, string _bodyText)
        {
            try
            {
                File.WriteAllText(root + fileName, _bodyText);
                _retry = 0;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(3000);
                    tryWriteFile(root, fileName, _bodyText);
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to write temp Medallia import file: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to write temp Medallia import file: {ex.Message}";
                }
            }
        }
    }
}
