using System;
using System.IO;
using System.Text;
using System.Threading;
using VRM.Integration.Servicebus.Core;
using VRM.Integration.Servicebus.Extensions;
using Medallia.VEFT.Messages;
using Medallia.VEFT.WebApi.Configuration;
using Amazon;
using Amazon.S3;
using Amazon.S3.Model;
using Medallia.VEFT.WebApi;
using System.Collections.Generic;

namespace Medallia.VEFT.WebApi.Processors
{
    class RetrieveFromS3Processor
    {
        private string _logBuffer { get; set; }
        static IAmazonS3 _client;
        private string _bucketName = MedalliaSecurityConfiguration.Current.Bucket_Name;
        private List<string> _key = new List<string>();
        private int _retry = 0;
        RetrieveFromS3Response response = new RetrieveFromS3Response();

        ///Deprecated: these lines to be removed once Config settings are validated
        //private string _bucketName = "inbound.va.ugw1-atlas-prod.us-gov-west-1.medallia";
        //private static string onPremiseVIMTDACString = "https://DNS.URL/VINEXT"; //Production
        //private static string onPremiseVIMTDACString = "https://DNS.URL/INT/VEFT/VINEXT"; //Nonproduction
        //private string uname = "VEFTPROutBound"; //Production
        //private string pass = "PAHWVv2vUoyGlWtpvFpm!@"; //Production
        //private string uname = "VEFTNPVOICEInbound"; //Nonproduction
        //private string pass = "PAHWVv2vUoyGlWtpvFpm"; //Nonproduction

        public IMessageBase Execute(RetrieveFromS3Request request)
        {
            try
            {

                #region initialize
                response.MessageId = request.MessageId;
                response.ExceptionOccurred = false;
                response.ExceptionMessage = "";
                response.FailureDetails = new List<FailureDetail>();
                var progressString = new StringBuilder();
                progressString.Append("top of processor.\n");
                _logBuffer = string.Empty;

                string root = MedalliaSecurityConfiguration.Current.FilePath;
                System.IO.Directory.CreateDirectory(root);
                if (request == null)
                {
                    response.ExceptionMessage = "Called with no message";
                    response.ExceptionOccurred = true;
                    return response;
                }
                #endregion
                #region pull CSV
                var s3Config = new AmazonS3Config
                {
                    ServiceURL = MedalliaSecurityConfiguration.Current.Service_URI,
                    //ServiceURL = "https://s3-fips-us-gov-west-1.amazonaws.com",
                    RegionEndpoint = RegionEndpoint.USGovCloudWest1
                };
                //using (_client = new AmazonS3Client("AKIAKYKWOYKEIURPP4OQ", "2rfT2PnRRyuZa+bL3nqs5d+hfftlHupcg9PPsSur"))
                using (_client = new AmazonS3Client(MedalliaSecurityConfiguration.Current.S3_Key, MedalliaSecurityConfiguration.Current.S3_Secret, s3Config))
                {
                    progressString.Append("client instantiated.\n");

                    ListObjectsRequest listObjects = new ListObjectsRequest()
                    {
                        BucketName = _bucketName,
                        Prefix = MedalliaSecurityConfiguration.Current.FileMatch,
                        MaxKeys = 500
                    };

                    ListObjectsResponse listResponse = _client.ListObjects(listObjects);
                    if (listResponse != null && listResponse.S3Objects.Count > 0)
                    {
                        foreach (var obj in listResponse.S3Objects)
                        {
                            //if (obj.Key.IndexOf("va_CREATE_INTERACTION_PROD_EXPORT") != -1 && DateTime.Compare(obj.LastModified, compareDate) > 0) //Production
                            //if (obj.Key != null && obj.Key.IndexOf("va_CREATE_INTERACTION_EXPORT") != -1) //Nonproduction
                            if (obj.Key != null && obj.Key.IndexOf(MedalliaSecurityConfiguration.Current.FileMatch) != -1)
                            {
                                _key.Add(obj.Key);
                            }
                        }
                    }
                    else
                    {
                        Logger.Instance.Warn("NO EXPORT FILE FOUND IN S3: Aborting retrieval.");
                        return response;
                    }
                    if (_key == null || _key.Count == 0)
                    {
                        Logger.Instance.Warn("NO EXPORT FILE FOUND IN S3: Aborting retrieval.");
                        return response;
                    }
                    List<string> written = new List<string>();
                    List<string> deleted = new List<string>();
                    foreach (var key in _key)
                    {
                        #region pull docs
                        try
                        {
                            GetObjectRequest pull = new GetObjectRequest()
                            {
                                BucketName = _bucketName,
                                Key = key,
                                ServerSideEncryptionCustomerMethod = ServerSideEncryptionCustomerMethod.None
                            };
                            Logger.Instance.Debug("getting doc");
                            progressString.Append("Getting object.\n");
                            bool write;
                            using (GetObjectResponse get = _client.GetObject(pull))
                            {
                                progressString.Append("Writing object.\n");
                                Logger.Instance.Debug($"writing to file: {root}{key}.csv");
                                write = tryWriteFile(root, key + ".csv", get);
                            }
                            if (write)
                            {
                                written.Add(key);
                                Logger.Instance.Debug("added to written");
                            }
                        }
                        catch (AmazonS3Exception ex)
                        {
                            response.FailureDetails.Add(new FailureDetail()
                            {
                                Reason = FailureReason.OtherException,
                                Details = $"Exception while trying to retrieve Interactions from S3 ({_key.IndexOf(key) + 1}/{(_key.Count)}): {ex.Message}"
                            });
                        }
                        #endregion 
                    }
                    if (written != null && written.Count > 0)
                    {
                        #region deletefromS3
                        foreach (var key in written)
                        {
                            try
                            {
                                DeleteObjectRequest deleteRequest = new DeleteObjectRequest()
                                {
                                    BucketName = _bucketName,
                                    Key = key
                                };
                                Logger.Instance.Debug("deleting from s3");
                                _client.DeleteObject(deleteRequest);
                                deleted.Add(key);
                            }
                            catch (AmazonS3Exception ex)
                            {
                                response.FailureDetails.Add(new FailureDetail()
                                {
                                    Reason = FailureReason.OtherException,
                                    Details = $"Exception while trying to delete Interactions from S3 ({written.IndexOf(key) + 1}/{(written.Count)}): {ex.Message}"
                                });
                            }
                        }
                    }
                    #endregion
                    if (deleted != null && deleted.Count > 0)
                    {
                        foreach (var key in deleted)
                        {
                            Logger.Instance.Debug("getting file");
                            string getString = tryGetFile(root, key + ".csv");
                            if (getString != null)
                            {
                                Logger.Instance.Debug("got file");
                                getString = getString.Replace("^", "{carrot}"); //insulate against unexpected character
                                getString = getString.Replace("\n", "^");
                                string[] rows = getString.Split('^');

                                try
                                {
                                    for (int i = 1; i < rows.Length - 1; i++)
                                    {
                                        string thisRow = rows[i];
                                        string[] interaction = thisRow.Split('|');
                                        #endregion
                                        #region create interaction

                                        var newInteraction = new CreateInteractionRequest()
                                        {
                                            MessageLogging = new MessageLogging()
                                            {
                                                IgnoreState = false,
                                                MaxRetry = -1,
                                                MessageName = "CreateInteractionRequest",
                                                MessageType = "LOB",
                                                OrganizationName = request.OrganizationName,
                                                ParentMessageName = "RetrieveFromS3Processor"
                                            },
                                            OrganizationName = request.OrganizationName,
                                            ContactFirstName = interaction[2],
                                            ContactLastName = interaction[3],
                                            DateOfContact = interaction[1],
                                            SurveyID = interaction[0],
                                            LastFour = interaction[4],
                                            CallbackNumber = interaction[6],
                                            SurveyLink = interaction[13],
                                            ContactEmail = interaction[7],
                                            DateOfBirth = interaction[5],
                                            FeedbackType = interaction[8],
                                            ExperienceScore = interaction[9],
                                            ExperienceComments = interaction[10],
                                            StationID = interaction[14],
                                            AlertType = interaction[15],
                                            SurveyGroup = interaction[16]
                                        };
                                        CreateInteractionProcessor processor = new CreateInteractionProcessor();
                                        var result = processor.Execute(newInteraction);
                                    }
                                }
                                #endregion
                                #region CreateInteraction FailureLogic
                                //TODO: Add more specific catches
                                catch (Exception ex)
                                {
                                    response.FailureDetails.Add(new FailureDetail()
                                    {
                                        Reason = FailureReason.UnexpectedException,
                                        Details = $"Unexpected Exception while trying to Create Interaction in CRM ({deleted.IndexOf(key) + 1}/{(deleted.Count)}): {ex.Message}"
                                    });
                                }
                            }
                            else Logger.Instance.Error($"File Not Found: {key.Replace(":", "_")}.csv");
                            #endregion
                        }
                    }
                    #region delete locally
                    foreach (var key in written)
                    {
                        //delete local csvs
                        tryDeleteFile(root, key + ".csv");
                    }
                    #endregion
                }
            }
            catch (AmazonS3Exception amazonS3Exception)
            {
                if (amazonS3Exception.ErrorCode != null &&
                    (amazonS3Exception.ErrorCode.Equals("InvalidAccessKeyId") ||
                    amazonS3Exception.ErrorCode.Equals("InvalidSecurity")))
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.SecurityException,
                        Details = $"Security Exception while trying to Retrieve from S3: {amazonS3Exception.Message}"
                    });
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.OtherException,
                        Details = $"Exception while trying to Retrieve from S3: {amazonS3Exception.Message}"
                    });
                    if (amazonS3Exception.InnerException != null)
                        response.FailureDetails[-1].Details += "\n" + $"Inner Exception: {amazonS3Exception.InnerException}";
                }
                response.ExceptionOccurred = true;
                response.ExceptionMessage = amazonS3Exception.Message;
                return response;
            }
            catch (Exception ex)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.UnexpectedException,
                    Details = $"Unexpected Exception while trying to Retrieve from S3: {ex.Message}"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                return response;
            }
            return response;
        }

        private void tryDeleteFile(string root, string fileName)
        {
            try
            {
                fileName = fileName.Replace(":", "_");
                File.Delete(root + fileName);
                _retry = 0;
            }
            catch (FileNotFoundException)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.OtherException,
                    Details = "File Not Found Exception: Unable to delete temp import file"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "File Not Found Exception: Unable to delete temp import file";
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    tryDeleteFile(root, fileName);
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception during retrieve from S3 while trying to delete locally: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to delete locally: {ex.Message}";
                }
            }
        }

        private bool tryWriteFile(string root, string fileName, GetObjectResponse get)
        {
            try
            {
                fileName = fileName.Replace(":","_");
                get.WriteResponseStreamToFile(root + fileName);
                _retry = 0;
                return true;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    fileName = fileName.Replace(":", "_");
                    tryWriteFile(root, fileName, get);
                }
                else
                {
                    Logger.Instance.Error($"Unable to write {fileName} to location {root}");
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to write Interaction imports: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to write Interaction imports: {ex.Message}";
                    throw new Exception(response.ExceptionMessage);
                }
                return false;
            }
        }

        private string tryGetFile(string root, string fileName)
        {

            try
            {
                fileName = fileName.Replace(":", "_");
                string getString = File.ReadAllText(root + fileName);
                _retry = 0;
                return getString;
            }
            catch (FileNotFoundException)
            {
                response.FailureDetails.Add(new FailureDetail()
                {
                    Reason = FailureReason.OtherException,
                    Details = "File Not Found Exception: Unable to find Pending Medallia import"
                });
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "File Not Found Exception: Unable to find Pending Medallia import";
                return null;
            }
            catch (Exception ex)
            {
                _retry++;
                if (_retry <= 10)
                {
                    Thread.Sleep(1000);
                    var result = tryGetFile(root, fileName);
                    return result;
                }
                else
                {
                    response.FailureDetails.Add(new FailureDetail()
                    {
                        Reason = FailureReason.UnexpectedException,
                        Details = $"Unknown exception while trying to read pending medallia import: {ex.Message}"
                    });
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = $"Unexpected exception while trying to read pending medallia import: {ex.Message}";
                    return null;
                }
            }
        }

    }
}