﻿using System;
using System.Configuration;
using System.IO;
using Medallia.VEFT.Messages;
using VEIS.Utilities;
using VEIS.Utilities.Models;

namespace S3RetrieveJob
{
    class Program
    {
        private static string root = "D:\\home\\site\\wwwroot\\VEFT\\";
        static void Main(string[] args)
        {
            System.IO.Directory.CreateDirectory(root + "\\Exception");

            RetrieveFromS3Request request = new RetrieveFromS3Request()
            {
                MessageId = Guid.NewGuid().ToString(),
                OrganizationName = ConfigurationManager.AppSettings["OrgName"],
                MessageLogging = new VRM.Integration.Servicebus.Extensions.MessageLogging()
                {
                    IgnoreState = false,
                    MaxRetry = -1,
                    MessageName = "RetrieveFromS3Request",
                    MessageType = "LOB",
                    OrganizationName = ConfigurationManager.AppSettings["OrgName"]
                }
            };
            var config = RetrieveVeisConfig();
            RetrieveFromS3Response response = WebApiUtility.SendReceiveVeisRequest<RetrieveFromS3Response>(config, "AWSRetrieve", request);

            if (response.ExceptionOccurred == true)
            {
                Console.WriteLine(response.ExceptionMessage);
                if (response.FailureDetails != null)
                {
                    foreach (var fail in response.FailureDetails)
                    {
                        Console.WriteLine("Failure Details: " + fail.Details);
                    }
                }
                throw new Exception(response.ExceptionMessage);
            }

        }

        internal static VeisConfig RetrieveVeisConfig()
        {
            VeisConfig config = new VeisConfig()
            {
                OrgOverride = ConfigurationManager.AppSettings["OrgName"],
            };

            config.OrgName = ConfigurationManager.AppSettings["OrgName"];

            config.VeisConfiguration = new VeisConfiguration()
            {
                CRMAuthInfo = new CRMAuthTokenConfiguration()
                {
                    ClientApplicationId = ConfigurationManager.AppSettings["ClientId"],
                    ClientSecret = ConfigurationManager.AppSettings["ClientSecret"],
                    TenantId = ConfigurationManager.AppSettings["TenantId"],
                    ResourceId = ConfigurationManager.AppSettings["ResourceId"],

                },
                SvcConfigInfo = new VEISSvcLOBConfiguration()
                {
                    ApimSubscriptionKey = ConfigurationManager.AppSettings["SubscriptionKey"],
                }
            };

            config.VeisConfiguration.SvcConfigInfo.SvcLOBServiceUrl = ConfigurationManager.AppSettings["MedalliaLOBUrl"];
            return config;
        }
    }
}
