﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Medallia.VEFT.Messages;
using WHH.VEFT.Messages;
using VRM.Integration.Servicebus.Core;
using VRM.Integration.Servicebus.Extensions;
using System.IO;
using System.Xml;
using VEIS.Utilities;
using VEIS.Utilities.Models;

namespace PATSRUnitTests
{
    [TestClass]
    public class MedalliaUnitTesting
    {
        private static string _orgName;

        private static Guid _crmeUserId = Guid.Parse("6FF0CA08-2FD2-E411-B6F7-00155D5575E0");

        private static string veisUrl = "https://DNS.URL/";

        private static string clientId = "58d50ca3-b921-4d9b-ac76-8965be2eb80b";

        private static string clientSecret = "4RdVDdhq8cv/h2263Ojdy8fD3JI9OwHcr5hOkntAwTc=";

        private static string tenantId = "f7c49e36-971b-42c7-b244-a88eed6c0bf6";

        private static string veisOrgName = "CRMEDEV";

        private static string resourceId = "4a77476c-ceed-45db-ad7e-ac2bbbc4f72a";

        private static string subscriptionKey = "88784cf9396e41dab087c90a5da0c2c1";

        private static string medalliaPath = "patsr.lob.medallia/api";

        private static string irisPath = "patsr.lob.iris/api";

        private static string whhPath = "patsr.lob.whh/api";
        #region Initialize
        [TestInitialize]
        public void testInit()
        {
        }
        #endregion Initialize

        [TestMethod]
        public void testCreateInteraction() //VRMRest.WebApi.WebApiUtility.SendReceive<CreateInteractionResponse>(_onPremDACUri, "CreateInteractionRequest", request, null, _OnPremDACwsd);
        {

            var logging = new MessageLogging()
            {
                IgnoreState = false,
                MaxRetry = -1,
                MessageName = "CreateInteractionRequest",
                MessageType = "LOB",
                OrganizationName = "FTP",
                ParentMessageName = "UnitTest"
            };

            CreateInteractionRequest createInteractionRequest = new CreateInteractionRequest
            {
                CallbackNumber = "555-555-1234",
                ContactFirstName = "John",
                ContactLastName = "Doe",
                DateOfBirth = "12/01/1970",
                DateOfContact = DateTime.Now.Date.ToString(),
                ExperienceComments = "LOVE IT",
                ExperienceScore = "5",
                LastFour = "1234",
                OrganizationName = "FTP",
                SurveyID = "1234567",
                SurveyLink = "WWW.BING.COM",
                FeedbackType = "Compliment",
                AlertType = "Homelessness",
                StationID = "333",
                SurveyGroup = "Outpatient Survey",
                ContactEmail = "Test@test.com",
                MessageLogging = logging
            };
            try
            {
                VeisConfig config = RetrieveVeisConfig(medalliaPath);
                CreateInteractionResponse createInteractionResponse = WebApiUtility.SendReceiveVeisRequest<CreateInteractionResponse>(config, "Interaction", createInteractionRequest);

                var test = createInteractionResponse.ExceptionMessage;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }


        [TestMethod]
        public void testMedaliaFAILURES() //VRMRest.WebApi.WebApiUtility.SendReceive<CreateInteractionResponse>(_onPremDACUri, "CreateInteractionRequest", request, null, _OnPremDACwsd);
        {

            var logging = new MessageLogging()
            {
                IgnoreState = false,
                MaxRetry = -1,
                MessageName = "CreateInteractionRequest",
                MessageType = "LOB",
                OrganizationName = "FTP"
            };

            var request = new CreateInteractionRequest
            {
                CallbackNumber = "555-555-1234",
                ContactFirstName = "John",
                //ContactLastName = "Doe",
                DateOfBirth = "12/01/1970",
                DateOfContact = "THIS IS NOT A DATE",
                ExperienceComments = "LOVE IT",
                ExperienceScore = "5",
                LastFour = "1234",
                OrganizationName = "FTP",
                SurveyID = "asdf",
                SurveyLink = "WWW.BING.COM",
                FeedbackType = "Compliment",
                AlertType = "Homelessness",
                StationID = "333",
                SurveyGroup = "Outpatient Survey",
                ContactEmail = "Test@test.com",
                MessageLogging = logging
            };
            try
            {
                VeisConfig config = RetrieveVeisConfig(medalliaPath);
                CreateInteractionResponse createInteractionResponse = WebApiUtility.SendReceiveVeisRequest<CreateInteractionResponse>(config, "Interaction", request);

                var test = createInteractionResponse.ExceptionMessage;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }

        [TestMethod]
        public void TestS3Retrieve()
        {
            try
            {
                RetrieveFromS3Request request = new RetrieveFromS3Request()
                {
                    MessageId = Guid.NewGuid().ToString(),
                    OrganizationName = "ftp",
                    MessageLogging = new VRM.Integration.Servicebus.Extensions.MessageLogging()
                    {
                        IgnoreState = false,
                        MaxRetry = -1,
                        MessageName = "RetrieveFromS3Request",
                        MessageType = "LOB",
                        OrganizationName = "FTP",
                        ParentMessageName = "UnitTest"
                    }
                };

                VeisConfig config = RetrieveVeisConfig(medalliaPath);
                RetrieveFromS3Response response = WebApiUtility.SendReceiveVeisRequest<RetrieveFromS3Response>(config, "AWSRetrieve", request);
                Assert.AreNotEqual(null, response);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }

        [TestMethod]
        public void TestS3Put()
        {
            try
            {
            }
            catch (Exception ex)
            {
            }
        }

        [TestMethod]
        public void testResponseNoLogging()
        {
            try
            {
                UpdateStatusRequest request = new UpdateStatusRequest()
                {
                    SurveyID = "1234567",
                    Status = "Closed"
                };

                VeisConfig config = RetrieveVeisConfig(medalliaPath);
                RetrieveFromS3Response response = WebApiUtility.SendReceiveVeisRequest<RetrieveFromS3Response>(config, "Status", request);

                Assert.AreNotEqual(null, response);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }

        internal static VeisConfig RetrieveVeisConfig(string path)
        {
            VeisConfig config = new VeisConfig()
            {
                OrgOverride = veisOrgName,
            };

            config.OrgName = veisOrgName;

            config.VeisConfiguration = new VeisConfiguration()
            {
                CRMAuthInfo = new CRMAuthTokenConfiguration()
                {
                    ClientApplicationId = clientId,
                    ClientSecret = clientSecret,
                    TenantId = tenantId,
                    ResourceId = resourceId,

                },
                SvcConfigInfo = new VEISSvcLOBConfiguration()
                {
                    ApimSubscriptionKey = subscriptionKey,
                }
            };

            config.VeisConfiguration.SvcConfigInfo.SvcLOBServiceUrl = veisUrl + path;
            return config;
        }
    }
}
