﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using WHH.VEFT.Messages;
using VRM.Integration.Servicebus.Core;
using VRM.Integration.Servicebus.Extensions;
using VEIS.Utilities;
using VEIS.Utilities.Models;

namespace PATSRUnitTests
{
    [TestClass]
    public class UnitTest2
    {
        private static string _orgName;

        private static Guid _crmeUserId = Guid.Parse("6FF0CA08-2FD2-E411-B6F7-00155D5575E0");

        private static string veisUrl = "https://DNS.URL/";

        private static string clientId = "58d50ca3-b921-4d9b-ac76-8965be2eb80b";

        private static string clientSecret = "4RdVDdhq8cv/h2263Ojdy8fD3JI9OwHcr5hOkntAwTc=";

        private static string tenantId = "f7c49e36-971b-42c7-b244-a88eed6c0bf6";

        private static string veisOrgName = "CRMEDEV";

        private static string resourceId = "4a77476c-ceed-45db-ad7e-ac2bbbc4f72a";

        private static string subscriptionKey = "88784cf9396e41dab087c90a5da0c2c1";

        private static string medalliaPath = "patsr.lob.medallia/api";

        private static string irisPath = "patsr.lob.iris/api";

        private static string whhPath = "patsr.lob.whh/api";

        #region Initialize
        [TestInitialize]
        public void testInit()
        {

        }
        #endregion Initialize


        [TestMethod]
        public void testWHHCreate() //VRMRest.WebApi.WebApiUtility.SendReceive<CreateInteractionResponse>(_onPremDACUri, "CreateInteractionRequest", request, null, _OnPremDACwsd);
        {
            var random = new Random();


            WHHCreateInteractionRequest wHHCreateInteractionRequest = new WHHCreateInteractionRequest
            {
                OrganizationName = "FTP",
                //MessageLogging = logging,
                CaseId = Guid.NewGuid().ToString(),
                CaseNumber = random.Next(1000000, 9999999),
                CreatedDate = DateTime.Now.Date.AddDays(-3).ToString(),
                VIEWS_Facility__c = "629",
                SENSITIVE_WHVAHotline_Subject__c = "This is the subject",
                WHVAHotline_Comments__c = "These are the comments",
                OCR_Notes = "This is the OCR Notes",
                WHVAHotline_SLA_Start_Date_Tim = DateTime.Now.Date.AddDays(-3).ToString(),
                WHVAHotline_SLA_Business_Days__c = 6,
                VIEWS_SLA_End_Date_DateTime__c = DateTime.Now.Date.AddDays(3).ToString(),
                Days_Until_SLA_Overdue__c = 3,
                VIEWS_Tier_1_Complaint_Category__c = "The complaint category",
                VIEWS_Complaint_Category_Description__c = "The complaint category description",
                Short_Description__c = "Other complaint type",
                Owner = "The owner",
                LastModifiedBy = "Last Modified By",
                Status = "Status",
                WHVAHotline_Priority_One_High__c = "Priority One String",
                PriorityOne_Indicator = true,
                WH_VA_Hotline_Reason_Priority__c = "",
                Secondary_Phone__c = "(123) 234-3456",
                WHHotline_Other_Location__c = "Some other facility",
                Views_Administration__c = "Board",
                Secondary_Email__c = "test@test.com",
                RecordType = "non-complaint",
                VIEWS_Other_Relationship_Informtation__c = "Someone other than the veteran?",
                VIEWS_Relationship_to_Veteran_ = "Other",
                Anonymous__c = false,
                FirstName = "John",
                LastName = "Doe",
                Email = "test@test.test",
                Phone = "123-123-1234",
                Birthdate = "01/15/1977",
                FirstName_Proxy = "Jane",
                LastName_Proxy = "Doe",
                Phone_Proxy = "321-321-3214",
                Email_Proxy = "thing@stuff.net",
                SecondaryPhone_Proxy = "345-345-3456",
                PreferredEmail_Proxy = "stuff@thing.net",
                Internal_OCR_DueDate = "06/06/2018",
                VISN = "",
                ICN = "1012901147V954482"
            };
            try
            {
                VeisConfig config = RetrieveVeisConfig(whhPath);
                WHHCreateInteractionResponse wHHCreateInteractionResponse = WebApiUtility.SendReceiveVeisRequest<WHHCreateInteractionResponse>(config, "Interaction", wHHCreateInteractionRequest);

                var test = wHHCreateInteractionResponse.ExceptionMessage;

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }

        [TestMethod]
        public void testWHHUpdate() //VRMRest.WebApi.WebApiUtility.SendReceive<CreateInteractionResponse>(_onPremDACUri, "CreateInteractionRequest", request, null, _OnPremDACwsd);
        {
            var random = new Random();

            var logging = new MessageLogging()
            {
                IgnoreState = false,
                MaxRetry = -1,
                MessageName = "CreateInteractionRequest",
                MessageType = "LOB",
                OrganizationName = "FTP"
            };

            var request = new WHHUpdateStatusRequest
            {
                OrganizationName = "FTP",
                MessageLogging = logging,
                CaseId = "C-629.201812345",
                DispositionNotes = "Some Notes",
                MessageId = Guid.NewGuid().ToString(),
                ModifiedBy = "Me",
                ModifiedOn = "2018-05-22T13:42:00.000Z",
                SurveyID = "500r0000003XZn6AAG",
                Status = "Resolved"
            };
            try
            {
                VeisConfig config = RetrieveVeisConfig(whhPath);
                WHHUpdateStatusResponse response = WebApiUtility.SendReceiveVeisRequest<WHHUpdateStatusResponse>(config, "Status", request);
                var test = "test";
                //var test = response.ExceptionMessage;
                //var res = request.SendReceive<WHHUpdateStatusResponse>(MessageProcessType.Local);

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine(ex.StackTrace);
                Assert.Fail(ex.Message);
            }
        }

        internal static VeisConfig RetrieveVeisConfig(string path)
        {
            VeisConfig config = new VeisConfig()
            {
                OrgOverride = veisOrgName,
            };

            config.OrgName = veisOrgName;

            config.VeisConfiguration = new VeisConfiguration()
            {
                CRMAuthInfo = new CRMAuthTokenConfiguration()
                {
                    ClientApplicationId = clientId,
                    ClientSecret = clientSecret,
                    TenantId = tenantId,
                    ResourceId = resourceId,

                },
                SvcConfigInfo = new VEISSvcLOBConfiguration()
                {
                    ApimSubscriptionKey = subscriptionKey,
                }
            };

            config.VeisConfiguration.SvcConfigInfo.SvcLOBServiceUrl = veisUrl + path;
            return config;
        }

    }
}
