﻿using System;
using System.Configuration;
using System.Reflection;
using System.Collections.Concurrent;

namespace VEIS.Core.Configuration
{

    public class CustomConfigurationFileReader
    {
        private Assembly _ConfigurationDefiningAssembly;
        private static ConcurrentDictionary<string, object> _sectionDictionary = new ConcurrentDictionary<string, object>();
        protected Assembly ConfigResolveEventHandler(object sender, ResolveEventArgs args)
        {
            return _ConfigurationDefiningAssembly;
        }

        public T GetCustomConfig<T>(string configDefiningAssemblyPath,
            string configFilePath,
            string sectionName) where T : IConfigurationSectionHandler
        {
            AppDomain.CurrentDomain.AssemblyResolve += ConfigResolveEventHandler;
            try
            {
                if (!_sectionDictionary.ContainsKey(sectionName))

                {
                    _ConfigurationDefiningAssembly = Assembly.LoadFrom(configDefiningAssemblyPath);

                    var exeFileMap = new ExeConfigurationFileMap { ExeConfigFilename = configFilePath };

                    var customConfig = ConfigurationManager.OpenMappedExeConfiguration(exeFileMap,
                        ConfigurationUserLevel.None);

                    var configSection = customConfig.GetSection(sectionName).GetAs<T>();

                    _sectionDictionary.TryAdd(sectionName, configSection);

                }

                object config;

                bool hasConfig = _sectionDictionary.TryGetValue(sectionName, out config);

                return (T)config;
            }
            finally
            {
                AppDomain.CurrentDomain.AssemblyResolve -= ConfigResolveEventHandler;
            }
        }
    }
}
