using System;
using System.Collections.Generic;
using System.Runtime.Serialization;

namespace VEIS.Messages.VideoVistService
{
    [DataContract]
    public class EcPatients
    {
        [DataMember]
        public EcPersonIdentifier Id { get; set; }

        [DataMember]
        public EcPersonName Name { get; set; }

        [DataMember]
        public EcContactInformation ContactInformation { get; set; }

        [DataMember]
        public EcLocation Location { get; set; }

        [DataMember]
        public EcVirtualMeetingRoom VirtualMeetingRoom { get; set; }

        [DataMember]
        public bool VistaDateTimeSpecified { get; set; }

        [DataMember]
        public DateTime VistaDateTime { get; set; }
    }

    [DataContract]
    public class EcPersonIdentifier
    {
        [DataMember]
        public string AssigningAuthority { get; set; }

        [DataMember]
        public string UniqueId { get; set; }
    }

    [DataContract]
    public class EcProviders
    {
        [DataMember]
        public EcPersonName Name { get; set; }

        [DataMember]
        public EcPersonIdentifier Id { get; set; }

        [DataMember]
        public EcContactInformation ContactInformation { get; set; }

        [DataMember]
        public EcVirtualMeetingRoom VirtualMeetingRoom { get; set; }

        [DataMember]
        public EcLocation Location { get; set; }

        [DataMember]
        public bool VistaDateTimeSpecified { get; set; }

        [DataMember]
        public DateTime VistaDateTime { get; set; }
    }

    [DataContract]
    public class EcPersonName
    {
        [DataMember]
        public string FirstName { get; set; }

        [DataMember]
        public string LastName { get; set; }

        [DataMember]
        public string MiddleInitial { get; set; }
    }

    [DataContract]
    public class EcContactInformation
    {
        [DataMember]
        public string Mobile { get; set; }

        [DataMember]
        public string PreferredEmail { get; set; }

        [DataMember]
        public string AlternativeEmail { get; set; }

        [DataMember]
        public int TimeZone { get; set; }

        [DataMember]
        public bool TimeZoneSpecified { get; set; }
    }

    [DataContract]
    public class EcVirtualMeetingRoom
    {
        [DataMember]
        public string Conference { get; set; }

        [DataMember]
        public string Pin { get; set; }

        [DataMember]
        public string Url { get; set; }
    }

    [DataContract]
    public class EcLocation
    {
        [DataMember]
        public EcLocationType Type { get; set; }

        [DataMember]
        public EcFacility Facility { get; set; }

        [DataMember]
        public EcClinic Clinic { get; set; }
    }

    [DataContract]
    public class EcFacility
    {
        [DataMember]
        public string Name { get; set; }

        [DataMember]
        public string SiteCode { get; set; }

        [DataMember]
        public int TimeZone { get; set; }
    }

    [DataContract]
    public class EcClinic
    {
        [DataMember]
        public string Ien { get; set; }

        [DataMember]
        public string Name { get; set; }
    }

    [DataContract]
    public class EcStatus
    {
        [DataMember]
        public string Description { get; set; }

        [DataMember]
        public EcStatusCode Code { get; set; }

        [DataMember]
        public bool CodeSpecified { get; set; }

        [DataMember]
        public EcReasonCode Reason { get; set; }

        [DataMember]
        public bool ReasonSpecified { get; set; }
    }

    public enum EcStatusCode
    {
        CHECKED_IN,
        CHECKED_OUT,
        NO_ACTION_TAKEN,
        NO_SHOW,
        CANCELLED_BY_CLINIC,
        NO_SHOW_AND_AUTO_RE_BOOK,
        CANCELLED_BY_CLINIC_AND_AUTO_RE_BOOK,
        INPATIENT_APPOINTMENT,
        CANCELLED_BY_PATIENT,
        CANCELLED_BY_PATIENT_AND_AUTO_REBOOK,
        FUTURE,
        NON_COUNT,
        DELETED,
        ACTION_REQUIRED
    }

    public enum EcReasonCode
    {
        APPOINTMENT_NO_LONGER_REQUIRED,
        CLINIC_CANCELLED,
        CLINIC_STAFFING,
        DEATH_IN_FAMILY,
        INPATIENT_STATUS,
        OTHER,
        PATIENT_DEATH,
        PATIENT_NOT_ELIGIBLE,
        SCHEDULING_CONFLICT_OR_ERROR,
        TRANSFER_OPT_CARE_TO_OTHER_VA,
        TRAVEL_DIFFICULTY,
        UNABLE_TO_KEEP_APPOINTMENT,
        WEATHER
    }

    [DataContract]
    public class EcWriteResults
    {
        [DataMember]
        public List<EcWriteResult> EcWriteResult { get; set; }
    }

    [DataContract]
    public class EcWriteResult
    {
        [DataMember]
        public string PersonId { get; set; }

        [DataMember]
        public EcPersonName Name { get; set; }

        [DataMember]
        public string FacilityCode { get; set; }

        [DataMember]
        public string FacilityName { get; set; }

        [DataMember]
        public string ClinicIen { get; set; }

        [DataMember]
        public string ClinicName { get; set; }

        [DataMember]
        public DateTime DateTime { get; set; }

        [DataMember]
        public EcVistaStatus VistaStatus { get; set; }

        [DataMember]
        public string Reason { get; set; }
    }

    public enum EcVistaStatus
    {
        BOOKED,
        FAILED_TO_BOOK,
        RECEIVED,
        FAILED_TO_RECEIVE,
        CANCELLED,
        FAILED_TO_CANCEL
    }

    public enum EcSchedulingRequestType
    {
        NEXT_AVAILABLE_APPT,
        OTHER_THAN_NEXT_AVAILABLE_CLINICIAN_REQUESTED,
        OTHER_THAN_NEXT_AVAILABLE_PATIENT_REQUESTED,
        WALKIN_APPTOINTMENT,
        MULTIPLE_APPTOINTMENT_BOOKING,
        AUTO_REBOOK,
        OTHER_THAN_NEXT_AVAILABLE_APPOINTMENT
    }

    public enum EcAppointmentType
    {
        COMPENSATION_AND_PENSION,
        CLASS_II_DENTAL,
        ORGAN_DONORS,
        EMPLOYEE,
        PRIMA_FACIA,
        RESEARCH,
        COLLATERAL_OF_VET,
        SHARING_AGREEMENT,
        REGULAR,
        COMPUTER_GENERATED,
        SERVICE_CONNECTED
    }

    public enum EcAppointmentKind
    {
        CLINIC_BASED,
        STORE_FORWARD,
        MOBILE_ANY,
        MOBILE_GFE,
        ADHOC
    }

    public enum EcLocationType
    {
        VA,
        NonVA
    }
}