﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using VRM.Integration.Servicebus.Core;

namespace VRM.Integration.Servicebus.Extensions
{
    [Serializable]
    [DataContract]
    //TODO: set all members other than IgnoreState to EmitDefaultValue = false to minimize impact on LOBs not leveraging this capability. 
    //Handler code must be updated to account for those missing (not null) members
    public abstract class MessageBaseExtensions : MessageBase
    {
        /// <summary>
        /// Class MessageLogging: This class contains all data members required for using database-controlled transaction logging in VIMT/ViNext
        /// Instantiate this class object and populate its constituent members to control database logging capabilities
        /// </summary>
        [DataMember(IsRequired = false)]
        public MessageLogging MessageLogging { get; set; }
        /// <summary>
        /// Class FailureDetails: This class contains Data Members appropriate to the specification of failure reasons
        /// Instantiate this class object and populate its constituent members to provide details on messaging failures for reporting
        /// </summary>
        [DataMember(IsRequired = false)]
        public List<FailureDetail> FailureDetails { get; set; }
    }

    public enum FailureReason
    {
        Unknown,
        DataValidation,
        OtherException,
        ConnectivityException,
        SecurityException,
        UnexpectedException,
        Timeout
    }

    public class FailureDetail
    {
        public FailureReason Reason { get; set; }
        public string Details { get; set; }
    }

    public class MessageLogging
    {
        /// <summary>
        /// DataMember OrganizationName: allows designation of the source (or destination) CRM Organization. 
        /// Enbales reporting on traffic by line of business
        /// </summary>
        public string OrganizationName { get; set; }
        /// <summary>
        /// DataMember RetryMessageId: allows designation of the original MessageId in the event of a retry message.
        /// For the moment, this string acts as a boolean flag for whether or not the message is a retry attempt
        /// Future implementation may leverage this element for 'blind' retries requiring nothing but the MessageId
        /// </summary>
        public string RetryMessageId { get; set; }
        /// <summary>
        /// DataMember MaxRetry: allows LOBs to set a maximum value for the number of retries to be accepted. 
        /// If MessageState.RetryCount meets or exceeds this value, a retry will not be performed.
        /// </summary>
        public int MaxRetry { get; set; }
        /// <summary>
        /// DataMember RetryHandler: Non-functional.
        /// Future implementations may allow use of this attribute to designate a custom RetryHandler class for retries rather than the standard LOB for this message
        /// </summary>
        public string RetryHandler { get; set; }
        /// <summary>
        /// DataMember MessageName: Allows naming of this message for DB tracking and reporting.
        /// Future implementations may leverage this attribute for blind retries, so LOBs are encouraged to use the exact string messagename, as found in the messagehandler
        /// </summary>
        public string MessageName { get; set; }
        /// <summary>
        /// DataMember MessageType: Allows type designation of this message (excpected use as either EC or LOB).
        /// </summary>
        public string MessageType { get; set; }
        /// <summary>
        /// DataMember ParentMessageName: Allows LOBs to assign the parent, or previous calling message for the purpose of designating chained behaviors
        /// Example: for an EC, the ParentMessageName would be the invoking LOB
        /// </summary>
        public string ParentMessageName { get; set; }

        /// <summary>
        /// DataMember IgnoreState: Allows LOBs to opt out of message state tracking in DB, for example, if they prefer to handle state client side or will not be issuing retries
        /// True: Do not track state in database. Proceed with retries, ignoring counts or states.
        /// False: Track state in database. Observe retrymax and state constraints on retries.
        /// Member defaults to False if not provided.
        /// </summary>
        public bool IgnoreState { get; set; }
    }
}
