﻿using System;
using VRM.Integration.Servicebus.Core;
using System.Data.SqlClient;
using System.Collections.Generic;
using System.Threading.Tasks;
using VRM.Integration.Servicebus.Extensions.Configuration;

namespace VRM.Integration.Servicebus.Extensions
{
    public abstract class RequestResponseHandlerExtensions : RequestResponseHandler
    {
        //private static string _connstr = "Data Source=VRMNPCRM15-AOAG.DEV.VRMCLOUD.LOCAL;Initial Catalog=NP_MessageLogging;Integrated Security=True"; //SQL connection string
        private static string _connstr = ExtensionsSecurityConfiguration.Current.connStr;

        private Message _message = new Message(); //Class representing the DB Message row corresponding to this transaction
        private MessageState _messageState = new MessageState(); //Class representing the DB MessageState row corresponding to this transaction
        private Transaction _transaction = new Transaction(); //Class representing the DB Transaction row corresponding to this transaction
        private Failure _failure = new Failure(); //Class representing the DB Failure row(s) corresponding to this transaction

        /// <summary>
        /// PLACEHOLDER - Not Yet Implemented!
        /// 
        /// The ExceptionDelimiter character exists for helping legacy adopters of this logging mechanism adopt a more robust logging approach.
        /// Many legacy LOB implementations use a flat exceptionOccurred/exceptionMessage return for all exceptions, but because of more robust 
        /// capture and reporting capabilities in SQL server, a flat object is no longer required. If your implementation uses a flat string, 
        /// but you have multiple exceptions being caught in a single string, simply delimit these exceptions with this specified character (default: pipe)
        /// and the logging extensions base class will split them into individually reported exceptions
        /// <para />NOTE: The preferred mechanism for passing back failures and exceptions is the FailureDetails property found in MessageBaseExtensions.
        /// This property allows you to assign categorization for failures as well as passing them in a list rather than character delimited. When possible, 
        /// you are encouraged to switch to this preferred method rather than exceptionOccurred/exceptionMessage and this delimiter
        /// </summary>
        public char ExceptionDelimiter = '|';
        
        public override IMessageBase HandleRequestResponse(object message)
        {
            #region PRE-Execution
            try
            {
                MessageLogging logObject = new MessageLogging();

                if ((MessageLogging)message?.GetType().GetProperty("MessageLogging")?.GetValue(message) != null)
                {
                    logObject = (MessageLogging)message.GetType().GetProperty("MessageLogging").GetValue(message);
                    string MessageId = (string)message.GetType().GetProperty("MessageId").GetValue(message);
                    ///Section not yet implemented. Reserved for inserting retry logic
                    if (logObject.RetryMessageId == null)
                    {
                        #region build and log message
                        _message.EncodedString = message.SerializeToString();
                        _message.MessageId = MessageId;
                        _message.MessageName = logObject.MessageName ?? "";
                        _message.OrganizationName = logObject.OrganizationName ?? "";
                        _message.MessageType = logObject.MessageType ?? "";
                        _message.ParentMessageName = logObject.ParentMessageName ?? "";
                        _message.insertMessage(_connstr);
                        #endregion

                        #region build and log message state
                        if (!logObject.IgnoreState)
                        {
                            _messageState.MessageId = MessageId;
                            _messageState.RetryCount = 0;
                            _messageState.State = "Active";
                            _messageState.insertMessageState(_connstr);
                        }
                        #endregion
                    }
                    else
                    {
                        _messageState.MessageId = logObject.RetryMessageId ;
                        _messageState.selectMessageState(_connstr);
                    }

                    #region build and log transaction
                    _transaction.TransactionId = Guid.NewGuid().ToString();
                    _transaction.MessageId = MessageId;
                    _transaction.StartTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff");
                    _transaction.insertTransaction(_connstr);
                    #endregion
                }
            }
            catch (Exception ex)
            {
                //swallow exceptions
                Logger.Instance.Debug($"Unexpected Exception occurred inside Servicebus Extensions RequestResponseHandler: {ex.Message}");
            }
            #endregion

            //Execute Processor
            object responsemessage = new object();
            try
            {
                responsemessage = LoggedRequestResponse(message);
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(ex.Message);
            }

            #region POST-Execution
            try
            {
                if ((MessageLogging)message?.GetType().GetProperty("MessageLogging")?.GetValue(message) != null)
                {
                    var logObject = (MessageLogging)message.GetType().GetProperty("MessageLogging").GetValue(message);
                    //TODO: incorporate the exceptionOccurred flag as a failover path for FailureDetails
                    List<FailureDetail> failureDetails = new List<FailureDetail>();
                    if ((List<FailureDetail>)responsemessage?.GetType().GetProperty("FailureDetails")?.GetValue(responsemessage) != null)
                        failureDetails = (List<FailureDetail>)responsemessage.GetType().GetProperty("FailureDetails").GetValue(responsemessage);
                    if (failureDetails != null && failureDetails.Count > 0)
                    {
                        #region handle failures
                        //_failure.TransactionId = _transaction.TransactionId;

                        //string[] reasons;
                        //if ((string)responsemessage?.GetType().GetProperty("ExceptionMessage")?.GetValue(responsemessage) != null)
                        //    reasons = ((string)responsemessage.GetType().GetProperty("ExceptionMessage").GetValue(responsemessage)).Split(ExceptionDelimiter);
                        foreach (var failure in failureDetails)
                        {
                            var thisFailure = new Failure()
                            {
                                TransactionId = _transaction.TransactionId,
                                FailureId = Guid.NewGuid().ToString(),
                                Reason = failure.Reason.ToString(),
                                Detail = failure.Details
                            };
                            
                            thisFailure.insertFailure(_connstr);
                        }

                        if (logObject.MaxRetry != -1)
                        {
                            if (_messageState.RetryCount >= logObject.MaxRetry)
                                _messageState.State = "Aborted";
                        }
                        _messageState.RetryCount++;
                        _transaction.Result = "Failure";
                        #endregion
                    }
                    else
                    {
                        _messageState.State = "Success";
                        _transaction.Result = "Success";
                    }
                    if (!logObject.IgnoreState)
                    {
                        _messageState.updateMessageState(_connstr);
                    }
                    _transaction.EndTime = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff");
                    _transaction.updateTransaction(_connstr);
                }
            }
            catch (Exception ex)
            {
                //swallow
                Logger.Instance.Debug($"Unexpected Exception occurred inside Servicebus Extensions RequestResponseHandler: {ex.Message}");
            }
            #endregion
            return (IMessageBase)responsemessage;
        }

        /// <summary>
        /// LoggedRequestResponse: Abstract method to invoke in handlers derrived from the RequestResponseHandlerExtensions.
        /// Handler execution to be contained within this method.
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        public abstract IMessageBase LoggedRequestResponse(object message);
        
    }

    public class Message
    {
        public string EncodedString { get; set; }
        public string MessageId { get; set; }
        public string MessageName { get; set; }
        public string OrganizationName { get; set; }
        public string MessageType { get; set; }
        public string ParentMessageName { get; set; }
        internal void insertMessage(string connStr)
        {
            string queryString = "INSERT INTO Message (MessageId,EncodedString,MessageName,OrganizationName,MessageType,ParentMessageName) VALUES (@MessageId,@MessageStr,@MessageName,@OrgName,@MessageType,@ParentMessageName)";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        command.Parameters.AddWithValue("@MessageStr", EncodedString ?? "");
                        command.Parameters.AddWithValue("@MessageName", MessageName ?? "");
                        command.Parameters.AddWithValue("@OrgName", OrganizationName ?? "");
                        command.Parameters.AddWithValue("@MessageType", MessageType ?? "");
                        command.Parameters.AddWithValue("@ParentMessageName", ParentMessageName ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
               Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.InsertMessage: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.InsertMessage: {0}.", ex.Message));
            }
        }
        internal void selectMessage(string connStr)
        {
            if (String.IsNullOrEmpty(MessageId))
            {
                Logger.Instance.Error("MessageId is null or empty. Unable to retrieve message.");
            }
            else
            {
                string queryString = "SELECT * FROM Message WHERE MessageId=@MessageId;";
                try
                {
                    using (SqlConnection connection = new SqlConnection(connStr))
                    {
                        using (SqlCommand command = new SqlCommand(queryString, connection))
                        {
                            command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                            connection.Open();
                            SqlDataReader reader = command.ExecuteReader();
                            while (reader.Read())
                            {
                                int EncodedStringIndex = reader.GetOrdinal("EncodedString");
                                int MessageNameIndex = reader.GetOrdinal("MessageName");
                                int OrganizationNameIndex = reader.GetOrdinal("OrganizationName");
                                int MessageTypeIndex = reader.GetOrdinal("MessageType");
                                int ParentMessageNameIndex = reader.GetOrdinal("ParentMessageName");
                                if (!reader.IsDBNull(EncodedStringIndex))
                                {
                                    EncodedString = reader.GetString(EncodedStringIndex);
                                }
                                if (!reader.IsDBNull(MessageNameIndex))
                                {
                                    MessageName = reader.GetString(MessageNameIndex);
                                }
                                if (!reader.IsDBNull(OrganizationNameIndex))
                                {
                                    OrganizationName = reader.GetString(OrganizationNameIndex);
                                }
                                if (!reader.IsDBNull(MessageTypeIndex))
                                {
                                    MessageType = reader.GetString(MessageTypeIndex);
                                }
                                if (!reader.IsDBNull(ParentMessageNameIndex))
                                {
                                    ParentMessageName = reader.GetString(ParentMessageNameIndex);
                                }
                            }
                        }
                    }
                }
                catch (SqlException ex)
                {
                   Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.SelectMessage: {0}.", ex.Message));
                }
                catch (Exception ex)
                {
                    Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.SelectMessage: {0}.", ex.Message));
                }
            }
        }
    }

    public class Transaction
    {
        public string MessageId { get; set; }
        public string TransactionId { get; set; }
        public string StartTime { get; set; }
        public string EndTime { get; set; }
        public string Result { get; set; }
        internal void insertTransaction(string connStr)
        {
            string queryString = "INSERT INTO dbo.[Transaction] (MessageId,TransactionId,StartTime) VALUES (@MessageId,@TransactionId,@StartTime)";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        command.Parameters.AddWithValue("@TransactionId", TransactionId ?? "");
                        command.Parameters.AddWithValue("@StartTime", StartTime ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.InsertTransaction: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.InsertTransaction: {0}.", ex.Message));
            }
        }
        internal void updateTransaction(string connStr)
        {
            string queryString = "UPDATE dbo.[Transaction] SET EndTime=@EndTime, Result=@Result WHERE TransactionId=@TransactionId";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@TransactionId", TransactionId ?? "");
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        command.Parameters.AddWithValue("@EndTime", EndTime ?? "");
                        command.Parameters.AddWithValue("@Result", Result ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.UpdateTransaction: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.UpdateTransaction: {0}.", ex.Message));
            }
        }
    }

    public class MessageState
    {
        public string MessageId { get; set; }
        public int RetryCount { get; set; }
        public string State { get; set; }
        internal void updateMessageState(string connStr)
        {
            string queryString = "UPDATE MessageState SET RetryCount=@RetryCount, State=@State WHERE MessageId=@MessageId";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        command.Parameters.AddWithValue("@RetryCount", RetryCount);
                        command.Parameters.AddWithValue("@State", State ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.UpdateMessageState: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.UpdateMessageState: {0}.", ex.Message));
            }
        }
        internal void selectMessageState(string connStr)
        {
            string queryString = "SELECT * FROM MessageState WHERE MessageId=@MessageId;";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        connection.Open();
                        SqlDataReader reader = command.ExecuteReader();
                        while (reader.Read())
                        {
                            int RetryCountIndex = reader.GetOrdinal("RetryCount");
                            int StateIndex = reader.GetOrdinal("State");
                            if (!reader.IsDBNull(RetryCountIndex))
                            {
                                RetryCount = (int)reader.GetSqlInt32(RetryCountIndex);
                            }
                            if (!reader.IsDBNull(StateIndex))
                            {
                                State = reader.GetString(StateIndex);
                            }
                        }
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.SelectMessageState: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.SelectMessageState: {0}.", ex.Message));
            }
        }
        internal void insertMessageState(string connStr)
        {
            string queryString = "INSERT INTO MessageState (MessageId,RetryCount,State) VALUES (@MessageId,@RetryCount,@State)";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@MessageId", MessageId ?? "");
                        command.Parameters.AddWithValue("@RetryCount", RetryCount);
                        command.Parameters.AddWithValue("@State", State ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.InsertMessageState: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.InsertMessageState: {0}.", ex.Message));
            }
        }
    }

    public class Failure
    {
        public string FailureId { get; set; }
        public string TransactionId { get; set; }
        public string Reason { get; set; }
        public string Detail { get; set; }
        internal void insertFailure(string connStr)
        {
            string queryString = "INSERT INTO Failure (FailureId,TransactionId,Reason,Detail) VALUES (@FailureId,@TransactionId,@Reason,@Detail)";
            try
            {
                using (SqlConnection connection = new SqlConnection(connStr))
                {
                    using (SqlCommand command = new SqlCommand(queryString, connection))
                    {
                        command.Parameters.AddWithValue("@FailureId", FailureId ?? Guid.NewGuid().ToString());
                        command.Parameters.AddWithValue("@TransactionId", TransactionId ?? "");
                        command.Parameters.AddWithValue("@Reason", Reason ?? "");
                        command.Parameters.AddWithValue("@Detail", Detail ?? "");
                        connection.Open();
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                Logger.Instance.Error(string.Format("SQL Exception in method MessageBaseExtensions.InsertFailure: {0}.", ex.Message));
            }
            catch (Exception ex)
            {
                Logger.Instance.Error(string.Format("Unexpected Exception in method MessageBaseExtensions.InsertFailure: {0}.", ex.Message));
            }
        }
    }
}