﻿using System;
using System.Collections.Generic;
using VRM.Integration.Servicebus.Core;
using WHH.VEFT.Messages;
using CRMOLConnect;
using CRMOLConnect.Config;
using System.ServiceModel.Description;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Client;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;
using System.Configuration;
//using VRM.Integration.Servicebus.Extensions;

namespace WHH.VEFT.WebApi.Processors
{
    class WHHResolutionProcessor
    {

        public IMessageBase Execute(WHHResolutionRequest request)
        {
            WHHResolutionResponse response = new WHHResolutionResponse()
            {
                ExceptionOccurred = false,
                ExceptionMessage = "",
                //FailureDetails = new List<FailureDetail>()
            };

            using (var OrgServiceProxy = VEIS.Utilities.CRMConnect.ConnectToCRMOL(request.OrganizationName.ToUpper()).Result)
            {
                var fetch = @"<fetch version=""1.0"" output-format=""xml-platform"" mapping=""logical"" distinct=""false"" >
                                  <entity name= ""incident"" >   
                                    <attribute name= ""incidentid"" />    
                                    <filter type=""and"">     
                                    <condition attribute=""patsr_caseguid"" operator=""eq"" value=""";
                fetch += request.CaseId;
                fetch += @"""/>         
                                 </filter>         
                                 </entity>
                                 </fetch>";

                Logger.Instance.Debug("fetch:" + fetch);
                EntityCollection interactions = OrgServiceProxy.RetrieveMultiple(new FetchExpression(fetch));

                if (interactions != null & interactions.Entities != null && interactions.Entities.Count > 0)
                {
                    Entity defaultInteraction = interactions.Entities[0];
                    try
                    {
                        var stateRequest = new SetStateRequest();

                        if (request.Acceptance)
                        {
                            Logger.Instance.Debug("In Acceptance");
                            //open
                            stateRequest.State = new OptionSetValue(0);
                            stateRequest.Status = new OptionSetValue(1);
                            stateRequest.EntityMoniker = new EntityReference("incident", defaultInteraction.Id);
                            SetStateResponse StateResponse = (SetStateResponse)OrgServiceProxy.Execute(stateRequest);
                            var interaction = new Entity("incident");
                            interaction.Id = defaultInteraction.Id;
                            interaction["patsr_whhlresolutiondetails_whhl"] = request.AcceptanceNotes;
                            OrgServiceProxy.Update(interaction);
                            CloseIncidentRequest closeRequest = new CloseIncidentRequest();
                            closeRequest.IncidentResolution = new Entity("incidentresolution");
                            closeRequest.IncidentResolution["subject"] = request.AcceptanceNotes;
                            closeRequest.IncidentResolution["incidentid"] = new EntityReference("incident", defaultInteraction.Id);
                            closeRequest.Status = new OptionSetValue(914980001);
                            CloseIncidentResponse anotherStateResponse = (CloseIncidentResponse)OrgServiceProxy.Execute(closeRequest);
                        }
                        else
                        {
                            Logger.Instance.Debug("In reject");
                            //reject
                            stateRequest.State = new OptionSetValue(0);
                            stateRequest.Status = new OptionSetValue(4);
                            stateRequest.EntityMoniker = new EntityReference("incident", defaultInteraction.Id);
                            SetStateResponse StateResponse = (SetStateResponse)OrgServiceProxy.Execute(stateRequest);
                            var interaction = new Entity("incident");
                            interaction.Id = defaultInteraction.Id;
                            interaction["patsr_whhlresolutiondetails_whhl"] = request.AcceptanceNotes;
                            OrgServiceProxy.Update(interaction);
                        }
                    }
                    catch (Exception e)
                    {
                        response.ExceptionOccurred = true;
                        response.ExceptionMessage = "Internal Error: " + e.ToString();
                        Logger.Instance.Error(e.ToString());
                    }
                }
                else
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Unable to identify an interaction corresponding to provided CaseId.";
                    Logger.Instance.Debug($"Unable to identify an interaction corresponding to CaseId: {request.CaseId}");
                }
                return response;
            }
        }
    }
}
