﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using Microsoft.Xrm.Sdk;
using WHH.VEFT.Messages;
using WHH.VEFT.WebApi;
using System.Net;
using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;
using VRM.Integration.Servicebus.Core;
using CRMOLConnect;
using CRMOLConnect.Config;
using System.ServiceModel.Description;
using Microsoft.Xrm.Sdk.Client;
using WHH.VEFT.WebApi.Configuration;
using System.Threading.Tasks;
using Microsoft.Crm.Sdk.Messages;
using System.Security.Cryptography.X509Certificates;

namespace WHH.VEFT.WebApi.Processors
{
    public class WHHUpdateStatusProcessor
    {
        private static int _defaultTimeout = 20;
        //Parameters for connecting to WHH
        private static string _WHHString =  VEFTSecurityConfiguration.Current.WHH_URI;
        private static string _ClientCertThumbprint = VEFTSecurityConfiguration.Current.ClientCertThumbprint;
        private static string _ClientCertName = VEFTSecurityConfiguration.Current.ClientCertName;
        private static bool _UseMutualAuth = VEFTSecurityConfiguration.Current.UseMutualAuth;

        private static WebServiceDetails _WHHwsd = new WebServiceDetails()
        {
            TargetURL = _WHHString,
            WSUserName = "",
            Password = ""
        };
        private static Uri _WHHUri = new Uri(_WHHString);

        public IMessageBase Execute(WHHUpdateStatusRequest request)
        {
            Logger.Instance.Debug("Execute StatusUpdate");
            WHHUpdateStatusResponse response = new WHHUpdateStatusResponse()
            {
                ExceptionOccurred = false,
                ExceptionMessage = "",
                //FailureDetails = new List<FailureDetail>()
            };

            #region test
            //Testing purposes
            //try
            //{
            //    using (HttpClient client = new HttpClient())
            //    {
            //        var res = client.GetAsync(_WHHUri).Result;//.ConfigureAwait(false);//.Result)ConfigureAwait(false);//..Result;//.ConfigureAwait(false);
            //        var responseValue = res.Content.ReadAsStringAsync().Result;
            //        Logger.Instance.Error("Got response value");
            //        Logger.Instance.Error(responseValue);

            //        //var r = result.ToString();//.Result.ToString();
            //        //string checkResult = result.ToString();
            //    }
            //}
            //catch (Exception e)
            //{
            //    response.ExceptionMessage = e.InnerException.Message;//.Message;
            //    Logger.Instance.Error(response.ExceptionMessage);
            //}
            //return response;
            #endregion
            #region validate data elements
            if (string.IsNullOrEmpty(request.CaseId))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing Case id";
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing CaseId"
                //});
            }
            #region old
            //else 
            //if (!Guid.TryParse(request.CaseId, out interactionId))
            //{
            //    response.ExceptionOccurred = true;
            //    response.ExceptionMessage = "Data Validation: Invalid CaseId";
            //    return response;
            //    //response.FailureDetails.Add(new FailureDetail()
            //    //{
            //    //    Reason = FailureReason.DataValidation,
            //    //    Details = "Invalid CaseId"
            //    //});
            //}
            #endregion
            if (string.IsNullOrEmpty(request.Status))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing Status";
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing Status"
                //});
            }
            else if (request.Status != "Resolved")
            {
                //Only pass to WHH for Inactive, no other statuses
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Only pass to WHH for Inactive, no other statuses";
                return response;
            }

            //if (response.FailureDetails != null && response.FailureDetails.Count > 0)
            //{
            //    response.ExceptionMessage = "Data Validation Error";
            //    response.ExceptionOccurred = true;
            //    return response;
            //}
            #endregion
            try
            {

                string resolvedOn;
                DateTime modifiedOn = DateTime.MinValue;
                var success = DateTime.TryParse(request.ModifiedOn, out modifiedOn);
                if (modifiedOn == DateTime.MinValue)
                    resolvedOn = "";
                else
                    resolvedOn = request.ModifiedOn;

                Logger.Instance.Debug($"UpdatingSFDC CaseId: {request.CaseId}");

                var whhRequest = new WHHRequest()
                {
                    //id = Guid.NewGuid().ToString(),
                    sfCaseId = request.CaseId,
                    patsCaseId = request.SurveyID,
                    patsStatus = request.Status,
                    patsResolvedBy = request.ModifiedBy ?? "Unknown",
                    patsResolvedByDateTime = request.ModifiedOn,//.OrganizationName,//resolvedOn,
                    patsResolvedDetails = request.DispositionNotes ?? request.Status
                };
                //if (interactionQuery != null)
                //{
                //    whhRequest.patsResolvedBy = interactionQuery.GetAttributeValue<EntityReference>("modifiedby").Name;
                //    whhRequest.patsResolvedDateTime = interactionQuery.GetAttributeValue<DateTime>("modifiedon").ToString();
                //    //TODO: add in Resolution details once added to Retrieve above
                //}
                //else
                //{
                //    //Log
                //}

                //Send resolution request
                Logger.Instance.Error(_WHHUri.ToString());
                response = sendReceive(_WHHUri, _WHHwsd, whhRequest, response);

                if(response == null)
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Error while sending request";
                    return response;
                }
                return response;
            }
            catch (TimeoutException ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Timeout during WHH Status Update {ex.Message}";
                Logger.Instance.Error(response.ExceptionMessage);
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.Timeout,
                //    Details = ex.Message
                //});
                return response;
            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception during WHH Status Update {ex.Message}";
                Logger.Instance.Error(response.ExceptionMessage);
                if (ex.InnerException != null)
                    Logger.Instance.Error(ex.InnerException);
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
                return response;
            }
        }

        private WHHUpdateStatusResponse sendReceive(Uri _WHHUri, WebServiceDetails _WHHwsd, WHHRequest whhRequest, WHHUpdateStatusResponse response, WebServiceDetails serviceDetails = null)
        {
            string message = string.Empty;
            //MessagePayload messagePayload;
            Logger.Instance.Info("In sendReceive");
            try
            {
                if ((System.Net.ServicePointManager.SecurityProtocol & SecurityProtocolType.Tls12) == 0)
                {
                    System.Net.ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls | SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
                }
                HttpClient client = null;

                //Read the certificate using thumbprint
                if (_UseMutualAuth)
                {
                    try
                    {
                        X509Certificate2 clientCert = WebApiUtility.GetClientCertificateByThumbprint(_ClientCertThumbprint);
                        Web​Request​Handler requestHandler = new WebRequestHandler();
                        requestHandler.ClientCertificates.Add(clientCert);
                        client = new HttpClient(requestHandler);
                    }
                    catch (Exception e)
                    {
                        Logger.Instance.Error("Error reading certificate" + e.Message);
                    }
                }
                else
                {
                    client = new HttpClient();
                }

                // ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
                //using (HttpClient client = new HttpClient())
                if (client != null)
                {
                    using (client)
                    using (MemoryStream memStream = ObjectToJSonStream(whhRequest))
                    //using (StreamContent sc = new StreamContent(memStream))
                    using (StreamReader sr = new StreamReader(memStream))
                    {
                        StringContent theContent = new StringContent(sr.ReadToEnd(), System.Text.Encoding.UTF8, "application/json");

                        client.Timeout = TimeSpan.FromSeconds(_defaultTimeout * 1000);

                        if (serviceDetails != null)
                        {
                            var byteArray = Encoding.ASCII.GetBytes(string.Format("{0}:{1}", serviceDetails.WSUserName, serviceDetails.Password));
                            client.DefaultRequestHeaders.Authorization = new System.Net.Http.Headers.AuthenticationHeaderValue("Basic", Convert.ToBase64String(byteArray));
                        }
                        client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                        //  BypassCertificateError();
                        //var scStr = sc.ReadAsStringAsync().Result;
                        HttpResponseMessage serviceResponse = client.PostAsync(_WHHUri, theContent).Result;//was sc
                        serviceResponse.EnsureSuccessStatusCode();
                        var responseValue = serviceResponse.Content.ReadAsStringAsync().Result;

                        //messagePayload = DeserializeResponse<MessagePayload>(responseValue);
                    }
                }
                else
                {
                    response.ExceptionOccurred = true;
                    response.ExceptionMessage = "Internal Error";
                    Logger.Instance.Error("HTTP client is NULL");
                }                
                    
                return response;
            }
            catch (WebException ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                Logger.Instance.Error(ex.Message);
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.SecurityException,
                //    Details = ex.Message
                //});
                return response;
            }
            catch (TimeoutException ex)
            {
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.Timeout,
                //    Details = ex.Message
                //});
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                Logger.Instance.Error(ex.Message);
                return response;
            }
            catch (Exception ex)
            {
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
                response.ExceptionOccurred = true;
                response.ExceptionMessage = ex.Message;
                Logger.Instance.Error(ex.Message);
                return response;
            }

            //var retObj = DeserializeBase64Response<T>(messagePayload.Message);
            //return retObj;
            //return null;
        }

        private static MemoryStream ObjectToJSonStream(object obj)
        {
            MemoryStream memStream = new MemoryStream();

            DataContractJsonSerializer ser = new DataContractJsonSerializer(obj.GetType());

            ser.WriteObject(memStream, obj);

            memStream.Position = 0;
            return memStream;
        }
    }

    public class WHHRequest
    {
        //public string id { get; set; }
        public string sfCaseId { get; set; }
        public string patsStatus { get; set; }
        public string patsCaseId { get; set; }
        public string patsResolvedDetails { get; set; }
        public string patsResolvedBy { get; set; }
        public string patsResolvedByDateTime { get; set; }
    }
}
