﻿using System;
using System.Collections.Generic;
using Microsoft.Xrm.Sdk;
using WHH.VEFT.Messages;
using CRMOLConnect;
using Microsoft.Xrm.Sdk.Client;
using CRMOLConnect.Config;
using VRM.Integration.Servicebus.Core;
using System.ServiceModel.Description;
using Microsoft.Crm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Discovery;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Net.Security;
using Microsoft.Xrm.Sdk.Query;
using System.Configuration;

namespace WHH.VEFT.Processors
{
    class WHHCreateInteractionProcessor
    {
        private Entity interaction = new Entity("incident");
        //This will come from case id, assign interaction to case id
        //make processor like upsert
        //Subsequest do update using case id/interaction id

        public IMessageBase Execute(WHHCreateInteractionRequest request)
        {
            WHHCreateInteractionResponse response = new WHHCreateInteractionResponse()
            {
                ExceptionOccurred = false,
                ExceptionMessage = "",
                //FailureDetails = new List<FailureDetail>()
            };

            #region validate data elements
            if (string.IsNullOrEmpty(request.CaseId))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing CaseID";
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing CaseId"
                //});
            }
            //else
            //if (!Guid.TryParse(request.CaseId, out interactionId))
            //{
            //    response.ExceptionOccurred = true;
            //    response.ExceptionMessage = "Data Validation: Invalid CaseID";
            //    return response;
            //    //response.FailureDetails.Add(new FailureDetail()
            //    //{
            //    //    Reason = FailureReason.DataValidation,
            //    //    Details = "Invalid CaseId"
            //    //});
            //}

            if (request.CaseNumber == 0)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing/Invalid CaseNumber";
                Logger.Instance.Warn($"DataValidation: CaseNumber {request.CaseNumber}");
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing/Invalid CaseNumber"
                //});
            }

            if (string.IsNullOrEmpty(request.VIEWS_Facility__c))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing Facility";
                Logger.Instance.Warn($"DataValidation: Facility {request.VIEWS_Facility__c}");
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing Facility"
                //});
            }

            if (string.IsNullOrEmpty(request.WHVAHotline_Comments__c))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Comments";
                Logger.Instance.Warn($"DataValidation: Comments {request.WHVAHotline_Comments__c}");
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing Comments"
                //});
            }

            if (string.IsNullOrEmpty(request.VIEWS_Relationship_to_Veteran_))
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = "Data Validation: Missing Relationship to Veteran";
                Logger.Instance.Warn($"DataValidation: Relationship {request.VIEWS_Relationship_to_Veteran_}");
                return response;
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.DataValidation,
                //    Details = "Missing Relationship to Veteran"
                //});
            }

            //if (response.FailureDetails != null && response.FailureDetails.Count > 0)
            //{
            //    response.ExceptionMessage = "Data Validation Error";
            //    response.ExceptionOccurred = true;
            //    return response;
            //}


            #endregion

            #region Create Interaction
            //validated above
            interaction["patsr_caseguid"] = request.CaseId;
            interaction["patsr_surveyid"] = request.CaseNumber;
            //DateTime values
            addDate(request.CreatedDate, "patsr_dateofcontact");
            addDate(request.Birthdate, "veo_dateofbirth");
            addDate(request.WHVAHotline_SLA_Start_Date_Tim, "patsr_whhlSLAStartDate_whhl");
            addDate(request.VIEWS_SLA_End_Date_DateTime__c, "patsr_whhlSLATargetDate_whhl");
            addDate(request.Internal_OCR_DueDate, "patsr_ocrinternalduedate");
            //Strings
            addString(request.VIEWS_Facility__c, "patsr_divison");
            addString(request.FirstName, "veo_firstname");
            addString(request.LastName, "veo_lastname");
            addString(request.Email, "veo_veteranemail");
            addString(request.Phone, "patsr_callbacknumber");
            addString(request.OCR_Notes, "patsr_whhldescription_whhl");
            addString(request.SENSITIVE_WHVAHotline_Subject__c, "patsr_whhlsubject_whhl");
            addString(request.VIEWS_Tier_1_Complaint_Category__c, "patsr_whhlVAHotlineComplaintCategory_whhl");
            addString(request.VIEWS_Complaint_Category_Description__c, "patsr_whhlcomplaintcategorydescription_whhl");
            addString(request.Short_Description__c, "patsr_whhlothercomplainttype_whhl");
            addString(request.Owner, "patsr_whhlCaseOwner_whhl");
            addString(request.LastModifiedBy, "patsr_whhllastmodifiedby_whhl");
            addString(request.Status, "patsr_whhlStatus_whhl");
            addString(request.WH_VA_Hotline_Reason_Priority__c, "patsr_whhlreasonpriorityonewasremoved_whhl");
            addString(request.Secondary_Phone__c, "patsr_whhlsecondaryphone_whhl");
            addString(request.WHHotline_Other_Location__c, "patsr_whhlOtherLocation_whhl");
            addString(request.Views_Administration__c, "patsr_whhladministration_whhl");
            addString(request.Secondary_Email__c, "patsr_whhlpreferredemail_whhl");
            addString(request.Social, "veo_lastfourssn");
            addString(request.Email_Proxy, "patsr_whhlemailproxy_whhl");
            addString(request.SecondaryPhone_Proxy, "patsr_whhlsecondaryphoneproxy_whhl");
            addString(request.WHVAHotline_Comments__c, "veo_description");
            //Integers
            interaction["patsr_whhlslabusinessdays_whhl"] = request.WHVAHotline_SLA_Business_Days__c;
            interaction["patsr_whhldaysoverdue_whhl"] = request.Days_Until_SLA_Overdue__c;
            interaction["patsr_whhlpriorityoneindicator_whhl"] = request.PriorityOne_Indicator;
            //Optionsets
            if (request.PriorityOne_Indicator == true)
            {
                interaction["patsr_prioritycode"] = new OptionSetValue(100000001);
                Logger.Instance.Info("Priority Inidcator true" + request.PriorityOne_Indicator.ToString());
            }
            else
            {
                interaction["patsr_prioritycode"] = new OptionSetValue(100000003);
                Logger.Instance.Info("Priority Inidcator false" + request.PriorityOne_Indicator.ToString());
            }
            if (!string.IsNullOrEmpty(request.RecordType))
            {
                interaction["patsr_feedbacktype"] = request.RecordType;
            }
            else
            {
                interaction["patsr_whhlveftinteractionpriority"] = new OptionSetValue(100000000);
                Logger.Instance.Info("Request.RecordType is Null or Empty");
            }
            interaction["patsr_casetype"] = new OptionSetValue(100000000);
            interaction["veo_contactmethod"] = new OptionSetValue(100000012);
            interaction["patsr_lob"] = new OptionSetValue(100000000);
            //Veteran Identity
            if (request.Anonymous__c)
            {
                interaction["patsr_contactingentity"] = new OptionSetValue(100000010);
                interaction["customerid"] = new EntityReference("contact", new Guid("{EBEDDB33-190D-E811-945F-0050568D743D}"));
            }
            else
                interaction["patsr_contactingentity"] = new OptionSetValue(100000000);
            if (!string.IsNullOrEmpty(request.ICN))
            {
                Logger.Instance.Info($"ICN: {request.ICN}");
                addString(request.ICN, "patsr_whhlicn");
                Logger.Instance.Info($"ICN: {interaction.GetAttributeValue<string>("patsr_whhlicn")}");
            }
            else
            {
                interaction["customerid"] = new EntityReference("contact", new Guid("{EBEDDB33-190D-E811-945F-0050568D743D}"));
                Logger.Instance.Info($"ICN Null Or Empty");
            }
            #endregion

            try
            {
                var debug = false;
                if (!debug)
                {
                    using (var OrgServiceProxy = VEIS.Utilities.CRMConnect.ConnectToCRMOL(request.OrganizationName.ToUpper()).Result)
                    {
                        //Logger.Instance.Error(userId.ToString());
                        Logger.Instance.Debug($"OrgSvc");


                        //Upsert operation
                        //retrieve record using case id
                        Guid FTPInteraction = new Guid(); ;// = null;
                        //Retrieve mult
                        var fetch = @"<fetch version=""1.0"" output-format=""xml-platform"" mapping=""logical"" distinct=""false"" >
                                      <entity name= ""incident"" >   
                                        <attribute name= ""incidentid"" />    
                                        <filter type=""and"">     
                                        <condition attribute=""ftp_caseguid_text"" operator=""eq"" value=""";
                        fetch += request.CaseId;
                        fetch += @"""/>         
                                 </filter>         
                                 </entity>
                                 </fetch>";

                        Logger.Instance.Debug("fetch:" + fetch);
                        EntityCollection interactions = OrgServiceProxy.RetrieveMultiple(new FetchExpression(fetch));

                        if (interactions != null & interactions.Entities != null && interactions.Entities.Count > 0)
                        {
                            Entity defaultInteraction = interactions.Entities[0];
                            interaction.Id = defaultInteraction.Id;
                            Logger.Instance.Debug("Before Update, Id:" + interaction.Id.ToString());
                            response.InteractionID = interaction.Id.ToString();
                            //Do update operation

                            OrgServiceProxy.Update(interaction);
                            //logic here
                        }
                        else
                        {
                            //Do insert operation
                            Logger.Instance.Debug("Before Interaction Create");

                            FTPInteraction = OrgServiceProxy.Create(interaction);
                            if (FTPInteraction != null)
                            {
                                Logger.Instance.Debug($"Interaction created: {FTPInteraction.ToString()}");
                                response.InteractionID = FTPInteraction.ToString();
                            }
                            else
                            {
                                response.ExceptionOccurred = true;
                                response.ExceptionMessage = "Exception during Interaction creation. Null value returned";
                                Logger.Instance.Error(response.ExceptionMessage);
                                //response.FailureDetails.Add(new FailureDetail()
                                //{
                                //    Reason = FailureReason.OtherException,
                                //    Details = "CRM create failure: No interaction created."
                                //});
                            }
                        }
                    }
                }
                //response.InteractionID = interactionId.ToString();
                return response;
            }
            catch (TimeoutException ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Timeout during Interaction creation {ex.Message}";
                Logger.Instance.Error(response.ExceptionMessage);
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.Timeout,
                //    Details = ex.Message
                //});
            }
            catch (Exception ex)
            {
                response.ExceptionOccurred = true;
                response.ExceptionMessage = $"Exception during Interaction creation {ex.Message}";
                Logger.Instance.Error(response.ExceptionMessage);
                //response.FailureDetails.Add(new FailureDetail()
                //{
                //    Reason = FailureReason.UnexpectedException,
                //    Details = ex.Message
                //});
            }

            return response;
        }

        private void addDate(string date, string dest)
        {
            DateTime val = new DateTime();
            if (!DateTime.TryParse(date, out val))
            {
                Logger.Instance.Warn($"Unable to parse value {date} to {dest}");
            }
            else interaction[dest] = val;
        }

        private void addString(string val, string dest)
        {
            if(!string.IsNullOrEmpty(val))
            {
                interaction[dest] = val;
            }
        }
    }
}
