using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Collections.ObjectModel;
using VRM.Utility;
using VRMRest.WebApi;

namespace VRM.VEFT.Plugins
{
	public class VEFTInteractionStatusChange : IPlugin
	{
		private string _dacUriString = null;

		private Uri _dacUri = null;

		private WebServiceDetails _dacWsd = null;

		private string UName = null;

		private string PW = null;

		private string URL = null;

		public VEFTInteractionStatusChange()
		{
		}

		public void Execute(IServiceProvider serviceProvider)
		{
			string[] strArrays;
			Exception exception;
			string str;

			IOrganizationServiceFactory service = (IOrganizationServiceFactory)serviceProvider.GetService(typeof(IOrganizationServiceFactory));
			IPluginExecutionContext pluginExecutionContext = (IPluginExecutionContext)serviceProvider.GetService(typeof(IPluginExecutionContext));
			IOrganizationService organizationService = service.CreateOrganizationService(new Guid?(pluginExecutionContext.UserId));
			ITracingService tracingService = (ITracingService)serviceProvider.GetService(typeof(ITracingService));

			tracingService.Trace("getting entity image.", new object[0]);

			Entity postImage = pluginExecutionContext.PostEntityImages["entity"];
			tracingService.Trace("got entity image.", new object[0]);

			string strpostImage = "MISSING";

            //get modified by name either from post image or current entity
			if (postImage.Contains("modifiedby"))
			{
				strpostImage = (postImage.Contains("modifiedby") ? postImage.GetAttributeValue<EntityReference>("modifiedby").Name : "MISSING");
			}
			else
			{
				Guid id = postImage.Id;
				strArrays = new string[] { "modifiedby" };
				Entity entity = organizationService.Retrieve("incident", id, new ColumnSet(strArrays));//REQUEST
				strpostImage = (entity.Contains("modifiedby") ? entity.GetAttributeValue<EntityReference>("modifiedby").Name : "MISSING");
			}

            //use name to get system user if still missing
			if ((!string.IsNullOrEmpty(strpostImage) ? false : postImage.Contains("modifiedby")))
			{
				Guid guid = ((EntityReference)postImage["modifiedby"]).Id;
				strArrays = new string[] { "fullname" };
				Entity entSysUser = organizationService.Retrieve("systemuser", guid, new ColumnSet(strArrays));
				strpostImage = (string)entSysUser["fullname"];
			}

			tracingService.Trace(string.Concat("modifiedByString: ", strpostImage), new object[0]);

			string strStatusCode = postImage.FormattedValues["statuscode"].ToString();
			string strSurveyId = postImage.GetAttributeValue<int>("patsr_surveyid").ToString();
			string attributeValue = postImage.GetAttributeValue<string>("description");

			OptionSetValue osvContactMethod = (OptionSetValue)postImage["veo_contactmethod"];

			if (((osvContactMethod.Value != 100000006 /*VEFT*/|| !(strStatusCode == "Assigned") && 
                !(strStatusCode == "Inactive")) && 
                (osvContactMethod.Value != 100000012 /*white house hotline*/ || !(strStatusCode == "Resolved")) ? false : strSurveyId != null))
			{
				tracingService.Trace("inside first big IF.", new object[0]);
				QueryByAttribute queryByAttribute = new QueryByAttribute("patsr_setting");
				strArrays = new string[] { "patsr_medalliaapilogin", "patsr_medalliaapipassword", "patsr_medalliaapiurl" };
				queryByAttribute.ColumnSet = new ColumnSet(strArrays);
				DataCollection<string> attributes = queryByAttribute.Attributes;
				strArrays = new string[] { "patsr_name" };
				attributes.AddRange(strArrays);
				queryByAttribute.Values.AddRange(new object[] { "Active Settings" });
				Entity entSettings = new Entity();

				try
				{
					EntityCollection collectedSettings = organizationService.RetrieveMultiple(queryByAttribute);
					if (collectedSettings.Entities.Count == 1)
					{
						entSettings = collectedSettings.Entities[0];
						this.UName = (string)entSettings["patsr_medalliaapilogin"];
						this.PW = ApiUtility.DecryptCredential((string)entSettings["patsr_medalliaapipassword"], 7);
						this.URL = (string)entSettings["patsr_medalliaapiurl"];
					}
				}
				catch (Exception exception1)
				{
					exception = exception1;
					tracingService.Trace("error retrieving Active Settings.", new object[0]);
					throw exception;
				}

				this._dacUri = this.getVIMTURI(organizationService);

				WebServiceDetails webServiceDetail = new WebServiceDetails()
				{
                    TargetURL = _dacUriString,
					WSUserName = UName,
					Password = PW
				};

				this._dacWsd = webServiceDetail;

				if (osvContactMethod.Value == 100000006)//VEFT
				{
					UpdateStatusRequest updateStatusRequest = new UpdateStatusRequest()
					{
						SurveyID = strSurveyId,
						Status = strStatusCode
					};

					UpdateStatusRequest updateStatusRequest1 = updateStatusRequest;
					tracingService.Trace("WebServiceDetails setup", new object[0]);

					try
					{
						WebApiUtility.SendReceive<UpdateStatusResponse>(this._dacUri, "MedalliaUpdateStatusRequest", updateStatusRequest1, null, this._dacWsd);
					}
					catch (Exception exception2)
					{
						exception = exception2;
						throw new InvalidPluginExecutionException(string.Format("Exception occurred while transmitting to DAC Server: {0}", exception.Message));
					}
				}
				else if (osvContactMethod.Value == 100000012)//white house hotline
                {
					tracingService.Trace("interaction is WHH.", new object[0]);
					string str3 = (postImage.Contains("patsr_caseguid") ? (string)postImage["patsr_caseguid"] : string.Empty);
					string str4 = (postImage.Contains("ftp_whhresolutiondetails_text") ? (string)postImage["patsr_whhResolutionDetails"] : string.Empty);
					if (string.IsNullOrEmpty(str3))
					{
						tracingService.Trace("missing caseGuidText.", new object[0]);
					}
					else
					{
						tracingService.Trace("have caseGuidText.", new object[0]);
						tracingService.Trace(string.Concat("modifiedby: ", strpostImage), new object[0]);

						WHHUpdateStatusRequest wHHUpdateStatusRequest = new WHHUpdateStatusRequest()
						{
							CaseId = str3,
							SurveyID = attributeValue,
							Status = strStatusCode,
							DispositionNotes = str4,
							ModifiedBy = strpostImage
						};

						WHHUpdateStatusRequest wHHUpdateStatusRequest1 = wHHUpdateStatusRequest;

						if (postImage.Contains("modifiedon"))
						{
							DateTime universalTime = (DateTime)postImage["modifiedon"];
							universalTime = universalTime.ToUniversalTime();
							str = universalTime.ToString("o");
						}
						else
						{
							str = DateTime.UtcNow.ToString("o");
						}

						wHHUpdateStatusRequest1.ModifiedOn = str;

						MessageLogging messageLogging = new MessageLogging()
						{
							OrganizationName = pluginExecutionContext.OrganizationName,
							MaxRetry = -1,
							MessageName = "WHHUpdateStatusRequest",
							MessageType = "LOB",
							ParentMessageName = "VEFTInteractionStatusChange",
							IgnoreState = true
						};

						wHHUpdateStatusRequest.MessageLogging = messageLogging;
						wHHUpdateStatusRequest.MessageId = Guid.NewGuid().ToString();
						WHHUpdateStatusRequest wHHUpdateStatusRequest2 = wHHUpdateStatusRequest;
						WHHUpdateStatusResponse wHHUpdateStatusResponse = new WHHUpdateStatusResponse();

						//Entity entIntegrationResult = new Entity("patsr_integrationresult");

						//entIntegrationResult["patsr_integrationdate"] = DateTime.Now;
						//entIntegrationResult["patsr_webservice"] = "White House Hotline (out)";
						//entIntegrationResult["patsr_url"] = this._dacUriString;
						//entIntegrationResult["patsr_interactionid"] = postImage.ToEntityReference();
						//entIntegrationResult["patsr_name"] = "WHH Outbound Integration Result";

						try
						{
							tracingService.Trace("ready to send WHHUpdateStatusRequest.", new object[0]);
							wHHUpdateStatusResponse = WebApiUtility.SendReceive<WHHUpdateStatusResponse>(this._dacUri, "WHHUpdateStatusRequest", wHHUpdateStatusRequest2, null, this._dacWsd);
							tracingService.Trace("got whhRes object.", new object[0]);
						}
						catch (Exception exception3)
						{
							exception = exception3;
							//entIntegrationResult["patsr_integrationstatus"] = "ERROR";
							//entIntegrationResult["patsr_errormessage"] = exception.Message.Substring(0, 2000);
							//tracingService.Trace("logging to patsr_integrationresult table.", new object[0]);
							//entIntegrationResult.Id = organizationService.Create(entIntegrationResult);
							throw new InvalidPluginExecutionException(string.Format("WHH: Exception occurred while transmitting to DAC Server: {0}", exception.Message));
						}

						if (!wHHUpdateStatusResponse.ExceptionOccurred)
						{
							//entIntegrationResult["patsr_integrationstatus"] = "OK";
						}
						else
						{
							//entIntegrationResult["patsr_integrationstatus"] = "ERROR";
							//entIntegrationResult["patsr_errormessage"] = wHHUpdateStatusResponse.ExceptionMessage.Substring(0, 2000);
						}

						try
						{
							tracingService.Trace("logging to ftp_integrationresult table.", new object[0]);
							//entIntegrationResult.Id = organizationService.Create(entIntegrationResult);
						}
						catch (Exception exception4)
						{
							exception = exception4;
							tracingService.Trace(string.Concat("error logging to ftp_integrationresult table: ", exception.Message), new object[0]);
						}
					}
				}
			}
		}

		private Uri getVIMTURI(IOrganizationService organizationService)
		{
			Uri uri;
			try
			{
				Uri uri1 = null;
				if (this.URL != null)
				{
					this._dacUriString = this.URL;
					if (this._dacUriString != null)
					{
						uri1 = new Uri(this._dacUriString);
					}
					if (uri1 == null)
					{
						throw new InvalidPluginExecutionException("No VIMT URL value found in settings entity. Please confirm settings are correctly configured.");
					}
				}
				uri = uri1;
			}
			catch (Exception exception)
			{
				throw new InvalidPluginExecutionException(string.Format("Exception occurred while reading settings entity: {0}\n\nPlease confirm settings entity exists in this environment.", exception.Message));
			}
			return uri;
		}
	}
}