using Microsoft.SharePoint.Client;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq.Expressions;
using System.Net;
using System.Text;
using System.Web;
using VRM.SharepointAPI;
using VRM.Utility;

namespace VRM.VEFT.Plugins
{
	public class UploadAttachment : IPlugin
	{
		private string _secureConfig = null;

		private string _unsecureConfig = null;

		public UploadAttachment(string unsecureConfig, string secureConfig)
		{
			this._secureConfig = secureConfig;
			this._unsecureConfig = unsecureConfig;
		}

		public void Execute(IServiceProvider serviceProvider)
		{
			ITracingService service = (ITracingService)serviceProvider.GetService(typeof(ITracingService));
			IPluginExecutionContext pluginExecutionContext = (IPluginExecutionContext)serviceProvider.GetService(typeof(IPluginExecutionContext));
			IOrganizationServiceFactory organizationServiceFactory = (IOrganizationServiceFactory)serviceProvider.GetService(typeof(IOrganizationServiceFactory));
			IOrganizationService organizationService = organizationServiceFactory.CreateOrganizationService(new Guid?(pluginExecutionContext.UserId));
			service.Trace("Entered the UploadAttachment Execute", new object[0]);
			if (!pluginExecutionContext.InputParameters.Contains("Target"))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the UploadAttachment plugin: {0}", "Parameter Target was not found."));
			}
			service.Trace("Target exists", new object[0]);
			if ((!pluginExecutionContext.InputParameters.Contains("FileName") || !pluginExecutionContext.InputParameters.Contains("FileBody") ? true : !pluginExecutionContext.InputParameters.Contains("Title")))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the UploadAttachment plugin: {0}", "Parameters FileName, FileBody, or Title were not found."));
			}
			service.Trace("InputParmeters exist", new object[0]);
			string item = (string)pluginExecutionContext.InputParameters["FileName"];
			string str = (string)pluginExecutionContext.InputParameters["FileBody"];
			string item1 = (string)pluginExecutionContext.InputParameters["Title"];
			if ((!(item != "") || !(str != "") ? true : !(item1 != "")))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the UploadAttachment plugin: {0}", "The data for parameter FileName, FileBody, or Title is missing."));
			}
			string str1 = "";
			string str2 = "";
			NetworkCredential networkCredential = new NetworkCredential();
			QueryByAttribute queryByAttribute = new QueryByAttribute("mcs_setting");
			string[] strArrays = new string[] { "patsr_sharepointpassword_text", "patsr_sharepointusername_text", "patsr_sharepointsiteurl_text", "patsr_sharepointlist_text" };
			queryByAttribute.ColumnSet = (new ColumnSet(strArrays));
			DataCollection<string> attributes = queryByAttribute.Attributes;
			strArrays = new string[] { "mcs_name" };
			attributes.AddRange(strArrays);
			queryByAttribute.Values.AddRange(new object[] { "Active Settings" });
			foreach (Entity entity in organizationService.RetrieveMultiple(queryByAttribute).Entities)
			{
				str1 = entity.Attributes["patsr_sharepointsiteurl_text"].ToString();
				str2 = entity.Attributes["patsr_sharepointlist_text"].ToString();
				networkCredential.UserName = entity.Attributes["patsr_sharepointusername_text"].ToString();
				networkCredential.Password = ApiUtility.DecryptCredential(entity.Attributes["patsr_sharepointpassword_text"].ToString(), 5);
			}
			if ((!(networkCredential.UserName != "") || !(networkCredential.Password != "") || !(str2 != "") ? true : !(str1 != "")))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the UploadAttachment plugin: {0}", "Sharepoint settings not available."));
			}
			service.Trace("Retrieved settings", new object[0]);
			byte[] numArray = Convert.FromBase64String(str);
			service.Trace("Converted document to byte array", new object[0]);
			strArrays = new string[] { str1, "/", str2, "/", HttpUtility.UrlEncode(item, Encoding.UTF8) };
			string str3 = string.Concat(strArrays);
			service.Trace(string.Concat("Destination: ", str3), new object[0]);
			try
			{
                //call to web api and not sharepoint


				ClientContext clientContext = new ClientContext(str1);
				try
				{
					SharepointApiUtility.SetupRequestAuthentication(clientContext, networkCredential);
					service.Trace("Getting Sharepoint Documents List...", new object[0]);
					List byTitle = clientContext.Web.Lists.GetByTitle(str2);
					service.Trace("Creating FileCreationInformation class...", new object[0]);
					FileCreationInformation fileCreationInformation = new FileCreationInformation()
					{
						ContentStream = new MemoryStream(numArray),
						Overwrite = true,
						Url = str3
					};
					Microsoft.SharePoint.Client.File file = byTitle.RootFolder.Files.Add(fileCreationInformation);
					service.Trace("Successfully uploaded document to sharepoint", new object[0]);
					ListItem listItemAllFields = file.ListItemAllFields;
					listItemAllFields["Title"] = item1;
					listItemAllFields.Update();
					service.Trace("Updated Title of document", new object[0]);
					clientContext.Load<ListItem>(listItemAllFields, new Expression<Func<ListItem, object>>[0]);
					clientContext.ExecuteQuery();
					Guid guid = (Guid)listItemAllFields["UniqueId"];
					service.Trace(string.Concat("Got SharepointID: ", guid.ToString()), new object[0]);
					pluginExecutionContext.OutputParameters["SharepointID"] = guid.ToString();
					pluginExecutionContext.OutputParameters["FilePath"] = str3;
					Entity entity1 = new Entity("patsr_attachment");
					entity1.Id = ((EntityReference)pluginExecutionContext.InputParameters["Target"]).Id;
					entity1.Attributes["patsr_sharepointid_text"] = guid.ToString();
					entity1.Attributes["patsr_filepath_text"] = str3;
					organizationService.Update(entity1);
				}
				finally
				{
					if (clientContext != null)
					{
						((IDisposable)clientContext).Dispose();
					}
				}
			}
			catch (Exception exception1)
			{
				Exception exception = exception1;
				service.Trace(string.Concat("Exception on File Upload: ", exception.Message), new object[0]);
				pluginExecutionContext.OutputParameters["SharepointID"] = "0";
				pluginExecutionContext.OutputParameters["FilePath"] = "ERROR";
				string str4 = string.Concat("Exception on File Upload: ", exception.Message);
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the UploadAttachment plugin: {0}", str4));
			}
		}
	}
}