using Microsoft.SharePoint.Client;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq.Expressions;
using System.Net;
using VRM.SharepointAPI;
using VRM.Utility;

namespace VRM.VEFT.Plugins
{
	public class ViewAttachment : IPlugin
	{
		private string _secureConfig = null;

		private string _unsecureConfig = null;

		public ViewAttachment(string unsecureConfig, string secureConfig)
		{
			this._secureConfig = secureConfig;
			this._unsecureConfig = unsecureConfig;
		}

		public void Execute(IServiceProvider serviceProvider)
		{
			ITracingService service = (ITracingService)serviceProvider.GetService(typeof(ITracingService));
			IPluginExecutionContext pluginExecutionContext = (IPluginExecutionContext)serviceProvider.GetService(typeof(IPluginExecutionContext));
			IOrganizationServiceFactory organizationServiceFactory = (IOrganizationServiceFactory)serviceProvider.GetService(typeof(IOrganizationServiceFactory));
			IOrganizationService organizationService = organizationServiceFactory.CreateOrganizationService(new Guid?(pluginExecutionContext.UserId));
			service.Trace("Entered the ViewAttachment Execute", new object[0]);
			if (!pluginExecutionContext.InputParameters.Contains("Target"))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Parameter Target was not found."));
			}
			service.Trace("Target exists", new object[0]);
			if (!pluginExecutionContext.InputParameters.Contains("SharepointID"))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Parameters SharepointID were not found."));
			}
			service.Trace("InputParmeters exist", new object[0]);
			Guid guid = new Guid(pluginExecutionContext.InputParameters["SharepointID"].ToString());
			if (!(guid.ToString() != ""))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "The data for parameter SharepointID is missing."));
			}
			string str = "";
			string str1 = "";
			NetworkCredential networkCredential = new NetworkCredential();
			QueryByAttribute queryByAttribute = new QueryByAttribute("mcs_setting");
			string[] strArrays = new string[] { "patsr_sharepointpassword_text", "patsr_sharepointusername_text", "patsr_sharepointsiteurl_text", "patsr_sharepointlist_text" };
			queryByAttribute.ColumnSet = new ColumnSet(strArrays);
			DataCollection<string> attributes = queryByAttribute.Attributes;
			strArrays = new string[] { "mcs_name" };
			attributes.AddRange(strArrays);
			queryByAttribute.Values.AddRange(new object[] { "Active Settings" });
			foreach (Entity entity in organizationService.RetrieveMultiple(queryByAttribute).Entities)
			{
				str = entity.Attributes["patsr_sharepointsiteurl_text"].ToString();
				str1 = entity.Attributes["patsr_sharepointlist_text"].ToString();
				networkCredential.UserName = entity.Attributes["patsr_sharepointusername_text"].ToString();
				networkCredential.Password = ApiUtility.DecryptCredential(entity.Attributes["patsr_sharepointpassword_text"].ToString(), 5);
			}
			if ((!(networkCredential.UserName != "") || !(networkCredential.Password != "") || !(str1 != "") ? true : !(str != "")))
			{
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", "Sharepoint settings not available."));
			}
			service.Trace("Retrieved settings", new object[0]);
			try
			{
				ClientContext clientContext = new ClientContext(str);
				try
				{
					SharepointApiUtility.SetupRequestAuthentication(clientContext, networkCredential);
					Microsoft.SharePoint.Client.File fileById = clientContext.Web.GetFileById(guid);
					ClientResult<Stream> clientResult = fileById.OpenBinaryStream();
					clientContext.Load<Microsoft.SharePoint.Client.File>(fileById, new Expression<Func<Microsoft.SharePoint.Client.File, object>>[0]);
					clientContext.ExecuteQuery();
					MemoryStream memoryStream = new MemoryStream();
					try
					{
						if (clientResult != null)
						{
							clientResult.Value.CopyTo(memoryStream);
							string base64String = Convert.ToBase64String(memoryStream.ToArray());
							pluginExecutionContext.OutputParameters["FileStream"] = base64String;
						}
					}
					finally
					{
						if (memoryStream != null)
						{
							((IDisposable)memoryStream).Dispose();
						}
					}
				}
				finally
				{
					if (clientContext != null)
					{
						((IDisposable)clientContext).Dispose();
					}
				}
			}
			catch (Exception exception1)
			{
				Exception exception = exception1;
				service.Trace(string.Concat("Exception on View Attachment: ", exception.Message), new object[0]);
				string str2 = string.Concat("Exception on View Attachment: ", exception.Message);
				throw new InvalidPluginExecutionException(string.Format("An error has occurred in the ViewAttachment plugin: {0}", str2));
			}
		}
	}
}