using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Messages;
using Microsoft.Xrm.Sdk.Query;
using Microsoft.Xrm.Sdk.Workflow;
using System;
using System.Activities;
using System.Runtime.CompilerServices;
using System.Collections.Generic;

namespace VRM.VEFT.Workflow
{
	public class CountActiveChildRecordsForCachedICN : CodeActivity
	{
		[Input("Cached ICN")]
		[ReferenceTarget("patsr_cachedicn")]
		[RequiredArgument]
		public InArgument<EntityReference> inputCachedICN
		{
			get;
			set;
		}

		[Output("Open Child Records")]
		public OutArgument<int> OpenChildRecords
		{
			get;
			set;
		}

		public CountActiveChildRecordsForCachedICN()
		{
		}

		protected override void Execute(CodeActivityContext executionContext)
		{
			//this.OpenChildRecords.Set(executionContext, 9999);

            OpenChildRecords.Set(executionContext, 9999);
            
            ITracingService tracingService = executionContext.GetExtension<ITracingService>();
            IWorkflowContext context = executionContext.GetExtension<IWorkflowContext>();
            IOrganizationService service = ((IOrganizationServiceFactory)executionContext.GetExtension<IOrganizationServiceFactory>()).CreateOrganizationService(context.UserId);

            tracingService.Trace("Starting CountOpenChildRecordsForCachedICN.Execute(), Correlation Id: {0}", context.CorrelationId);

            try
            {
                tracingService.Trace("Beginning");
                //patsr_cachedicn
                Entity contextRecord = new Entity(context.PrimaryEntityName);
                contextRecord.Id = context.PrimaryEntityId;
                EntityReference cachedICNRef = inputCachedICN.Get<EntityReference>(executionContext);
                RetrieveRequest cachedICNRequest = new RetrieveRequest()
                {
                    Target = cachedICNRef,
                    ColumnSet = new ColumnSet("patsr_name"),
                    RelatedEntitiesQuery = new RelationshipQueryCollection()
                    {
                        //new KeyValuePair<Relationship, QueryBase>(
                        //    new Relationship("ftp_ftp_cachedicn_ftp_addendum"),
                        //    new QueryExpression("ftp_addendum")
                        //    {
                        //        ColumnSet = new ColumnSet("activityid", "statecode", "statuscode"),
                        //        Criteria =
                        //        {
                        //            Conditions =
                        //            {
                        //                new ConditionExpression("statecode", ConditionOperator.Equal, "Open")
                        //            }
                        //        }
                        //    }
                        //),
                        new KeyValuePair<Relationship, QueryBase>(
                            //new Relationship("ftp_ftp_cachedicn_ftp_interaction"),
                            new Relationship("patsr_patsr_cachedicn_patsr_request_CachedICN"),
                            new QueryExpression("patsr_request")
                            {
                                ColumnSet = new ColumnSet("activityid", "statecode", "statuscode"),
                                Criteria =
                                {
                                    FilterOperator = LogicalOperator.Or,
                                    Filters =
                                    {
                                        new FilterExpression()
                                        {
                                            Conditions =
                                            {
                                                new ConditionExpression("statecode", ConditionOperator.Equal, 0)
                                            }
                                        },
                                        new FilterExpression()
                                        {
                                            FilterOperator = LogicalOperator.And,
                                            Conditions =
                                            {
                                                //count Inactive:Resolved interactions as 'open' records because they're awaiting acceptance from WHH integration
                                                new ConditionExpression("statecode", ConditionOperator.Equal, 1),
                                                new ConditionExpression("statuscode", ConditionOperator.Equal, 100000004) //Resolved
                                            }
                                        }
                                    }
                                }
                            }
                        ),
                        new KeyValuePair<Relationship, QueryBase>(
                            //new Relationship("ftp_ftp_cachedicn_incident"),
                            new Relationship("patsr_patsr_cachedicn_incident_CachedICN"),
                            new QueryExpression("incident")
                            {
                                ColumnSet = new ColumnSet("incidentid", "statecode", "statuscode"),
                                Criteria =
                                {
                                    Conditions =
                                    {
                                        new ConditionExpression("statecode", ConditionOperator.Equal, 0)
                                    }
                                }
                            }
                        )
                    }
                };

                tracingService.Trace("Going for the precious");

                try
                {
                    RetrieveResponse cachedICNResponse = (RetrieveResponse)service.Execute(cachedICNRequest);
                    int openChildRecordCount = 0;
                    foreach (Relationship thisRelationship in cachedICNResponse.Entity.RelatedEntities.Keys)
                    {
                        openChildRecordCount += ((EntityCollection)cachedICNResponse.Entity.RelatedEntities[thisRelationship]).Entities.Count;
                    }
                    OpenChildRecords.Set(executionContext, openChildRecordCount);
                    tracingService.Trace("Good to go");
                }
                catch (Exception ex1)
                {
                    OpenChildRecords.Set(executionContext, 1); //return > 0
                    tracingService.Trace("Exception 1: " + ex1.Message);
                    throw ex1;
                }
            }
            catch (Exception ex2)
            {
                tracingService.Trace("Exception: {0}", ex2.ToString());
                tracingService.Trace("Exception 2: " + ex2.Message);
                throw ex2;
            }

            tracingService.Trace("Exiting CountOpenChildRecordsForCachedICN.Execute(), Correlation Id: {0}", context.CorrelationId);
        } 
    }
}