using Microsoft.Xrm.Sdk;
using System;
using System.ServiceModel;

namespace VRM.VEFT.Workflow.Helpers
{
	[Serializable]
	public class Logger : ILogger
	{
		private int _sequence = 1;

		private ITracingService _tracingService;

		private IOrganizationService _service;

		private string _method;

		private string _relatedEntityName;

		private Guid _relatedEntityId;

		private string _module;

		private bool _debug;

		private bool _txnTiming;

		private bool _granularTiming;

		public bool setDebug
		{
			set
			{
				this._debug = value;
			}
		}

		public Guid setEntityId
		{
			get
			{
				return this._relatedEntityId;
			}
			set
			{
				this._relatedEntityId = value;
			}
		}

		public string setEntityName
		{
			get
			{
				return this._relatedEntityName;
			}
			set
			{
				this._relatedEntityName = value;
			}
		}

		public bool setGranularTiming
		{
			set
			{
				this._granularTiming = value;
			}
		}

		public string setMethod
		{
			get
			{
				return this._method;
			}
			set
			{
				this._method = value;
			}
		}

		public string setModule
		{
			get
			{
				return this._module;
			}
			set
			{
				this._module = value;
			}
		}

		public IOrganizationService setService
		{
			set
			{
				this._service = value;
			}
		}

		public ITracingService setTracingService
		{
			set
			{
				this._tracingService = value;
			}
		}

		public bool setTxnTiming
		{
			set
			{
				this._txnTiming = value;
			}
		}

		public Logger()
		{
		}

		public void WriteDebugMessage(string message)
		{
			if (this._debug)
			{
				this.writeOutMessage(message, true, false, false);
			}
		}

		public void WriteGranularTimingMessage(string message)
		{
			if (this._granularTiming)
			{
				this.writeOutMessage(message, false, true, false);
			}
		}

		private void writeOutMessage(string message, bool debugMessage, bool granularTiming, bool TxnTiming)
		{
			try
			{
				Entity entity = new Entity("mcs_log");
				entity["mcs_name"] = this._module;
				DateTime now = DateTime.Now;
				string[] str = new string[7];
				int hour = now.Hour;
				str[0] = hour.ToString("00");
				str[1] = ":";
				hour = now.Minute;
				str[2] = hour.ToString("00");
				str[3] = ":";
				hour = now.Second;
				str[4] = hour.ToString("00");
				str[5] = ":";
				hour = now.Millisecond;
				str[6] = hour.ToString("000");
				string str1 = string.Concat(str);
				message = string.Concat(str1, " --", message);
				entity["mcs_errormessage"] = message;
				if (debugMessage)
				{
					entity["mcs_debugmessage"] = true;
				}
				if (granularTiming)
				{
					entity["mcs_grantiming"] = true;
				}
				if (TxnTiming)
				{
					entity["mcs_txntiming"] = true;
				}
				if (!this._relatedEntityId.ToString().StartsWith("000"))
				{
					entity["mcs_entityid"] = this._relatedEntityId.ToString();
				}
				if (this._relatedEntityName != null)
				{
					entity["mcs_entityname"] = this._relatedEntityName;
				}
				entity["mcs_method"] = this._method;
				entity["mcs_sequence"] = this._sequence;
				Logger logger = this;
				logger._sequence = logger._sequence + 1;
				if (this._tracingService != null)
				{
					this._tracingService.Trace(message, new object[0]);
				}
				this._service.Create(entity);
			}
			catch (FaultException<OrganizationServiceFault> faultException)
			{
				return;
			}
			catch (Exception exception)
			{
			}
		}

		private void writeOutMessage(string message, bool debugMessage, bool granularTiming, bool TxnTiming, decimal duration)
		{
			try
			{
				Entity entity = new Entity("mcs_log");
				entity["mcs_name"] = this._module;
				DateTime now = DateTime.Now;
				string[] str = new string[7];
				int hour = now.Hour;
				str[0] = hour.ToString("00");
				str[1] = ":";
				hour = now.Minute;
				str[2] = hour.ToString("00");
				str[3] = ":";
				hour = now.Second;
				str[4] = hour.ToString("00");
				str[5] = ":";
				hour = now.Millisecond;
				str[6] = hour.ToString("000");
				string str1 = string.Concat(str);
				message = string.Concat(str1, " --", message);
				entity["mcs_errormessage"] = message;
				if (debugMessage)
				{
					entity["mcs_debugmessage"] = true;
				}
				if (granularTiming)
				{
					entity["mcs_grantiming"] = true;
				}
				if (TxnTiming)
				{
					entity["mcs_txntiming"] = true;
				}
				if (!this._relatedEntityId.ToString().StartsWith("000"))
				{
					entity["mcs_entityid"] = this._relatedEntityId.ToString();
				}
				if (this._relatedEntityName != null)
				{
					entity["mcs_entityname"] = this._relatedEntityName;
				}
				if (duration != new decimal(0))
				{
					entity["crme_duration"] = duration;
					entity["crme_loglevel"] = new OptionSetValue(935950001);
				}
				entity["mcs_method"] = this._method;
				entity["mcs_sequence"] = this._sequence;
				Logger logger = this;
				logger._sequence = logger._sequence + 1;
				if (this._tracingService != null)
				{
					this._tracingService.Trace(message, new object[0]);
				}
				this._service.Create(entity);
			}
			catch (FaultException<OrganizationServiceFault> faultException)
			{
				return;
			}
			catch (Exception exception)
			{
			}
		}

		public void WriteToFile(string message)
		{
			this.writeOutMessage(message, false, false, false);
		}

		public void WriteTxnTimingMessage(string message)
		{
			if (this._txnTiming)
			{
				this.writeOutMessage(message, false, false, true);
			}
		}

		public void WriteTxnTimingMessage(string message, decimal duration)
		{
			if (this._txnTiming)
			{
				this.writeOutMessage(message, false, false, true, duration);
			}
		}
	}
}