using Microsoft.Xrm.Sdk;
using System;

namespace VRM.VEFT.Workflow.Helpers
{
	public abstract class Runner
	{
		private readonly IServiceProvider _ServiceProvider;

		private IPluginExecutionContext _PluginExecutionContext;

		private IOrganizationServiceFactory _OrganizationServiceFactory;

		private IOrganizationService _OrganizationService;

		private ITracingService _TracingService;

		private VRM.VEFT.Workflow.Helpers.Logger _Logger;

		internal VRM.VEFT.Workflow.Helpers.Logger Logger
		{
			get
			{
				VRM.VEFT.Workflow.Helpers.Logger logger = this._Logger;
				if (logger == null)
				{
					VRM.VEFT.Workflow.Helpers.Logger logger1 = new VRM.VEFT.Workflow.Helpers.Logger()
					{
						setService = this.OrganizationService,
						setTracingService = this.TracingService,
						setModule = string.Format("{0}:", this.GetType())
					};
					VRM.VEFT.Workflow.Helpers.Logger logger2 = logger1;
					VRM.VEFT.Workflow.Helpers.Logger logger3 = logger2;
					this._Logger = logger2;
					logger = logger3;
				}
				return logger;
			}
		}

		internal IOrganizationService OrganizationService
		{
			get
			{
				IOrganizationService organizationService = this._OrganizationService;
				if (organizationService == null)
				{
					IOrganizationService organizationService1 = this.OrganizationServiceFactory.CreateOrganizationService(new Guid?(this.PluginExecutionContext.InitiatingUserId));
					IOrganizationService organizationService2 = organizationService1;
					this._OrganizationService = organizationService1;
					organizationService = organizationService2;
				}
				return organizationService;
			}
		}

		internal IOrganizationServiceFactory OrganizationServiceFactory
		{
			get
			{
				IOrganizationServiceFactory organizationServiceFactory = this._OrganizationServiceFactory;
				if (organizationServiceFactory == null)
				{
					IOrganizationServiceFactory service = (IOrganizationServiceFactory)this.ServiceProvider.GetService(typeof(IOrganizationServiceFactory));
					IOrganizationServiceFactory organizationServiceFactory1 = service;
					this._OrganizationServiceFactory = service;
					organizationServiceFactory = organizationServiceFactory1;
				}
				return organizationServiceFactory;
			}
		}

		internal IPluginExecutionContext PluginExecutionContext
		{
			get
			{
				IPluginExecutionContext pluginExecutionContext = this._PluginExecutionContext;
				if (pluginExecutionContext == null)
				{
					IPluginExecutionContext service = (IPluginExecutionContext)this.ServiceProvider.GetService(typeof(IPluginExecutionContext));
					IPluginExecutionContext pluginExecutionContext1 = service;
					this._PluginExecutionContext = service;
					pluginExecutionContext = pluginExecutionContext1;
				}
				return pluginExecutionContext;
			}
		}

		internal IServiceProvider ServiceProvider
		{
			get
			{
				return this._ServiceProvider;
			}
		}

		internal ITracingService TracingService
		{
			get
			{
				ITracingService tracingService = this._TracingService;
				if (tracingService == null)
				{
					ITracingService service = (ITracingService)this.ServiceProvider.GetService(typeof(ITracingService));
					ITracingService tracingService1 = service;
					this._TracingService = service;
					tracingService = tracingService1;
				}
				return tracingService;
			}
		}

		protected Runner(IServiceProvider serviceProvider)
		{
			if (serviceProvider == null)
			{
				throw new ArgumentNullException("serviceProvider");
			}
			this._ServiceProvider = serviceProvider;
		}

		public abstract Entity GetPrimaryEntity();

		public abstract Entity GetSecondaryEntity();
	}
}