using System;
using System.IO;
using System.Net.Http;
using System.Runtime.CompilerServices;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Web;

namespace VRM.VEFT.Workflow.Helpers
{
	public class Utility
	{
		public const string OneWayPassTest = "TestMessages#OneWayPassTest";

		public const string TwoWayPassTest = "TestMessages#TwoWayPassTest";

		public const string CreateCRMLogEntryRequest = "CRMe#CreateCRMLogEntryRequest";

		private const string _urlRestPath = "/Servicebus/Rest/{0}";

		private const string _urlParams = "?messageId={0}&messageType=text%2Fjson&isQueued=false";

		private const string SEND = "Send";

		private const string SEND_RECEIVE = "SendReceive";

		public Utility()
		{
		}

		private static T DeserializeResponse<T>(string message)
		{
			T t;
			byte[] numArray = Convert.FromBase64String(message);
			UTF8Encoding uTF8Encoding = new UTF8Encoding();
			string str = uTF8Encoding.GetString(numArray);
			string str1 = Regex.Replace(str, "new Date\\(([-+0-9]*)\\)", "\"\\/Date($1)\\/\"");
			MemoryStream memoryStream = new MemoryStream();
			try
			{
				memoryStream.Write(uTF8Encoding.GetBytes(str1), 0, str1.Length);
				memoryStream.Position = (long)0;
				t = (T)(new DataContractJsonSerializer(typeof(T))).ReadObject(memoryStream);
			}
			finally
			{
				if (memoryStream != null)
				{
					((IDisposable)memoryStream).Dispose();
				}
			}
			return t;
		}

		public static Uri FormatUri(Uri baseUri, string method, string messageId)
		{
			string str = string.Format("/Servicebus/Rest/{0}", method);
			string str1 = string.Format("?messageId={0}&messageType=text%2Fjson&isQueued=false", HttpUtility.UrlEncode(messageId));
			Uri uri = new Uri(string.Concat(str, str1), UriKind.Relative);
			return new Uri(baseUri, uri);
		}

		public static void LogError(Uri baseUri, string org, string configFieldName, Guid userId, string method, string message, string callingMethod)
		{
			try
			{
				VRM.VEFT.Workflow.Helpers.CreateCRMLogEntryRequest createCRMLogEntryRequest = new VRM.VEFT.Workflow.Helpers.CreateCRMLogEntryRequest()
				{
					MessageId = Guid.NewGuid().ToString(),
					OrganizationName = org,
					UserId = userId,
					crme_Name = string.Format("Exception: {0}:{1}", "Error in ", method),
					crme_ErrorMessage = message,
					crme_Debug = false,
					crme_GranularTiming = false,
					crme_TransactionTiming = false,
					crme_Method = string.Concat(callingMethod, ":", method),
					crme_LogLevel = 935950003,
					crme_Sequence = 1,
					NameofDebugSettingsField = configFieldName
				};
				Utility.SendReceive<CreateCRMLogEntryResponse>(baseUri, "CRMe#CreateCRMLogEntryRequest", createCRMLogEntryRequest, null);
			}
			catch (Exception exception)
			{
			}
		}

		public static void LogError(Uri baseUri, string org, string configFieldName, Guid userId, string method, Exception ex, string callingMethod)
		{
			string str = Utility.StackTraceToString(ex);
			Utility.LogError(baseUri, org, configFieldName, userId, method, str, callingMethod);
		}

		public static void LogError(Uri baseUri, LogSettings logSettings, string method, Exception ex)
		{
			Utility.LogError(baseUri, logSettings.Org, logSettings.ConfigFieldName, logSettings.UserId, method, ex, logSettings.callingMethod);
		}

		private static MemoryStream ObjectToJSonStream(object obj)
		{
			MemoryStream memoryStream = new MemoryStream();
			(new DataContractJsonSerializer(obj.GetType())).WriteObject(memoryStream, obj);
			memoryStream.Position = (long)0;
			return memoryStream;
		}

		public static HttpResponseMessage Send(Uri baseUri, string messageId, object requestObj, LogSettings logSettings)
		{
			HttpResponseMessage result = null;
			Uri uri = Utility.FormatUri(baseUri, "Send", messageId);
			try
			{
				HttpClient httpClient = new HttpClient();
				try
				{
					string empty = string.Empty;
					MemoryStream jSonStream = Utility.ObjectToJSonStream(requestObj);
					try
					{
						StreamContent streamContent = new StreamContent(jSonStream);
						try
						{
							result = httpClient.PostAsync(uri, streamContent).Result;
							result.EnsureSuccessStatusCode();
						}
						finally
						{
							if (streamContent != null)
							{
								((IDisposable)streamContent).Dispose();
							}
						}
					}
					finally
					{
						if (jSonStream != null)
						{
							((IDisposable)jSonStream).Dispose();
						}
					}
				}
				finally
				{
					if (httpClient != null)
					{
						((IDisposable)httpClient).Dispose();
					}
				}
			}
			catch (Exception exception1)
			{
				Exception exception = exception1;
				if (logSettings != null)
				{
					Utility.LogError(baseUri, logSettings, "Send", exception);
				}
				throw;
			}
			return result;
		}

		public static void SendAsync(Uri baseUri, string messageId, object obj, LogSettings logSettings, Action<HttpResponseMessage> callBack)
		{
			(new Thread(() => {
				try
				{
					Thread.CurrentThread.IsBackground = false;
					HttpResponseMessage httpResponseMessage = Utility.Send(baseUri, messageId, obj, logSettings);
					if (callBack != null)
					{
						callBack(httpResponseMessage);
					}
				}
				catch (Exception exception1)
				{
					Exception exception = exception1;
					if (logSettings != null)
					{
						Utility.LogError(baseUri, logSettings, "SendAsync", exception);
					}
				}
			})).Start();
		}

		public static T SendReceive<T>(Uri baseUri, string messageId, object obj, LogSettings logSettings)
		{
			string result;
			string empty = string.Empty;
			Uri uri = Utility.FormatUri(baseUri, "SendReceive", messageId);
			try
			{
				HttpClient httpClient = new HttpClient();
				try
				{
					MemoryStream jSonStream = Utility.ObjectToJSonStream(obj);
					try
					{
						StreamContent streamContent = new StreamContent(jSonStream);
						try
						{
							HttpResponseMessage httpResponseMessage = httpClient.PostAsync(uri, streamContent).Result;
							httpResponseMessage.EnsureSuccessStatusCode();
							result = httpResponseMessage.Content.ReadAsStringAsync().Result;
						}
						finally
						{
							if (streamContent != null)
							{
								((IDisposable)streamContent).Dispose();
							}
						}
						int num = result.IndexOf("<Message>") + 9;
						int num1 = result.IndexOf("</Message>");
						if (num1 != -1)
						{
							empty = result.Substring(num, num1 - num);
						}
					}
					finally
					{
						if (jSonStream != null)
						{
							((IDisposable)jSonStream).Dispose();
						}
					}
				}
				finally
				{
					if (httpClient != null)
					{
						((IDisposable)httpClient).Dispose();
					}
				}
			}
			catch (Exception exception1)
			{
				Exception exception = exception1;
				if (logSettings != null)
				{
					Utility.LogError(baseUri, logSettings, "SendReceive", exception);
				}
				throw;
			}
			return Utility.DeserializeResponse<T>(empty);
		}

		public static string StackTraceToString(Exception ex)
		{
			return Utility.StackTraceToString(ex, new StringBuilder()).ToString();
		}

		internal static StringBuilder StackTraceToString(Exception ex, StringBuilder sb)
		{
			sb.AppendLine("***************************");
			sb.AppendLine(ex.Message);
			sb.AppendLine(ex.StackTrace);
			if (ex.InnerException != null)
			{
				sb = Utility.StackTraceToString(ex.InnerException, sb);
			}
			return sb;
		}

		public enum LogLevel
		{
			Debug = 935950000,
			Info = 935950001,
			Warn = 935950002,
			Error = 935950003,
			Fatal = 935950004,
			Timing = 935950005
		}
	}
}