using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using Microsoft.Xrm.Sdk.Workflow;
using System;
using System.Activities;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.CompilerServices;
using System.Web.Script.Serialization;
using VRM.VEFT.Workflow.Helpers;
using VRM.VEFT.Workflow.Messages;

namespace VRM.VEFT.Workflow
{
	public class LinkInteractionToVeteranFromMVIUsingICN : CodeActivity
	{
		private Logger _logger;

		private Entity _me;

		private LogSettings _logSettings;

		[Input("Cached ICN")]
		[ReferenceTarget("patsr_cachedicn")]
		[RequiredArgument]
		public InArgument<EntityReference> inputCachedICN
		{
			get;
			set;
		}
        
		public LinkInteractionToVeteranFromMVIUsingICN()
		{
		}
                
		private void AuditPersonSearchResponse(PatientPerson person, CorrespondingIdsResponse selectedPersonResponse, string pICN, bool searchWasAttended, string personSearchAuditURL)
		{
			LinkInteractionToVeteranFromMVIUsingICN.MVILog mVILog = new LinkInteractionToVeteranFromMVIUsingICN.MVILog()
			{
				Address = person.FullAddress,
				Attended = searchWasAttended,
				ClassCode = "",
				CreatedOn = DateTime.UtcNow,
				DOB = person.BirthDate,
				EDIPI = person.EdiPi,
				First = selectedPersonResponse.FirstName,
				ICN = pICN,
				Last = selectedPersonResponse.FamilyName,
				Middle = selectedPersonResponse.MiddleName,
				MMN = "",
				Organization_Name = selectedPersonResponse.OrganizationName,
				Phone = selectedPersonResponse.PhoneNumber,
				POBC = "",
				POBS = "",
				RequestType = ""
			};
			Guid guid = new Guid();
			//guid = guid;
			mVILog.SessionId = guid.ToString();
			mVILog.Sex = "";
			mVILog.Source = "MVI";
			mVILog.Type = "Response";
			mVILog.UserName = (string)this._me.GetAttributeValue<string>("fullname");
			mVILog.SSN = person.SocialSecurityNumber;
			this.POSTAuditRequest(personSearchAuditURL, (new JavaScriptSerializer()).Serialize(mVILog), "MVILog");
		}

		private void AuditSelectedPersonRequest(SelectedPersonRequest request, string selectedPersonAuditURL)
		{
			LinkInteractionToVeteranFromMVIUsingICN.MVILog2 mVILog2 = new LinkInteractionToVeteranFromMVIUsingICN.MVILog2()
			{
				Organization_Name = request.OrganizationName,
				CreatedOn = DateTime.UtcNow,
				DOB = request.DateofBirth,
				EDIPI = request.Edipi,
				ICN = request.ICN,
				RequestType = "SelectedPersonRequest"
			};
			Guid guid = new Guid();
			guid = guid;
			mVILog2.SessionId = guid.ToString();
			mVILog2.SSN = request.SocialSecurityNumber;
			mVILog2.UserName = string.Concat((string)this._me.GetAttributeValue<string>("firstname"), " ", (string)this._me.GetAttributeValue<string>("lastname"));
			this.POSTAuditRequest(selectedPersonAuditURL, (new JavaScriptSerializer()).Serialize(mVILog2), "MVILog2");
		}
        
		protected override void Execute(CodeActivityContext executionContext)
		{            
			Exception exception;
			Entity entity;
			string[] strArrays;
			object[] str;
			bool flag;
			ITracingService extension = executionContext.GetExtension<ITracingService>();
			IWorkflowContext workflowContext = executionContext.GetExtension<IWorkflowContext>();
			IOrganizationService organizationService = executionContext.GetExtension<IOrganizationServiceFactory>().CreateOrganizationService(new Guid?(workflowContext.UserId));
			Entity entity1 = new Entity("mcs_log");
			entity1["createdon"] = DateTime.Now;
			entity1["mcs_method"] = "MVI (from workflow)";
			//entity1["patsr_interactionid"] = new EntityReference(workflowContext.PrimaryEntityName, workflowContext.PrimaryEntityId);
			entity1["mcs_name"] = "LinkInteractionToVeteranFromMVIUsingICN Result";

			try
			{
				PatientPerson sensitivityInfoFromESR = null;
				string name = this.inputCachedICN.Get<EntityReference>(executionContext).Name;
				if (string.IsNullOrEmpty(name))
				{
					throw new InvalidPluginExecutionException("ICN is empty.");
				}
				Entity item = new Entity();
				try
				{
					QueryByAttribute queryByAttribute = new QueryByAttribute("patsr_setting");
					queryByAttribute.ColumnSet = new ColumnSet(true);
					queryByAttribute.AddAttributeValue("patsr_name", "Active Settings");
					item = organizationService.RetrieveMultiple(queryByAttribute).Entities[0];
				}
				catch (Exception exception1)
				{
					exception = exception1;
					extension.Trace("error getting Active Settings.", new object[0]);
					throw exception;
				}
				string str1 = (item.Contains("patsr_unexpectedmessage") ? (string)item.GetAttributeValue<string>("patsr_unexpectedmessage") : workflowContext.OrganizationName);
				bool flag1 = (!item.Contains("patsr_retrievesenitivitylevel") ? false : (bool)item.GetAttributeValue<bool>("patsr_retrievesenitivitylevel"));
				bool flag2 = (!item.Contains("patsr_enableauditing") ? false : (bool)item.GetAttributeValue<bool>("patsr_enableauditing"));
				string str2 = (item.Contains("patsr_restendpointforvimt") ? (string)item.GetAttributeValue<string>("patsr_restendpointforvimt") : string.Empty);
				bool flag3 = (!item.Contains("patsr_searchlogtiming") ? false : (bool)item.GetAttributeValue<bool>("patsr_searchlogtiming"));
				bool flag4 = (!item.Contains("patsr_searchlogsoap") ? false : (bool)item.GetAttributeValue<bool>("patsr_searchlogsoap"));
				entity1["mcs_errormessage"] = str2;

				EntityCollection entityCollection = new EntityCollection();
                //try
                //{
                //	QueryExpression queryExpression = new QueryExpression("bah_keyvaluepair");
                //	strArrays = new string[] { "bah_name_text", "bah_stringvalue_text" };
                //	queryExpression.ColumnSet = new ColumnSet(strArrays);
                //	queryExpression.Criteria.FilterOperator = LogicalOperator.And;
                //	queryExpression.Criteria.Conditions.Add(new ConditionExpression("bah_name_text", 0, "MVI_Log"));
                //	queryExpression.Criteria.Conditions.Add(new ConditionExpression("bah_name_text", 0, "MVI_Log2"));
                //	queryExpression.Criteria.Conditions.Add(new ConditionExpression("bah_name_text", 0, "MVI_Sensitivity_Log"));
                //	queryExpression.Criteria.Conditions.Add(new ConditionExpression("bah_name_text", 0, "sensitive_endpoint"));
                //	entityCollection = organizationService.RetrieveMultiple(queryExpression);
                //}
                //catch (Exception exception2)
                //{
                //	exception = exception2;
                //	extension.Trace("error getting key-value pairs.", new object[0]);
                //	throw exception;
                //}
                string item1 = (item.Contains("patsr_sensitive_endpoint") ? (string)item.GetAttributeValue<string>("patsr_sensitive_endpoint") : string.Empty);
                string item2 = (item.Contains("patsr_mvi_sensitivity_log") ? (string)item.GetAttributeValue<string>("patsr_mvi_sensitivity_log") : string.Empty);
                string item3 = (item.Contains("patsr_mvi_log") ? (string)item.GetAttributeValue<string>("patsr_mvi_log") : string.Empty);
                string str3 = (item.Contains("patsr_mvi_log2") ? (string)item.GetAttributeValue<string>("patsr_mvi_log2") : string.Empty);

    //            string item1 = (string)entityCollection.Entities.FirstOrDefault<Entity>((Entity r) => (string)r.GetAttributeValue<string>("bah_name_text") == "sensitive_endpoint").GetAttributeValue<string>("bah_stringvalue_text");
				//string item2 = (string)entityCollection.Entities.FirstOrDefault<Entity>((Entity r) => (string)r.GetAttributeValue<string>("bah_name_text") == "MVI_Sensitivity_Log").GetAttributeValue<string>("bah_stringvalue_text");
				//string item3 = (string)entityCollection.Entities.FirstOrDefault<Entity>((Entity r) => (string)r.GetAttributeValue<string>("bah_name_text") == "MVI_Log").GetAttributeValue<string>("bah_stringvalue_text");
				//string str3 = (string)entityCollection.Entities.FirstOrDefault<Entity>((Entity r) => (string)r.GetAttributeValue<string>("bah_name_text") == "MVI_Log2").GetAttributeValue<string>("bah_stringvalue_text");
				LogSettings logSetting = new LogSettings()
				{
					Org = str1,
					ConfigFieldName = "RESTCALL",
					UserId = workflowContext.InitiatingUserId,
					callingMethod = "VRM.VEFT.Workflow.LinkInteractionToVeteranFromMVIUsingICN"
				};
				this._logSettings = logSetting;
				Logger logger = new Logger()
				{
					setService = organizationService,
					setTracingService = extension,
					setModule = string.Format("{0}:", base.GetType()),
					setDebug = true
				};
				this._logger = logger;
				Guid initiatingUserId = workflowContext.InitiatingUserId;
				strArrays = new string[] { "firstname", "lastname", "fullname" };
				this._me = organizationService.Retrieve("systemuser", initiatingUserId, new ColumnSet(strArrays));
				SelectedPersonRequest selectedPersonRequest = new SelectedPersonRequest()
				{
					ICN = name,
					Debug = false,
					FetchMessageProcessType = "remote",
					LogSoap = flag4,
					LogTiming = flag3
				};
				Guid guid = new Guid();
				//guid = guid;
				selectedPersonRequest.MessageId = guid.ToString();
				selectedPersonRequest.noAddPerson = false;
				selectedPersonRequest.OrganizationName = str1;
				selectedPersonRequest.UserId = workflowContext.InitiatingUserId;
				selectedPersonRequest.UserFirstName = (string)this._me.GetAttributeValue<string>("firstname");
				selectedPersonRequest.UserLastName = (string)this._me.GetAttributeValue<string>("lastname");
				SelectedPersonRequest selectedPersonRequest1 = selectedPersonRequest;
				if (flag2)
				{
					this.AuditSelectedPersonRequest(selectedPersonRequest1, str3);
				}
				CorrespondingIdsResponse correspondingIdsResponse = Utility.SendReceive<CorrespondingIdsResponse>(new Uri(str2), "FTPSelectedPersonRequest", selectedPersonRequest1, this._logSettings);
				Logger logger1 = this._logger;
				bool exceptionOccured = correspondingIdsResponse.ExceptionOccured;
				logger1.WriteDebugMessage(string.Concat("selectedPersonSearchResponse.ExceptionOccured: ", exceptionOccured.ToString()));
				if (correspondingIdsResponse.ExceptionOccured)
				{
					this._logger.WriteDebugMessage(string.Format("ERROR::SelectedPersonSearch: Search Exception Message: {0}", (!string.IsNullOrEmpty(correspondingIdsResponse.RawMviExceptionMessage) ? correspondingIdsResponse.RawMviExceptionMessage : "No exception message available.")));
					throw new InvalidPluginExecutionException(correspondingIdsResponse.RawMviExceptionMessage);
				}
				bool flag5 = (correspondingIdsResponse.CorrespondingIdList == null ? false : (int)correspondingIdsResponse.CorrespondingIdList.Length > 0);
				this._logger.WriteDebugMessage(string.Format("DEBUG::SelectedPersonSearch: Search Response Message: Corresponding IDs Found: {0}", flag5));
				if (!flag5)
				{
					throw new InvalidPluginExecutionException("Did not find any corresponding IDs from SelectedPersonSearch.");
				}
				string patientIdentifier = "";
				try
				{
					patientIdentifier = ((IEnumerable<CorrespondingIDs>)correspondingIdsResponse.CorrespondingIdList).First<CorrespondingIDs>((CorrespondingIDs x) => x.AssigningFacility == "200DOD").PatientIdentifier;
				}
				catch (Exception exception3)
				{
					exception = exception3;
					extension.Trace("Error finding 200DOD corresponding ID for EDIPI.", new object[0]);
				}
				if (string.IsNullOrEmpty(patientIdentifier) || !(patientIdentifier != "UNK"))
				{
					flag = (string.IsNullOrEmpty(correspondingIdsResponse.FamilyName) || string.IsNullOrEmpty(correspondingIdsResponse.SocialSecurityNumber) ? true : string.IsNullOrEmpty(correspondingIdsResponse.DateofBirth));
				}
				else
				{
					flag = false;
				}
				if (!flag)
				{
					PersonSearchRequest personSearchRequest = new PersonSearchRequest();
					guid = new Guid();
					guid = guid;
					personSearchRequest.MessageId = guid.ToString();
					personSearchRequest.OrganizationName = str1;
					personSearchRequest.UserId = workflowContext.InitiatingUserId;
					personSearchRequest.Debug = false;
					personSearchRequest.LogSoap = flag4;
					personSearchRequest.LogTiming = flag3;
					personSearchRequest.UserFirstName = (string)this._me.GetAttributeValue<string>("firstname");
					personSearchRequest.UserLastName = (string)this._me.GetAttributeValue<string>("lastname");
					personSearchRequest.FetchMessageProcessType = "remote";
					personSearchRequest.IsAttended = true;
					personSearchRequest.FirstName = correspondingIdsResponse.FirstName;
					personSearchRequest.FamilyName = correspondingIdsResponse.FamilyName;
					personSearchRequest.SocialSecurityNumber = correspondingIdsResponse.SocialSecurityNumber;
					personSearchRequest.BirthDate = correspondingIdsResponse.DateofBirth;
					PersonSearchRequest personSearchRequest1 = personSearchRequest;
					if ((string.IsNullOrEmpty(patientIdentifier) ? false : patientIdentifier != "UNK"))
					{
						personSearchRequest1.Edipi = patientIdentifier;
						personSearchRequest1.IsAttended = false;
					}
					PersonSearchResponse personSearchResponse = Utility.SendReceive<PersonSearchResponse>(new Uri(str2), "FTPPersonSearchRequest", personSearchRequest1, this._logSettings);
					if (personSearchResponse.ExceptionOccured)
					{
						this._logger.WriteDebugMessage(string.Format("ERROR::PersonSearch: Search Exception Message: {0} MVI Message: {1}", personSearchResponse.MVIMessage, personSearchResponse.RawMviExceptionMessage));
						throw new InvalidPluginExecutionException(correspondingIdsResponse.RawMviExceptionMessage);
					}
					bool flag6 = (personSearchResponse.Person == null ? false : (int)personSearchResponse.Person.Length > 0);
					this._logger.WriteDebugMessage(string.Format("DEBUG::PersonSearch: Search Response Message: {0} Person Found: {1}", personSearchResponse.MVIMessage, flag6));
					try
					{
						PatientPerson[] person = personSearchResponse.Person;
						for (int i = 0; i < (int)person.Length; i++)
						{
							PatientPerson patientPerson = person[i];
							if ((sensitivityInfoFromESR != null || patientPerson.CorrespondingIdList == null ? false : (int)patientPerson.CorrespondingIdList.Length > 0))
							{
								if (((IEnumerable<CorrespondingIDs>)patientPerson.CorrespondingIdList).FirstOrDefault<CorrespondingIDs>((CorrespondingIDs id) => (id.AssigningFacility != "200M" ? false : id.PatientIdentifier == name)) != null)
								{
									sensitivityInfoFromESR = patientPerson;
									extension.Trace("Found personFromMVI matching inputICN in personSearchResponse", new object[0]);
									break;
								}
							}
						}
					}
					catch (Exception exception4)
					{
						exception = exception4;
						extension.Trace("Error finding person from personSearchResponse with a CorrespondingID matching the input ICN.", new object[0]);
						throw exception;
					}
				}
				if (sensitivityInfoFromESR == null)
				{
					throw new InvalidPluginExecutionException("Did not find person in MVI.");
				}
				if (!flag1)
				{
					sensitivityInfoFromESR.VeteranSensitivityLevel = "false:false";
				}
				else
				{
					sensitivityInfoFromESR.VeteranSensitivityLevel = this.GetSensitivityInfoFromESR(item1, name);
				}
				if (flag2)
				{
					this.AuditPersonSearchResponse(sensitivityInfoFromESR, correspondingIdsResponse, name, (string.IsNullOrEmpty(patientIdentifier) ? true : !(patientIdentifier != "UNK")), item3);
				}
				Name name1 = ((IEnumerable<Name>)sensitivityInfoFromESR.NameList).FirstOrDefault<Name>((Name n) => n.NameType.Equals("Legal", StringComparison.OrdinalIgnoreCase));
				if (name1 == null)
				{
					name1 = sensitivityInfoFromESR.NameList.FirstOrDefault<Name>();
				}
				string str4 = (name1 != null ? name1.GivenName : correspondingIdsResponse.FirstName);
				string str5 = (name1 != null ? name1.MiddleName : correspondingIdsResponse.MiddleName);
				string str6 = (name1 != null ? name1.FamilyName : correspondingIdsResponse.FamilyName);
				QueryExpression queryExpression1 = new QueryExpression();
				queryExpression1.EntityName = "contact";
                //strArrays = new string[] { "firstname", "middlename", "lastname", "fullname", "governmentid", "birthdate", "patsr_manuallycreatedfrommvicode", "veo_deceaseddate" };
                strArrays = new string[] { "firstname", "middlename", "lastname", "fullname", "governmentid", "birthdate", "veo_deceaseddate" };
                queryExpression1.ColumnSet = new ColumnSet(strArrays);
				queryExpression1.Orders.Add(new OrderExpression("modifiedon", OrderType.Ascending));
				queryExpression1.TopCount = new int?(1);
				queryExpression1.Criteria.Conditions.Add(new ConditionExpression("lastname", 0, str6));
				queryExpression1.Criteria.Conditions.Add(new ConditionExpression("governmentid", 0, sensitivityInfoFromESR.SocialSecurityNumber));
				queryExpression1.Criteria.Conditions.Add(new ConditionExpression("birthdate", 0, sensitivityInfoFromESR.BirthDate));

                ////////////////////
                //TODO:
                //where are these?
                //queryExpression1.Criteria.Conditions.Add(new ConditionExpression("patsr_manuallycreatedfrommvicode", 0, (object)100000000));
                //queryExpression1.Criteria.Conditions.Add(new ConditionExpression("patsr_searchtype_text", 0, "CREATEUPDATEMVI"));
                ////////////////////

                queryExpression1.Criteria.Conditions.Add(new ConditionExpression("statecode", 0, "Active"));
				QueryExpression queryExpression2 = queryExpression1;
				if (!string.IsNullOrEmpty(str4))
				{
					queryExpression2.Criteria.AddCondition(new ConditionExpression("firstname", 0, str4));
				}
				if (!string.IsNullOrEmpty(str5))
				{
					queryExpression2.Criteria.AddCondition(new ConditionExpression("middlename", 0, str5));
				}
				Entity entity2 = new Entity();
				try
				{
					EntityCollection entityCollection1 = organizationService.RetrieveMultiple(queryExpression2);
					if (entityCollection1.Entities.Count != 1)
					{
						throw new InvalidPluginExecutionException("Did not find/create contact record in CRM.");
					}
					entity2 = entityCollection1.Entities.First<Entity>();
				}
				catch (Exception exception5)
				{
					exception = exception5;
					extension.Trace("Error retrieving contact.", new object[0]);
					throw exception;
				}
				Entity entity3 = new Entity("contact");
				entity3.Id = entity2.Id;
				entity3["veo_sensitiveveteran"] = (sensitivityInfoFromESR.VeteranSensitivityLevel == "true:true" ? true : sensitivityInfoFromESR.VeteranSensitivityLevel == "true:false");
				entity3["veo_deceaseddate"] = sensitivityInfoFromESR.DeceasedDate;
				try
				{
					organizationService.Update(entity3);
					extension.Trace("updated contact with Sensitivity Level and DeceasedDate.", new object[0]);
				}
				catch (Exception exception6)
				{
					exception = exception6;
					extension.Trace("Error updating veteran sensitivity and deceased date values.", new object[0]);
					throw exception;
				}
				entity = new Entity("incident");
				entity.Id = workflowContext.PrimaryEntityId;
				entity["patsr_veteran"] = entity2.ToEntityReference();
				try
				{
					extension.Trace("updating interaction...", new object[0]);
					organizationService.Update(entity);
					extension.Trace("updated interaction.", new object[0]);
					extension.Trace("logging to log table.", new object[0]);
					this.LogToIntegrationResultTable(organizationService, entity1, "OK", "Found veteran in MVI and linked to interaction.");
				}
				catch (Exception exception7)
				{
					exception = exception7;
					extension.Trace("Error updating interaction w/ veteran.", new object[0]);
					throw exception;
				}
			}
			catch (Exception exception9)
			{
				exception = exception9;
				extension.Trace("Caught an error somewhere.", new object[0]);
				str = new object[] { exception.ToString() };
				extension.Trace("Exception: {0}", str);
				extension.Trace("logging to log table.", new object[0]);
				this.LogToIntegrationResultTable(organizationService, entity1, "ERROR", exception.Message);
				extension.Trace("Proceeding to link interaction to Anonymous Veteran.", new object[0]);
				QueryByAttribute queryByAttribute1 = new QueryByAttribute("contact");
				queryByAttribute1.AddAttributeValue("firstname", "Anonymous");
				queryByAttribute1.AddAttributeValue("lastname", "Veteran");
				EntityCollection entityCollection2 = organizationService.RetrieveMultiple(queryByAttribute1);
				if (entityCollection2.Entities.Count != 1)
				{
					throw new InvalidPluginExecutionException("After some other error, could not find Anonymous Vetearn record to when trying to link interaction to Anonymous Veteran.");
				}
				entity = new Entity("incident");
				entity.Id = workflowContext.PrimaryEntityId;
				entity["patsr_veteran"] = entityCollection2.Entities.First<Entity>().ToEntityReference();
				try
				{
					organizationService.Update(entity);
					extension.Trace("Linked interaction to Anonymous Veteran after some kind of error in workflow.", new object[0]);
				}
				catch (Exception exception8)
				{
					throw new InvalidPluginExecutionException(string.Concat("After some other error, encountered an additional error linking interaction to Anonymous Veteran: ", exception8.Message));
				}
				throw new InvalidPluginExecutionException("Caught error.", exception);
			}
			str = new object[] { workflowContext.CorrelationId };
			extension.Trace("Exiting LinkInteractionToVeteranFromMVIUsingICN.Execute(), Correlation Id: {0}", str);
		}
        
		private string GetSensitivityInfoFromESR(string url, string icn)
		{
			string str;
			if ((url == "" ? false : !(icn == "")))
			{
				url = url.Replace("xml", "json");
				if (url.Contains("{0}"))
				{
					url = url.Replace("{0}", "");
				}
				WebClient webClient = new WebClient();
				try
				{
					string str1 = webClient.DownloadString(string.Concat(url, icn));
					LinkInteractionToVeteranFromMVIUsingICN.ESRSensitiveObject eSRSensitiveObject = (new JavaScriptSerializer()).Deserialize<LinkInteractionToVeteranFromMVIUsingICN.ESRSensitiveObject>(str1);
					str = ((eSRSensitiveObject.Data.EnrollmentDeterminationInfo == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility == null || eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type == null ? true : !(eSRSensitiveObject.Data.EnrollmentDeterminationInfo.PrimaryEligibility.Type.ToUpper() == "EMPLOYEE")) ? string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":false") : string.Concat(eSRSensitiveObject.Data.SensitivityInfo.SensityFlag.ToLower(), ":true"));
				}
				finally
				{
					if (webClient != null)
					{
						((IDisposable)webClient).Dispose();
					}
				}
			}
			else
			{
				str = "false:false";
			}
			return str;
		}

		private void LogToIntegrationResultTable(IOrganizationService pService, Entity record, string integrationStatus, string errorMessage)
		{
			object obj;
			record["patsr_integrationstatus"] = integrationStatus;
			Entity entity = record;
			if (!string.IsNullOrEmpty(errorMessage))
			{
				obj = (errorMessage.Length > 2000 ? errorMessage.Substring(0, 2000) : errorMessage);
			}
			else
			{
				obj = "";
			}
			entity["patsr_errormessage"] = obj;
			try
			{
				record.Id = pService.Create(record);
			}
			catch (Exception exception)
			{
			}
		}

		private void POSTAuditRequest(string url, string json, string logTable)
		{
			HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(url);
			httpWebRequest.ContentType = "application/json";
			httpWebRequest.Method = "POST";
			StreamWriter streamWriter = new StreamWriter(httpWebRequest.GetRequestStream());
			try
			{
				streamWriter.Write(json);
				streamWriter.Flush();
				streamWriter.Close();
			}
			finally
			{
				if (streamWriter != null)
				{
					((IDisposable)streamWriter).Dispose();
				}
			}
			try
			{
				HttpWebResponse response = (HttpWebResponse)httpWebRequest.GetResponse();
				StreamReader streamReader = new StreamReader(response.GetResponseStream());
				try
				{
					streamReader.ReadToEnd();
					streamReader.Close();
				}
				finally
				{
					if (streamReader != null)
					{
						((IDisposable)streamReader).Dispose();
					}
				}
				response.Close();
			}
			catch (Exception exception1)
			{
				Exception exception = exception1;
				this._logger.WriteDebugMessage(string.Concat("ERROR::Error creating ", logTable, " response record: ", exception.Message));
			}
		}

		internal class Data
		{
			public LinkInteractionToVeteranFromMVIUsingICN.Demographics Demographics
			{
				get;
				set;
			}

			public LinkInteractionToVeteranFromMVIUsingICN.EnrollmentDeterminationInfo EnrollmentDeterminationInfo
			{
				get;
				set;
			}

			public LinkInteractionToVeteranFromMVIUsingICN.SensitivityInfo SensitivityInfo
			{
				get;
				set;
			}

			public Data()
			{
			}
		}

		internal class Demographics
		{
			public string PreferredFacility
			{
				get;
				set;
			}

			public Demographics()
			{
			}
		}

		internal class EnrollmentDeterminationInfo
		{
			public LinkInteractionToVeteranFromMVIUsingICN.PrimaryEligibility PrimaryEligibility
			{
				get;
				set;
			}

			public string Veteran
			{
				get;
				set;
			}

			public EnrollmentDeterminationInfo()
			{
			}
		}

		internal class ESRSensitiveObject
		{
			public LinkInteractionToVeteranFromMVIUsingICN.Data Data
			{
				get;
				set;
			}

			public object DebugInfo
			{
				get;
				set;
			}

			public object ErrorMessage
			{
				get;
				set;
			}

			public bool ErrorOccurred
			{
				get;
				set;
			}

			public object Status
			{
				get;
				set;
			}

			public ESRSensitiveObject()
			{
			}
		}

		internal class MVILog
		{
			public string Organization_Name;

			public string First;

			public string Last;

			public string Middle;

			public string DOB;

			public string SSN;

			public string EDIPI;

			public string Phone;

			public bool Attended;

			public string Address;

			public string Sex;

			public string MMN;

			public string POBC;

			public string POBS;

			public string RequestType;

			public string ICN;

			public string Source;

			public string ClassCode;

			public string Type;

			public DateTime CreatedOn;

			public string SessionId;

			public string UserName;

			public MVILog()
			{
			}
		}

		internal class MVILog2
		{
			public string Organization_Name;

			public string DOB;

			public string SSN;

			public string EDIPI;

			public string Phone;

			public string ICN;

			public string IDs;

			public DateTime CreatedOn;

			public string SessionId;

			public string UserName;

			public string RequestType;

			public MVILog2()
			{
			}
		}

		internal class PrimaryEligibility
		{
			public string Type
			{
				get;
				set;
			}

			public PrimaryEligibility()
			{
			}
		}

		internal class SensitivityInfo
		{
			public string SensityFlag
			{
				get;
				set;
			}

			public SensitivityInfo()
			{
			}
		}
	}
}