using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Query;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;

namespace VRM.VEFT.Workflow.Models
{
	public class Facility
	{
		public readonly static string CrmEntityName;

		public readonly static string CrmPrimaryIdentiferFieldName;

		public readonly static string CrmPrimaryAttributeName;

		public readonly static string FullStationNameFieldName;

		public readonly static string CommonStationNameFieldName;

		public readonly static string StationNumberFieldName;

		public readonly static string VISNLookupFieldName;

		public readonly static string CoCClasificationFieldName;

		public readonly static string StationIDFieldName;

		public readonly static string LastUpdatedFieldName;

		public static ColumnSet CrmColumns;

		public string CoCClassification { get; set; }
		public string CommonStationName { get; set; }
		public string CRMDisplayname { get; set; }
		public string StationID { get; set; }
		public string StationName { get; set; }
		public string StationNumber { get; set; }
		public string VisnID { get; set; }
		static Facility()
		{
			Facility.CrmEntityName = "equipment";
			Facility.CrmPrimaryIdentiferFieldName = "equipmentid";
			Facility.CrmPrimaryAttributeName = "name";
			Facility.FullStationNameFieldName = "name";
			Facility.CommonStationNameFieldName = "veo_alsoknownas";
			Facility.StationNumberFieldName = "veo_stationnumber";
			Facility.VISNLookupFieldName = "veo_visnid";
			Facility.CoCClasificationFieldName = "patsr_cocclassification_text";
			Facility.StationIDFieldName = "patsr_vaststationid_text";
			Facility.LastUpdatedFieldName = "patsr_lastupdatedfromvaston";
			string[] crmPrimaryAttributeName = new string[] { Facility.CrmPrimaryAttributeName, Facility.FullStationNameFieldName, Facility.CommonStationNameFieldName, Facility.StationNumberFieldName, Facility.VISNLookupFieldName, Facility.CoCClasificationFieldName, Facility.StationIDFieldName };
			Facility.CrmColumns = new ColumnSet(crmPrimaryAttributeName);
		}

		public Facility(VASTStation stationFromVAST)
		{
			this.StationID = stationFromVAST.StationID.Trim();
			this.VisnID = stationFromVAST.VisnID.Trim();
			this.StationNumber = stationFromVAST.StationNumber.Trim();
			this.StationName = stationFromVAST.StationName.Trim();
			this.CommonStationName = stationFromVAST.CommonStationName.Trim();
			this.CoCClassification = stationFromVAST.CocClassification.Trim();
			this.CRMDisplayname = string.Concat(stationFromVAST.StationNumber.Trim(), " - ", stationFromVAST.CommonStationName.Trim());
		}

        public Entity MapToCRMRecord(Guid recordId, Dictionary<string, Entity> visnDictionary)
        {
            Entity entity = new Entity("equipment");
            entity.Id = recordId;
            entity[Facility.CrmPrimaryAttributeName] =  this.CRMDisplayname;
            entity[Facility.StationIDFieldName] = this.StationID;
            entity[Facility.StationNumberFieldName] = this.StationNumber;
            entity[Facility.CommonStationNameFieldName] = this.CommonStationName;
            entity[Facility.FullStationNameFieldName] = this.StationName;
            entity[Facility.CoCClasificationFieldName] = this.CoCClassification;
            entity[Facility.LastUpdatedFieldName] = DateTime.Now;
            KeyValuePair<string, Entity> keyValuePair = visnDictionary.FirstOrDefault<KeyValuePair<string, Entity>>((KeyValuePair<string, Entity> entry) => entry.Key == this.VisnID);
            Entity value = keyValuePair.Value;
            if (value != null)
            {
                entity[Facility.VISNLookupFieldName] = value.ToEntityReference();
            }
            return entity;
        }

        public bool MatchesCRMRecord(Entity record)
        {
            return (!record.Contains(Facility.StationIDFieldName) || 
                !((string)record[Facility.StationIDFieldName] == this.StationID) || 
                !record.Contains(Facility.StationNumberFieldName) || 
                !((string)record[Facility.StationNumberFieldName] == this.StationNumber) || 
                !record.Contains(Facility.CommonStationNameFieldName) || 
                !((string)record[Facility.CommonStationNameFieldName] == this.CommonStationName) || 
                !record.Contains(Facility.CrmPrimaryAttributeName) || 
                !((string)record[Facility.CrmPrimaryAttributeName] == this.CRMDisplayname) || 
                !record.Contains(Facility.FullStationNameFieldName) || 
                !((string)record[Facility.FullStationNameFieldName] == this.StationName) || 
                !record.Contains(Facility.VISNLookupFieldName) || 
                !(((EntityReference)record[Facility.VISNLookupFieldName]).Name == this.VisnID) || 
                !record.Contains(Facility.CoCClasificationFieldName) ? false : (string)record[Facility.CoCClasificationFieldName] == this.CoCClassification);
        }
    }
}